align 16
prepare_floating_tables:
	push	rax rcx rdx rbx rbp rsi rdi r8 r9

; we have 32-bits for exponent and 128-bits for fraction
	mov	ebx,1			; initial 1.0 = 1h in low 64-bits of fraction
	xor	edi,edi			; upper 64-bits of fraction
	xor	esi,esi			; exponent=0

; prepare table for 2^x where 0 =< x
	lea	r8,[byg]
	lea	r9,[expbyg]
	mov	[r8],rbx
	mov	[r8+8],rdi
	mov	[r9],esi
	add	r8,16
	add	r9,4

; we didn't multiply by 2 and we want to divide by 10, so we just divide by 5
	mov	ecx,5
fpt0:	bt	rdi,63			; *2 will overflow if bit 127 is = 1
	jc	fpt05

	shl	rbx,1			; 2* low 64-bits of fraction
	rcl	rdi,1			; 2* high 64-bits of fraction
	jmp	fpt1

fpt05:
; divide fraction by 10 and increase exponent by +1
	inc	esi			; exponent + 1

; upper 64-bits of fraction
	mov	rax,rdi
	xor	edx,edx
	div	rcx
	mov	rdi,rax
; rdx=remainder after above dividing
; lower 64-bits of fraction, note that rdx isn't able to be more than 4 now
; (because rdx is remainder after dividing by 5)
	mov	rax,rbx
	div	rcx
	mov	rbx,rax

	add	dl,100h-2		; check rounding (not exact for 5, but what better
					; shall we do? It sets carry flag for dl = 2, 3, 4
	adc	rbx,0
	adc	rdi,0

fpt1:	mov	[r8],rbx
	mov	[r8+8],rdi
	mov	[r9],esi
	add	r8,16
	add	r9,4
	cmp	si,5000			; 5000 is enough for 80-bit floatings upto 10^+4940
	jc	fpt0

; prepare table for 2^x where x =< 0
	mov	ebx,1			; low 64-bits of fraction
	xor	edi,edi			; high 64-bits of fraction
	xor	esi,esi			; exponent
; we will divide fraction by 2 every step and multiply by 10 (and decrease exponent) when
; there is no threating of overflow
; so it means:
; multiply by 5 of numbers smaller than 33333333333333333333333333333333h and exponent-1
; divide by 2 without decreasing exponent of numbers above 33333333333333333333333333333333h

; note that number 1.0 has been already stored, we start with 0.5 (= 1.0 / 2)
	lea	r8,[byg-16]		; so go 1 number back
	lea	r9,[expbyg-4]		; exponent back too

	mov	rax,3333333333333333h	; 10000000000000000h / 5
; note that it is safe to do 33333333333333333333333333333333h * 5 without overflow risk
; for simplicity, we allow multiplying only for high 64-bits at most 3333333333333332h
; then we needn't to perform any check of low 64-bits

fpt2:	cmp	rdi,rax
	jc	fpt3

; divide by 2 without changing exponent
	shr	rdi,1
	rcr	rbx,1
; rounding - lowest bit came into carry flag
	adc	rbx,0
	adc	rdi,0
	jmp	fpt4

fpt3:
; multiply by 5 with decreasing exponent
	dec	esi			; exponent - 1

	mov	rbp,rbx
	mov	rdx,rdi
	shl	rbx,1			; 2* low 64-bits
	rcl	rdi,1			; 2* high 64-bits (+ overflow if any from previous shl)

	shl	rbx,1			; 2* low 64-bits
	rcl	rdi,1			; 2* high 64-bits (+ overflow if any from previous shl)

; rdi:rbx is now 2*2* rdx:rbp
; we need 5* so we add rdx:rbp
	add	rbx,rbp
	adc	rdi,rdx

fpt4:
; done, store result and go to next turn
	mov	[r8],rbx
	mov	[r8+8],rdi
	mov	[r9],esi
	sub	r8,16
	sub	r9,4
	cmp	si,-5000		; 5000 is enough for 80-bit floatings upto 10^-4940
	jnc	fpt2

	pop	r9 r8 rdi rsi rbp rbx rdx rcx rax
	ret

align 16
convert_float_32:	; 32 bits = single precision
; in:	rax points to the buffer with the floating point number
;	rdi points to the buffer to store ascii result
	push	rax rcx rdx rbx rbp rsi r8 r9
	sub	rsp,8*11

	cld
	lea	rbx,[rax]

; sign
;	btr	dword [rbx],31		; test and reset sign bit of 32-bit floating bit 31
; No ! We mustn't destroy this bit since convert_float_80 may will need it to use...
; We will clear this bit later in register, not in memory not to change memory in ThreadContext
	bt	dword [rbx],31		; test sign bit of 32-bit floating bit 31
	setc	dl
	lea	eax,[rdx+rdx*1+'+']
	stosb

; prepare empty string
	push	rdi
	mov	eax,'    '
	stosd
	stosd
	stosd
	stosd
	mov	byte [rdi],0	; 0-terminator
	pop	rdi

; get exponent (free of sign bit already)
;	mov	edx,dword [rbx]
;	shr	edx,23
;
;; get fraction
;	mov	eax,dword [rbx]
	mov	edx,dword [rbx]
	mov	eax,edx
	btr	edx,31			; clear the sign bit
	shr	edx,23			; and shift the value down to start from bit 0
	and	eax,7FFFFFh		; (2 ^ 23) - 1
	or	edx,edx
	jnz	convert_float_32_L2
	or	eax,eax
	jnz	convert_float_32_L1

; exponent=0 fraction=0
; only 0 instead of 0000000000000000
	mov	al,'0'
	stosb
	lea	rdi,[rdi+15]
	jmp	convert_float_32_epi

convert_float_32_L1:
; exponent=0 fraction<>0
; denormalized real
	call	hexa_64_rax
	lea	rdi,[rdi+16]
	jmp	convert_float_32_epi

convert_float_32_L2:
	cmp	dl,0FFh
	jz	convert_float_32_L8

; 0 < exponent < 7Fh
; normalized real f=fraction e=exponent b=bias=1023 for 64-bit floatings
; 1.f  2(e-b)

; eax=floating fraction
; edx=floating exponent

	sub	dx,7Fh+23		; 7Fh correction for exponent = 0
					; 23 correction for the lowest bit of fraction
	movsx	rsi,dx

	bts	eax,23			; set highest bit of fraction

	lea	rcx,[expbyg]
	mov	r9d,[rcx+rsi*4]		; get decimal exponent

	sal	rsi,1			; *2
	lea	rcx,[byg]

	mov	rbx,rax			; save fraction
	mul	qword [rcx+rsi*8+8]	; multiply with upper 64-bits of 128-bits in table
	mov	r8,rdx			; save the highest qword of multiplication
	mov	rbp,rax			; save the medium qword of multiplication
	mov	rax,rbx
	mul	qword [rcx+rsi*8]	; multiply with lower 64-bits of 128-bits in table
	mov	rbx,rax			; save the lowest qword of multiplication
	add	rbp,rdx			; medium qword
	adc	r8,0			; highest qword
; r8:rbp:rbx holds hexadecimal result
; r9d holds decimal exponent

	lea	rsi,[rsp+8*8]
	mov	rax,'00000000'
	mov	[rsi],rax
	mov	[rsi+8],rax
	mov	[rsi+16],rax

	mov	ecx,10
LN32_7:	dec	rsi
	mov	rax,r8
	xor	edx,edx
	div	rcx
	mov	r8,rax
	mov	rax,rbp
	div	rcx
	mov	rbp,rax
	mov	rax,rbx
	div	rcx
	mov	rbx,rax
	add	dl,'0'
	mov	[rsi],dl
	or	rax,rbp
	or	rax,r8
	jnz	LN32_7
	lea	edx,[rsp+8*8-1]
	sub	edx,esi

; rounding
	lea	rax,[rsi+7]
	call	fp_rounding

	movsb
	mov	al,'.'
	stosb
	movsd
	movsb
	movsb
	movsb				; 8 digits
	mov	eax,'*10^'
	stosd

	add	r9d,edx			; correction for moving decimal dot upper

	lea	rsi,[rsp+8*8]
	mov	eax,'0000'
	mov	[rsi-4],eax
	mov	eax,r9d
	mov	ebp,eax
	neg	ebp
	or	eax,eax
	cmovs	eax,ebp
	sets	dl
	lea	edx,[rdx+rdx*1+'+']
	mov	byte [rdi],dl
	inc	rdi

LN32_8:	dec	rsi
	xor	edx,edx
	div	ecx
	add	dl,'0'
	mov	[rsi],dl
	or	eax,eax			; not necessary
	jnz	LN32_8

	lea	rsi,[rsp+8*8-2]
	movsb
	movsb
	mov	byte [rdi],0

convert_float_32_epi:
	add	rsp,8*11
	pop	r9 r8 rsi rbp rbx rdx rcx rax
	ret

convert_float_32_L8:
	or	eax,eax
	jnz	convert_float_32_L9

; exponent=7Fh fraction=0
	mov	rax,'Infinity'
	stosq
	lea	rdi,[rdi+8]
	jmp	convert_float_32_epi

convert_float_32_L9:
; exponent=FFh fraction<>0
	bt	eax,22		; test most significant bit of fraction
	mov	edx,'QNaN'	; Quiet Not a Number
	mov	eax,'SNaN'	; Signalling Not a Number
	cmovc	eax,edx
	stosd
	lea	rdi,[rdi+12]
	jmp	convert_float_32_epi

align 16
convert_float_64:	; 64 bits = double precision
	push	rax rcx rdx rbx rbp rsi r8 r9
	sub	rsp,8*11

	cld
	lea	rbx,[rax]

; sign
;	btr	dword [rbx+4],31	; test and reset sign bit of 64-bit floating bit 63
; No ! We mustn't destroy this bit since another convert_float_... may will need it to use.
; We will clear this bit later in register, not in memory not to change memory in ThreadContext
	bt	dword [rbx+4],31	; test sign bit of 64-bit floating bit 63
	setc	dl
	lea	eax,[rdx+rdx*1+'+']
	stosb

; prepare empty string
	push	rdi
	mov	eax,'    '
	mov	ecx,6
	repz stosd
	stosb
	mov	byte [rdi],cl	; 0-terminator
	pop	rdi

; get exponent (free of sign bit already)
	mov	edx,dword [rbx+4]
	btr	edx,31			; clear the sign bit
	shr	edx,20			; and shift the value down to start from bit 0

; get fraction
	mov	rax,qword [rbx]
	mov	rcx,000FFFFFFFFFFFFFh	; (2 ^ 52) - 1
	and	rax,rcx
	or	edx,edx
	jnz	convert_float_64_L2
	or	rax,rax
	jnz	convert_float_64_L1

; exponent=0 fraction=0
; only 0 instead of 0000000000000000
	mov	al,'0'
	stosb
	lea	rdi,[rdi+24]
	jmp	convert_float_64_epi

convert_float_64_L1:
; exponent=0 fraction<>0
; denormalized real
	call	hexa_64_rax
	lea	rdi,[rdi+25]
	jmp	convert_float_64_epi

convert_float_64_L2:
	cmp	dx,07FFh
	jz	convert_float_64_L8

; 0 < exponent < 7FFh
; normalized real f=fraction e=exponent b=bias=1023 for 64-bit floatings
; 1.f  2(e-b)

; rax=floating fraction
; dx=floating exponent

	sub	dx,3FFh+52		; 7FFh correction for exponent = 0
					; 52 correction for the lowest bit of fraction
	movsx	rsi,dx

	bts	rax,52			; set highest bit of fraction

	lea	rcx,[expbyg]
	mov	r9d,[rcx+rsi*4]		; get decimal exponent

	sal	rsi,1			; *2
	lea	rcx,[byg]

	mov	rbx,rax			; save fraction
	mul	qword [rcx+rsi*8+8]	; multiply with upper 64-bits of 128-bits in table
	mov	r8,rdx			; save the highest qword of multiplication
	mov	rbp,rax			; save the medium qword of multiplication
	mov	rax,rbx
	mul	qword [rcx+rsi*8]	; multiply with lower 64-bits of 128-bits in table
	mov	rbx,rax			; save the lowest qword of multiplication
	add	rbp,rdx			; medium qword
	adc	r8,0			; highest qword
; r8:rbp:rbx holds hexadecimal result
; r9d holds decimal exponent

	lea	rsi,[rsp+8*8]
	mov	rax,'00000000'
	mov	[rsi],rax
	mov	[rsi+8],rax
	mov	[rsi+16],rax

	mov	ecx,10
LN64_7:	dec	rsi
	mov	rax,r8
	xor	edx,edx
	div	rcx
	mov	r8,rax
	mov	rax,rbp
	div	rcx
	mov	rbp,rax
	mov	rax,rbx
	div	rcx
	mov	rbx,rax
	add	dl,'0'
	mov	[rsi],dl
	or	rax,rbp
	or	rax,r8
	jnz	LN64_7
	lea	edx,[rsp+8*8-1]
	sub	edx,esi

; rounding
	lea	rax,[rsi+15]
	call	fp_rounding

	movsb
	mov	al,'.'
	stosb
	movsq
	movsd
	movsb
	movsb
	movsb				; 16 digits
	mov	eax,'*10^'
	stosd

	add	r9d,edx			; correction for moving decimal dot upper

	lea	rsi,[rsp+8*8]
	mov	eax,'0000'
	mov	[rsi-4],eax
	mov	eax,r9d
	mov	ebp,eax
	neg	ebp
	or	eax,eax
	cmovs	eax,ebp
	sets	dl
	lea	edx,[rdx+rdx*1+'+']
	mov	byte [rdi],dl
	inc	rdi

LN64_8:	dec	rsi
	xor	edx,edx
	div	ecx
	add	dl,'0'
	mov	[rsi],dl
	or	eax,eax			; not necessary
	jnz	LN64_8

	lea	rsi,[rsp+8*8-3]
	movsb
	movsb
	movsb
	mov	byte [rdi],0

convert_float_64_epi:
	add	rsp,8*11
	pop	r9 r8 rsi rbp rbx rdx rcx rax
	ret

convert_float_64_L8:
	or	rax,rax
	jnz	convert_float_64_L9

; exponent=7FFh fraction=0
	mov	rax,'Infinity'
	stosq
	lea	rdi,[rdi+17]
	jmp	convert_float_64_epi

convert_float_64_L9:
; exponent=7FFh fraction<>0
	bt	rax,51		; test most significant bit of fraction
	mov	edx,'QNaN'	; Quiet Not a Number
	mov	eax,'SNaN'	; Signalling Not a Number
	cmovc	eax,edx
	stosd
	lea	rdi,[rdi+21]
	jmp	convert_float_64_epi

align 16
convert_float_80:	; 80 bits = FPU st register
; in:	rax points to the buffer with the floating point number
;	rdi points to the buffer to store ascii result
	push	rax rcx rdx rbx rbp rsi r8 r9
	sub	rsp,8*11

	cld
	lea	rbx,[rax]

; sign
;	btr	dword [rbx+9],7	; test and reset sign bit of 80-bit floating (bit 9*8+7=79)
; No ! We mustn't destroy this bit since another convert_float_... may will need it to use.
; We will clear this bit later in register, not in memory not to change memory in ThreadContext
	bt	dword [rbx+9],7	; test sign bit of 80-bit floating (bit 9*8+7=79)
	setc	dl
	lea	eax,[rdx+rdx*1+'+']
	stosb

; prepare empty string
	push	rdi
	mov	eax,'    '
	mov	ecx,7
	repz stosd
	mov	byte [rdi],cl	; 0-terminator
	pop	rdi

; get exponent (free of sign bit already)
	mov	edx,dword [rbx+8]	; in fact, only word is required but word operand
					; requires 66h prefix so wastes 1 extra opcode byte
	btr	edx,15			; clear the sign bit

; get fraction
	mov	rax,qword [rbx]
	or	dx,dx
	jnz	convert_float_80_L2
	or	rax,rax
	jnz	convert_float_80_L1

; exponent=0 fraction=0
; only 0 instead of 0000000000000000
	mov	al,'0'
	stosb
	jmp	convert_float_80_epi

convert_float_80_L1:
; exponent=0 fraction<>0
; denormalized real
	call	hexa_64_rax
	lea	rdi,[rdi+16]
	jmp	convert_float_80_epi

convert_float_80_L2:
	cmp	dx,7FFFh
	jz	convert_float_80_L8

; 0 < exponent < 7FFFh
; normalized real f=fraction e=exponent b=bias=16383 for 80-bit floatings
; 1.f  2(e-b)

; rax=floating fraction
; dx=floating exponent
; e.g. let rax=80000005h then we can transform it (2^64+2^2+2^0)*10^exponent
; so we simple get value at dx-64 from table
; and binary multiply with fraction (rax)

	sub	dx,3FFFh+63		; 3FFFh correction for exponent = 0
					; 64 correction for the lowest bit of fraction
					; we will get value from the table for the lowest
					; bit of fraction, so the highest bit of fraction
					; is 63 and the difference from bit 0 is 63 bits
	movsx	rsi,dx

	lea	rcx,[expbyg]
	mov	r9d,[rcx+rsi*4]		; get decimal exponent

	sal	rsi,1			; *2
	lea	rcx,[byg]

	mov	rbx,rax			; save fraction
	mul	qword [rcx+rsi*8+8]	; multiply with upper 64-bits of 128-bits in table
	mov	r8,rdx			; save the highest qword of multiplication
	mov	rbp,rax			; save the medium qword of multiplication
	mov	rax,rbx
	mul	qword [rcx+rsi*8]	; multiply with lower 64-bits of 128-bits in table
	mov	rbx,rax			; save the lowest qword of multiplication
	add	rbp,rdx			; medium qword
	adc	r8,0			; highest qword
; r8:rbp:rbx holds hexadecimal result
; r9d holds decimal exponent

	lea	rsi,[rsp+8*8]
	mov	rax,'00000000'
	mov	[rsi],rax
	mov	[rsi+8],rax
	mov	[rsi+16],rax

	mov	ecx,10
LN7:	dec	rsi
	mov	rax,r8
	xor	edx,edx
	div	rcx
	mov	r8,rax
	mov	rax,rbp
	div	rcx
	mov	rbp,rax
	mov	rax,rbx
	div	rcx
	mov	rbx,rax
	add	dl,'0'
	mov	[rsi],dl
	or	rax,rbp
	or	rax,r8
	jnz	LN7
	lea	edx,[rsp+8*8-1]
	sub	edx,esi

; rounding
	lea	rax,[rsi+17]
	call	fp_rounding

	movsb
	mov	al,'.'
	stosb
	movsq
	movsq
	movsb				; 18 digits
	mov	eax,'*10^'
	stosd

	add	r9d,edx			; correction for moving decimal dot upper

	lea	rsi,[rsp+8*8]
	mov	eax,'0000'
	mov	[rsi-4],eax
	mov	eax,r9d
	mov	ebp,eax
	neg	ebp
	or	eax,eax
	cmovs	eax,ebp
	sets	dl
	lea	edx,[rdx+rdx*1+'+']
	mov	byte [rdi],dl
	inc	rdi

LN8:	dec	rsi
	xor	edx,edx
	div	ecx
	add	dl,'0'
	mov	[rsi],dl
	or	eax,eax			; not necessary
	jnz	LN8

	lea	rsi,[rsp+8*8-4]
	movsd
	mov	byte [rdi],0

convert_float_80_epi:
	add	rsp,8*11
	pop	r9 r8 rsi rbp rbx rdx rcx rax
	ret

convert_float_80_L8:
	or	rax,rax
	jnz	convert_float_80_L9

; exponent=7FFFh fraction=0
	mov	rax,'Infinity'
	stosq
	jmp	convert_float_80_epi

convert_float_80_L9:
; exponent=7FFFh fraction<>0
	mov	edx,'QNaN'	; Quiet Not a Number
	mov	eax,'SNaN'	; Signalling Not a Number
	cmovs	eax,edx
	stosd
	jmp	convert_float_80_epi

fp_rounding:
	mov	byte [rsi-1],'0'	; in case of rounding number 9.999... -> 10.00...
	cmp	byte [rax+1],'5'
	jc	fp_r9
fp_r0:	inc	byte [rax]
	cmp	byte [rax],'9'
	jbe	fp_r9
	mov	byte [rax],'0'
	dec	rax
	jmp	fp_r0
fp_r9:	cmp	rax,rsi
	cmovc	rsi,rax
	ret

align 16
ascii2fpu:
	push	rax rcx rdx rbx rbp rsi r8 r9 r10 r11 r12 r13 r14 r15
	sub	rsp,8*5

; get mantissa
	mov	word [rsp],00000011b*100h + 7Fh	; 00b Round to nearest (even), 11b 64 bit precision (mantissa)
	fldcw	word [rsp]
	call	get_sign
	mov	qword [rsp+8*2],r11
	call	get_ascii_mantissa
	jc	ascii2fpu_egzyd
	call	format_fp_mantissa
	lea	r15,[rsp+8*0]
	call	pack_number
	jc	ascii2fpu_egzyd
	fbld	tbyte [rsp+8*0]
	cmp	byte [rsp+8*2],0
	jz	skip_change_mantissa_sign
	fchs
skip_change_mantissa_sign:
	lea	rax,[ten_powers]
	lea	rax,[rax + r14*2]
	fld	tbyte [rax + r14*8]		; every uses tbyte = ten bytes
	fmulp	st1,st0

; get exponent
	or	r13,r13				; exponent size
	jz	exponent_done
	lea	rsi,[r12]			; begin of exponent
	mov	r10,r13				; exponent ascii string size
	call	get_sign
	mov	qword [rsp+8*2],r11
	mov	r11,r10				; exponent ascii string size
	lea	r15,[rsp+8*0]
	call	pack_number
	jc	ascii2fpu_egzyd
	fbld	tbyte [rsp+8*0]
	cmp	byte [rsp+8*2],0
	jz	skip_change_exponent_sign
	fchs
skip_change_exponent_sign:
; Calculate 10 to the power of ST. Return result in ST. Uses 10**N = 2**(N*Log2(10)).
	fldl2t					; load Log2(10)
	fmulp	st1,st0				; multiply
	call	exp2

; and terminaly multiply mantissa with 10**exponent
	fmulp	st1,st0
exponent_done:
	add	rsp,8*5				; clears carry too

ascii2fpu_epi:
	pop	r15 r14 r13 r12 r11 r10 r9 r8 rsi rbp rbx rdx rcx rax
	ret

ascii2fpu_egzyd:
	add	rsp,8*5
	stc
	jmp	ascii2fpu_epi

align 16
get_sign:
; input:	ascii string at rsi
; return:	process + or - at begin of ascii string, adjust rsi and r10
;		r11=0 if no sign at begin of string or if + at begin of string
;		r11=1 if - at begin of string
;		r12 exponent begin, 0 if no exponent
	push	rax
	mov	al,[rsi]
	cmp	al,'-'
	setz	r11b
	cmp	al,'+'
	setz	al
	or	al,r11b
	movzx	r11d,r11b
	movzx	eax,al
	add	rsi,rax
	sub	r10,rax
	pop	rax
	ret

align 16
get_ascii_mantissa:
; input:	decimal number ascii string at rsi, may contain . as decimal separator, is ended by *10^
;		string size in r10, mantissa must be < 20 (because fbld instruction for load it at the end)
; output:	carry flag if ascii string exceede interval <1,19>
;		r11 mantissa size
;		r12 point to exponent begin
;		r13 number of digits of exponent
	or	r10,r10
	jz	get_ascii_mantissa_bad_size
	mov	r11d,18+1		; 18 digits and 1 decimal pointer .
	cmp	r10,r11
	cmovc	r11,r10
get_ascii_mantissa_L3:
	lea	r13,[rsi+r10]		; point to end of string
	lea	r12,[rsi+r11+4]		; point to exponent begin
	sub	r13,r12			; exponent size
	cmp	dword [rsi+r11],'*10^'
	jz	get_ascii_mantissa_L7
	dec	r11
	jnz	get_ascii_mantissa_L3
	mov	r11d,18+1		; no exponent found, we grab first digits, max 18+1
	cmp	r10,r11
	cmovc	r11,r10
	xor	r13,r13			; zero exponent size
get_ascii_mantissa_L7:
	clc
	ret
get_ascii_mantissa_bad_size:
	stc
	ret

align 16
format_fp_mantissa:
; input:	decimal number ascii string at rsi, may contain . as decimal separator
;		mantissa size in r11
; output:	format string in buffer = remove . in string if it is present
;		r11 string size without .
;		r14 = number_of_digits_after . (correction for exponent)
; destroy:	rsi, destroy string at rsi (move -1 all digits after decimal separator e.g. 123.456 -> 1234566)
	push	rax
	push	rcx
	push	rdi
	mov	al,'.'
	mov	rcx,r11
	lea	rdi,[rsi]
	cld
	repnz scasb
	setnz	al			; al=0 in case of decimal pointer in number, 1 if decimal pointer ins't present
	movzx	eax,al
	mov	r14,rcx
;	jnz	format_fp_mantissa_L5
	push	rsi			; save rsi
	lea	rsi,[rdi]
	lea	rdi,[rdi-1]
	repz movsb
	pop	rsi			; restore rsi
;format_fp_mantissa_L5:
	lea	r11,[r11+rax-1]		; -1 in case of decimal separator in number, don't change r11 in case of absent decimal pointer
	pop	rdi
	pop	rcx
	pop	rax
	ret

align 16
pack_number:
; input:	rsi decimal number ascii string
;		r15 10 byte output buffer
;		r11 string size
; destroy:	r11, r15
	or	r11,r11			; zero string size ?
	jz	pack_number_L9
	cmp	r11,18
	jnbe	pack_number_L9
	push	rax
	xor	eax,eax
	mov	qword [r15],rax		; zeroing output buffer
	mov	dword [r15+6],eax	; zeroing output buffer

align 16
pack_number_L3:
	mov	al,[rsi + r11 - 1]
	sub	al,'0'
	jc	pack_number_L8
	cmp	al,10
	jnc	pack_number_L8
	mov	byte [r15],al
	dec	r11
	jz	pack_number_L7

	mov	al,[rsi + r11 - 1]
	sub	al,'0'
	jc	pack_number_L8
	cmp	al,10
	jnc	pack_number_L8
	shl	al,4
	or	byte [r15],al
	inc	r15
	dec	r11
	jnz	pack_number_L3
pack_number_L7:
	pop	rax
	clc
	ret
pack_number_L8:
	pop	rax
pack_number_L9:
	stc
	ret

align 16
exp2:
; Calculate 2 to the power of ST. Return result in ST(0).
; We needn't to set 03BFh new control word - round to nearest - it has been set already
	fld	st0				; duplicate number
	frndint					; round to integer, ST => W
	fxch					; exchange
	fsub	st0,st1				; get difference, ST => F

; check sign of fractional portion (F)
	ftst					; set flags
	push	rax
	fstsw	word [rsp]			; save flags
	pop	rax
	and	ah,45h				; mask relevant bits
	cmp	ah,1				; check negative bit
	ja	Exp2err				; jump if other bits set, error
	jz	Exp2neg				; jump if negative

; fractional part is positive for
	f2xm1					; ST => (2**F) - 1
	fld1					; load one
	faddp	st1,st0				; ST => 2**F put whole portion (W) on top
	fxch					; put whole portion (W) on top
	fld1					; load one
	fscale					; ST => 2**W
	fmulp	st2,st0				; ST(2) => (2**F) * (2**W)
	fstp	st0				; pop stack
	jmp	Exp2don

; fractional part is negative
Exp2neg:
	fabs					; take absolute value
	f2xm1					; ST => (2**F) - 1
	fld1					; load one
	faddp	st1,st0				; ST => 2**F
	fxch					; put whole portion (W) on top
	fld1					; load one
	fscale					; ST => 2**W
	fdivrp	st2,st0				; ST(2) => (2**W) / (2**F)
	fstp	st0				; pop stack

;finished
Exp2don:
	ret

Exp2err:
	fstp	st0
	fstp	st0				; clear stack
	fld1					; return one
	ret

; -------

align 16
dword_decimal_ascii:
; input:	eax hexa number
;		rdi buffer for store decimal ascii
; output:	make 0-terminated ascii string at RDI
	push	rax rcx rdx rbx rsi
	lea	rsi,[rsp]
	sub	rsp,8*2

	mov	ebx,10
	lea	ecx,[rsi]
dword_decimal_ascii_L0:
	dec	rsi
	xor	edx,edx
	div	ebx
	add	dl,'0'
	mov	[rsi],dl
	or	eax,eax			; not necessary
	jnz	dword_decimal_ascii_L0
	sub	ecx,esi
	cld
	repz movsb
	mov	byte [rdi],cl

	add	rsp,8*2
	pop	rsi rbx rdx rcx rax
	ret

align 16
qword_signed_decimal_ascii:
; input:	rax number
;		rdi buffer for store ascii
; output:	make 0-terminated ascii string at RDI
	push	rax rcx rdx rbx rsi
	lea	rsi,[rsp]
	sub	rsp,8*4

	mov	rcx,rax
	neg	rax		; rcx = -1 * rax
	or	rcx,rcx		; is rax negative value ?
	cmovs	rcx,rax
	sets	al		; set CL=0 if rax positive, CL=1 if rax negative value
	lea	eax,[rax+rax*1+'+']
	cld
	stosb

	mov	rax,rcx
	mov	ebx,10
	lea	ecx,[rsi]
qword_signed_decimal_ascii_L0:
	dec	rsi
	xor	edx,edx
	div	rbx
	add	dl,'0'
	mov	[rsi],dl
	or	rax,rax			; not necessary
	jnz	qword_signed_decimal_ascii_L0
	sub	ecx,esi
	repz movsb
	mov	byte [rdi],cl

	add	rsp,8*4
	pop	rsi rbx rdx rcx rax
	ret

align 16
bcd2decimalsigned:
; in:	xmm0 holds bcd number
	push	rax rcx rsi
	sub	rsp,8*4
	emms
	movdqa	dqword [rsp],xmm0
	mov	al,'+'
	cld
	stosb
	lea	rsi,[rdi]
	movzx	eax,word [rsp+8]
	test	ah,80h
	setnz	cl
	shl	cl,1
	add	byte [rdi-1],cl	; change '+' to '-' (2Bh to 2Dh)
	and	ah,01111111b
	bsr	ecx,eax
	jz	bcd2decimalsigned_L5
	shr	cl,2
	add	cl,1
; ecx = number of nonzero nybbles
	lea	rdi,[rsp+16]
	call	hexa_64_rax
	lea	rdi,[rsi]
	lea	rsi,[rsp+16+16]
	sub	rsi,rcx
	repz movsb
	lea	rsi,[rdi]
	lea	rdi,[rsp]
	mov	rax,qword [rdi]
	call	hexa_64_rax
	xchg	rsi,rdi
	mov	cl,16
bcd2decimalsigned_L3:
	repz movsb
	mov	ax,'.0'
	stosw
	mov	byte [rdi],cl		; 0-terminator
	add	rsp,8*4
	pop	rsi rcx rax
	ret

bcd2decimalsigned_L5:
	lea	rdi,[rsp]
	mov	rax,qword [rdi]
	call	hexa_64_rax
	xor	edi,edi
	bsr	rcx,rax
	cmovz	rcx,rdi			; at least one digit must be displayed
	shr	cl,2
	add	cl,1
	lea	rdi,[rsi]
	lea	rsi,[rsp+16]
	sub	rsi,rcx
	jmp	bcd2decimalsigned_L3
