; tool for checking potential stack misalignment parts in asm program
; syntax:
; fxa16.exe file.d64
; e.g.
; fxa16.exe fdbg.d64
; output: hexadecimal number of line with potential stack misalignment
; it check is with very primitive algorythm:
; it scans for sub rsp,val instruction
; then it scans back how much registers are pushed
; the simple algorythm may be very easily cheated by inserting another instructions between push - sub instruction pair
; the d64 file may be obtained using fdisasm tool

; compiled with FASM 1.67.29

format PE64 GUI at 100000000h on 'hdr.exe'


include '%fasminc%\win64a.inc'
include	'macro_align.inc'


section '.text' code readable executable

start:

	sub	rsp,8*(4+11)

	call	qword [GetCommandLineA]
	push	rax
	pop	rsi     	; parse shell line parameters
	lea	rdi,[file_name]
	lea	rcx,[rdi + file_name_max_size]
L00:	inc	rsi
	cmp	byte [rsi],' '
	ja	L00
L01:	cmp	byte [rsi],0
	jz	exit
	inc	rsi
	cmp	byte [rsi],' '
	jna	L01
L02:	movsb
	cmp	rdi,rcx
	jnbe	exit		; too long, too many characters on shell line
	cmp	byte [rsi],' '
	ja	L02
	mov	byte [rdi],0

	lea	rax,[file_name]
	mov	rcx,rdi
	sub	rcx,rax
	dec	rdi
	mov	al,'.'
	std
	repnz scasb
	jnz	exit
	cld
	scasw		; only for rdi+2

have_file_extension:
	mov	eax,[rdi]
	or	eax,00202020h	; convert capitals to lowercase, e.g. 'ObJ' -> 'obj'
	cmp	eax,'d64'
	jnz	exit

	mov	qword [rsp+8*(4+2)],0				; hTemplateFile
	mov	dword [rsp+8*(4+1)],FILE_ATTRIBUTE_NORMAL	; dwFlagsAndAttributes
	mov	dword [rsp+8*(4+0)],OPEN_EXISTING		; dwCreationDistribution
	xor	r9,r9						; lpSecurityAttributes
	xor	r8d,r8d						; dwShareMode
	mov	edx,GENERIC_READ				; dwDesiredAccess
	lea	rcx,[file_name]					; lpFileName
	call	qword [CreateFileA]
	xchg	rbx,rax			; save file handle
	
if INVALID_HANDLE_VALUE = -1
	inc	rax
else
	cmp	rax,INVALID_HANDLE_VALUE
end if
	jz	exit

	xor	edx,edx			; lpFileSizeHigh	; we needn't filesize > 2^64	;lea	rdx,[rsp+8*(4+0)]
	push	rbx
	pop	rcx			; hFile
	call	qword [GetFileSize]
	cmp	eax,-1
	jz	close_exit

	xchg	ebp,eax			; save filesize

	mov	r9d,PAGE_READWRITE
	mov	r8d,MEM_COMMIT
	mov	edx,ebp
	xor	ecx,ecx
	call	qword [VirtualAlloc]
	lea	r15,[rax]		; save memory buffer
	or	rax,rax
	jz	close_exit

	mov	qword [rsp+8*(4+0)],0	; lpOverlapped
	lea	r9,[rsp+8*(4+1)]	; lpNumberOfBytesRead
	mov	r8d,ebp			; nNumberOfBytesToRead
	push	rax
	pop	rdx			; lpBuffer
	push	rbx
	pop	rcx			; hFile
	call	qword [ReadFile]
	or	eax,eax
	jz	close_exit
	cmp	[rsp+8*(4+1)],ebp
	jnz	close_exit

	push	rbx
	pop	rcx
	call	qword [CloseHandle]

; check the file
; r15 points to the buffer
; ebp size

	mov	r14d,1			; line counter
	lea	rsi,[r15]
	mov	ebx,ebp
	cld
L0:	cmp	dword [rsi],'sub '
	jnz	L8
	cmp	dword [rsi+4],'rsp,'
	jnz	L8

; scan number of pushed registers to the back
	xor	edx,edx		; counter
	lea	rdi,[rsi]

	lea	ecx,[rdi]	; 2 = 0D0Ah
	sub	ecx,r15d
	jc	L7		; this occures only if the first instruction in *.d64 is sub rsp,
	push	0Ah
	pop	rax
	std
	repnz scasb
; rdi points to 0D preceeding sub rsp,
; DF=1

L1:	mov	al,0Ah
	repnz scasb
	setz	al
; rax=0 at the begin of data
; rax=1 inside the data
	cmp	dword [rdi+rax*2],'push'
; so +2 means 2 bytes ahead 0Dh byte which is before 0Ah and then 'push' follows

; none of 2 following instructions touches zero flag set by above cmp
	setz	al
	lea	edx,[rdx+rax*1]
	lea	rdi,[rdi-1]
	jrcxz	L7
	loopz	L1		; loop only if zero flag ZF=0
				; leave loop if the last instruc wasn't push

L7:	lea	rdi,[rsi+8]	; after sub rsp,
	mov	al,0Dh
	lea	ecx,[r15+rbp*1]
	sub	ecx,edi
	cld
	repnz scasb
	and	dl,1		; leave only lowest bit
	xor	dl,1
	shl	dl,3		; shift the lowest bit (0->0 1->8)
	mov	al,[rdi-2]	; grab last digit of sub rsp,xxxxxxxx
	and	al,1111b
	cmp	al,dl
	jnz	unaligned

L8:	cmp	byte [rsi],0Ah	; new line?
	jnz	L9
	inc	r14		; increase line counter

L9:	inc	rsi		; lodsb		rsi+1 in 1 byte instruction, DF=0 (cld)
	dec	ebx
	cmp	ebx,8
	jnbe	L0

unalloc_exit:
	mov	r8d,MEM_DECOMMIT
	mov	edx,ebp
	lea	rcx,[r15]
	call	qword [VirtualFree]

exit:	;add	rsp,8*(4+11)
	;xor	eax,eax
	;ret
; this is safer and destroys other possible threads also:
	xor	ecx,ecx
	call	qword [ExitProcess]

unaligned:
	lea	rcx,[rdi+1]
;	lea	eax,[rsi]
;	sub	eax,r15d
	mov	rax,r14
	lea	rdi,[rsp+8*4]
	push	rdi
	pop	rdx
	call	hexa_64_rax_aligned
	lea	rdi,[rdi+16]
	cld
	mov	al,0Dh
	stosb
	mov	al,0Ah
	stosb
	sub	ecx,esi
	repz movsb
	mov	[rdi],cl

	xor	r9,r9
	xor	r8,r8
;	lea	rdx,[rdx]
	xor	ecx,ecx
	call	qword [MessageBoxA]

;	jmp	unalloc_exit
	jmp	L8

unalloc_close_exit:
	mov	r8d,MEM_DECOMMIT
	mov	edx,ebp
	lea	rcx,[r15]
	call	qword [VirtualFree]

close_exit:
	push	rbx
	pop	rcx
	call	qword [CloseHandle]

	add	rsp,8*(4+11)
	push	1
	pop	rax
	ret

align 16
include	'hexa_data.inc'
include	'fta16_hexa_proc_ftr.inc'


section '.data' data readable writeable
align 16
file_name_max_size	= 512
file_name	rb	file_name_max_size


section '.idata' import data readable writeable

			dd	0,0,0,	RVA kernel_name,	RVA kernel_table
			dd	0,0,0,	RVA user_name,		RVA user_table
			dd	0,0,0,	0,			0

kernel_table:
CloseHandle		dq	RVA _CloseHandle
CreateFileA		dq	RVA _CreateFileA
ExitProcess		dq	RVA _ExitProcess
GetCommandLineA		dq	RVA _GetCommandLineA
GetFileSize		dq	RVA _GetFileSize
ReadFile		dq	RVA _ReadFile
VirtualAlloc		dq	RVA _VirtualAlloc
VirtualFree		dq	RVA _VirtualFree
;WriteFile		dq	RVA _WriteFile
			dq	0
user_table:
MessageBoxA		dq	RVA _MessageBoxA
			dq	0


kernel_name		db	'KERNEL32.DLL',0
user_name		db	'USER32.DLL',0

; kernel32.dll:
_CloseHandle		db	0,0,'CloseHandle',0
_CreateFileA		db	0,0,'CreateFileA',0
_ExitProcess		db	0,0,'ExitProcess',0
_GetCommandLineA	db	0,0,'GetCommandLineA',0
_GetFileSize		db	0,0,'GetFileSize',0
_ReadFile		db	0,0,'ReadFile',0
_VirtualAlloc		db	0,0,'VirtualAlloc',0
_VirtualFree		db	0,0,'VirtualFree',0
;_WriteFile		db	0,0,'WriteFile',0
; user32.dll
_MessageBoxA		db	0,0,'MessageBoxA',0
