; psquery.c
;
; NTSTATUS
; NtQueryInformationThread(
;     __in HANDLE ThreadHandle,
;     __in THREADINFOCLASS ThreadInformationClass,
;     __out_bcount(ThreadInformationLength) PVOID ThreadInformation,
;     __in ULONG ThreadInformationLength,
;     __out_opt PULONG ReturnLength
;     )
;
; /*++
;
; Routine Description:
;
;     This function queries the state of a thread object and returns the
;     requested information in the specified record structure.
;
; Arguments:
;
;     ThreadHandle - Supplies a handle to a thread object.
;
;     ThreadInformationClass - Supplies the class of information being
;         requested.
;
;     ThreadInformation - Supplies a pointer to a record that is to
;         receive the requested information.
;
;     ThreadInformationLength - Supplies the length of the record that is
;         to receive the requested information.
;
;     ReturnLength - Supplies an optional pointer to a variable that is to
;         receive the actual length of information that is returned.
;
; --*/
;
; ...
;
;    case ThreadBasicInformation:
;
;        if (ThreadInformationLength != (ULONG) sizeof (THREAD_BASIC_INFORMATION)) {
;            return STATUS_INFO_LENGTH_MISMATCH;
;        }
;
;        st = ObReferenceObjectByHandle (ThreadHandle,
;                                        THREAD_QUERY_INFORMATION,
;                                        PsThreadType,
;                                        PreviousMode,
;                                        &Thread,
;                                        NULL);
;        if (!NT_SUCCESS (st)) {
;            return st;
;        }
;
;        if (KeReadStateThread (&Thread->Tcb)) {
;            BasicInfo.ExitStatus = Thread->ExitStatus;
;        } else {
;            BasicInfo.ExitStatus = STATUS_PENDING;
;        }
;
;        BasicInfo.TebBaseAddress = (PTEB) Thread->Tcb.Teb;
;        BasicInfo.ClientId = Thread->Cid;
;        BasicInfo.AffinityMask = Thread->Tcb.Affinity;
;        BasicInfo.Priority = Thread->Tcb.Priority;
;        BasicInfo.BasePriority = KeQueryBasePriorityThread (&Thread->Tcb);
;
;        ObDereferenceObject (Thread);
;
;        //
;        // Either of these may cause an access violation. The
;        // exception handler will return access violation as
;        // status code. No further cleanup needs to be done.
;        //
;
;        try {
;            *(PTHREAD_BASIC_INFORMATION) ThreadInformation = BasicInfo;
;
;            if (ARGUMENT_PRESENT (ReturnLength) ) {
;                *ReturnLength = sizeof (THREAD_BASIC_INFORMATION);
;            }
;        } except (EXCEPTION_EXECUTE_HANDLER) {
;            return GetExceptionCode ();
;        }
;
;        return STATUS_SUCCESS;

; ntpsapi.h
;
; //
; // Thread Information Classes
; //
;
; typedef enum _THREADINFOCLASS {
;     ThreadBasicInformation,
;     ThreadTimes,
;     ThreadPriority,
;     ThreadBasePriority,
;     ThreadAffinityMask,
;     ThreadImpersonationToken,
;     ThreadDescriptorTableEntry,
;     ThreadEnableAlignmentFaultFixup,
;     ThreadEventPair_Reusable,
;     ThreadQuerySetWin32StartAddress,
;     ThreadZeroTlsCell,
;     ThreadPerformanceCount,
;     ThreadAmILastThread,
;     ThreadIdealProcessor,
;     ThreadPriorityBoost,
;     ThreadSetTlsArrayAddress,
;     ThreadIsIoPending,
;     ThreadHideFromDebugger,
;     ThreadBreakOnTermination,
;     ThreadSwitchLegacyState,
;     ThreadIsTerminated,
;     MaxThreadInfoClass
;     } THREADINFOCLASS;


; //
; // Basic Thread Information
; //  NtQueryInformationThread using ThreadBasicInfo
; //
;
; typedef struct _THREAD_BASIC_INFORMATION {
;     NTSTATUS ExitStatus;
;     PTEB TebBaseAddress;
;     CLIENT_ID ClientId;
;     ULONG_PTR AffinityMask;
;     KPRIORITY Priority;
;     LONG BasePriority;
; } THREAD_BASIC_INFORMATION;
; typedef THREAD_BASIC_INFORMATION *PTHREAD_BASIC_INFORMATION;


struc	THREAD_BASIC_INFORMATION
{
.ExitStatus				dd	?
.Pad1					dd	?
.TebBaseAddress				dq	?
.ClientId				dq	?
.AffinityMask				dq	?
.Priority				dd	?
.Pad2					dd	?
.BasePriority				dq	?
}

virtual at 0
vtbi		THREAD_BASIC_INFORMATION
sizeof.THREAD_BASIC_INFORMATION	=	$
end virtual




format PE64 GUI at (1 shl 32) on 'nul'
entry start

include	'%fasminc%\win64a.inc'


section '.code' code readable executable writeable

start:

a = 1			; return address from procedure
b = (a+c+d+e) and 1	; align stack at 16
c = (sizeof.THREAD_BASIC_INFORMATION + (8-1)) / 8
d = 2			; 1 qword for 5th API param and qword 1 for pointer
e = 4			; 4 qwords reserved for API

	sub	rsp,8*(e+d+c+b)

virtual at rsp+8*(e+d)
thrd_basic_info	THREAD_BASIC_INFORMATION
end virtual

	lea	rax,[rsp+8*(4+1)]
	mov	[rsp+8*(4+0)],rax			; __out_opt PULONG ReturnLength
;	mov	r9d,4					; __in ULONG ThreadInformationLength
	lea	r8,[thrd_basic_info]			; __out_bcount(ThreadInformationLength) PVOID ThreadInformation
	xor	edx,edx					; __in THREADINFOCLASS ThreadInformationClass
;	mov	rcx,-2					; __in HANDLE ThreadHandle
							; hCurrentThread = FFFFFFFFFFFFFFFE
	lea	rcx,[rdx-2]
	lea	r9d,[rdx + sizeof.THREAD_BASIC_INFORMATION]
	call	[NtQueryInformationThread]
	or	eax,eax
	js	exit

; PEB base address
	mov	rax,[thrd_basic_info.TebBaseAddress]

; this is the same as mov rax,[gs: dword 30h] but with the above way we can obtain it for alien thread(s)

	bswap	rax
	movdqa	xmm3,dqword [Sum1]
	movdqa	xmm4,dqword [Comp1]
	movdqa	xmm2,dqword [Mask1]
	movdqa	xmm5,dqword [Num1]
	movq	xmm0,rax
	movdqa	xmm1,xmm0
	psrlq	xmm0,4
	pand	xmm0,xmm2
	pand	xmm1,xmm2
	punpcklbw	xmm0,xmm1
	movdqa	xmm1,xmm0
	pcmpgtb	xmm0,xmm4
	pand	xmm0,xmm5
	paddb	xmm1,xmm3
	paddb	xmm1,xmm0

	xor	r9,r9
	lea	r8,[title]
	lea	rdx,[msg]
	xor	ecx,ecx
	movdqa	[rdx],xmm1
	call	[MessageBoxA]

exit:	xor	eax,eax
	add	rsp,8*(e+d+c+b)
	ret


align 10h
title			db	'TebBaseAddress',0

align 10h
msg			db	'                ','h',0


align 10h
Sum1			dq	3030303030303030h, 3030303030303030h
Mask1			dq	0f0f0f0f0f0f0f0fh, 0f0f0f0f0f0f0f0fh
Comp1			dq	0909090909090909h, 0909090909090909h
Num1			dq	0707070707070707h, 0707070707070707h



section '.idata' import data readable writeable

				dd	0,0,0,	RVA ntdll_name,		RVA ntdll_table
				dd	0,0,0,	RVA user_name,		RVA user_table
				dd	0,0,0,	0,			0

align 8
ntdll_table:
NtQueryInformationThread	dq	RVA _NtQueryInformationThread
				dq	0

user_table:
MessageBoxA			dq	RVA _MessageBoxA
				dq	0

ntdll_name			db	'NTDLL.DLL',0
user_name			db	'USER32.DLL',0

; ntdll.dll:
_NtQueryInformationThread	db	0,0,'NtQueryInformationThread',0

; user32.dll
_MessageBoxA			db	0,0,'MessageBoxA',0
