; psquery.c
;
; NTSTATUS
; NtQueryInformationThread(
;     __in HANDLE ThreadHandle,
;     __in THREADINFOCLASS ThreadInformationClass,
;     __out_bcount(ThreadInformationLength) PVOID ThreadInformation,
;     __in ULONG ThreadInformationLength,
;     __out_opt PULONG ReturnLength
;     )
;
; /*++
;
; Routine Description:
;
;     This function queries the state of a thread object and returns the
;     requested information in the specified record structure.
;
; Arguments:
;
;     ThreadHandle - Supplies a handle to a thread object.
;
;     ThreadInformationClass - Supplies the class of information being
;         requested.
;
;     ThreadInformation - Supplies a pointer to a record that is to
;         receive the requested information.
;
;     ThreadInformationLength - Supplies the length of the record that is
;         to receive the requested information.
;
;     ReturnLength - Supplies an optional pointer to a variable that is to
;         receive the actual length of information that is returned.
;
; --*/
;
; ...
;
;    case ThreadQuerySetWin32StartAddress:
;        if (ThreadInformationLength != sizeof (ULONG_PTR)) {
;            return STATUS_INFO_LENGTH_MISMATCH;
;        }
;
;        st = ObReferenceObjectByHandle (ThreadHandle,
;                                        THREAD_QUERY_INFORMATION,
;                                        PsThreadType,
;                                        PreviousMode,
;                                        &Thread,
;                                        NULL);
;
;        if (!NT_SUCCESS (st)) {
;            return st;
;        }
;
;        Win32StartAddressValue = Thread->Win32StartAddress;
;        ObDereferenceObject (Thread);
;
;        try {
;            *(PVOID *) ThreadInformation = Win32StartAddressValue;
;
;            if (ARGUMENT_PRESENT (ReturnLength) ) {
;                *ReturnLength = sizeof (ULONG_PTR);
;            }
;        } except (EXCEPTION_EXECUTE_HANDLER) {
;            return GetExceptionCode ();
;        }


; ntpsapi.h
;
; //
; // Thread Information Classes
; //
;
; typedef enum _THREADINFOCLASS {
;     ThreadBasicInformation,
;     ThreadTimes,
;     ThreadPriority,
;     ThreadBasePriority,
;     ThreadAffinityMask,
;     ThreadImpersonationToken,
;     ThreadDescriptorTableEntry,
;     ThreadEnableAlignmentFaultFixup,
;     ThreadEventPair_Reusable,
;     ThreadQuerySetWin32StartAddress,
;     ThreadZeroTlsCell,
;     ThreadPerformanceCount,
;     ThreadAmILastThread,
;     ThreadIdealProcessor,
;     ThreadPriorityBoost,
;     ThreadSetTlsArrayAddress,
;     ThreadIsIoPending,
;     ThreadHideFromDebugger,
;     ThreadBreakOnTermination,
;     ThreadSwitchLegacyState,
;     ThreadIsTerminated,
;     MaxThreadInfoClass
;     } THREADINFOCLASS;



format PE64 GUI at (1 shl 32) on 'nul'
entry start

include	'%fasminc%\win64a.inc'


section '.code' code readable executable writeable

start:

a = 1			; return address from procedure
b = (a+c+d) and 1	; align stack at 16
c = 3			; 1 qword for 5th API param, qword 1 for pointer, 1 qword for obtaining handle
d = 4			; 4 qwords reserved for API

	sub	rsp,8*(d+c+b)

	lea	rax,[rsp+8*(4+1)]
	mov	[rsp+8*(4+0)],rax			; __out_opt PULONG ReturnLength
;	mov	r9d,8					; __in ULONG ThreadInformationLength
	lea	r8,[rsp+8*(4+2)]			; __out_bcount(ThreadInformationLength) PVOID ThreadInformation
	mov	edx,9					; __in THREADINFOCLASS ThreadInformationClass
;	mov	rcx,-2					; __in HANDLE ThreadHandle
							; hCurrentThread = FFFFFFFFFFFFFFFE
	lea	rcx,[rdx-9 -2]
	lea	r9d,[rdx-9 +8]
	call	[NtQueryInformationThread]
	or	eax,eax
	js	exit

; Win32StartAddress
	mov	rax,[rsp+8*(4+2)]

	bswap	rax
	movdqa	xmm3,dqword [Sum1]
	movdqa	xmm4,dqword [Comp1]
	movdqa	xmm2,dqword [Mask1]
	movdqa	xmm5,dqword [Num1]
	movq	xmm0,rax
	movdqa	xmm1,xmm0
	psrlq	xmm0,4
	pand	xmm0,xmm2
	pand	xmm1,xmm2
	punpcklbw	xmm0,xmm1
	movdqa	xmm1,xmm0
	pcmpgtb	xmm0,xmm4
	pand	xmm0,xmm5
	paddb	xmm1,xmm3
	paddb	xmm1,xmm0

	xor	r9,r9
	lea	r8,[title]
	lea	rdx,[msg]
	xor	ecx,ecx
	movdqa	[rdx],xmm1
	call	[MessageBoxA]

exit:	xor	eax,eax
	add	rsp,8*(d+c+b)
	ret


align 10h
title			db	'Thread->Win32StartAddress',0

align 10h
msg			db	'                ','h',0


align 10h
Sum1			dq	3030303030303030h, 3030303030303030h
Mask1			dq	0f0f0f0f0f0f0f0fh, 0f0f0f0f0f0f0f0fh
Comp1			dq	0909090909090909h, 0909090909090909h
Num1			dq	0707070707070707h, 0707070707070707h



section '.idata' import data readable writeable

				dd	0,0,0,	RVA ntdll_name,		RVA ntdll_table
				dd	0,0,0,	RVA user_name,		RVA user_table
				dd	0,0,0,	0,			0

align 8
ntdll_table:
NtQueryInformationThread	dq	RVA _NtQueryInformationThread
				dq	0

user_table:
MessageBoxA			dq	RVA _MessageBoxA
				dq	0

ntdll_name			db	'NTDLL.DLL',0
user_name			db	'USER32.DLL',0

; ntdll.dll:
_NtQueryInformationThread	db	0,0,'NtQueryInformationThread',0

; user32.dll
_MessageBoxA			db	0,0,'MessageBoxA',0
