; sample to try fdbg Control -> Toggle ... (safe) and fdbg SYS -> ... (dangerous !!! experimental !!!)
; there was a presentation about these features at 2nd FASM conference, Brno, Czech Republic, 2007-august-25
; http://fdbg.x86asm.net/debugging-in-long-mode-in-theory.html#Other_Features
; for more, go to www.amd.com then download 24593.pdf then read chapter 13.2.5 Control-Transfer Breakpoint Features
;
; After enabling LBR bit of DebugCtlMSR, the source and destination addresses of control-transfer events are saved by the processor - branches (call, jmp), interrupts, exceptions. We have four registers: LastBranchFromIP, LastBranchToIP, LastExceptionFromIP, LastExceptionToIP.
; As instructions execute, addresses of branching instructions are recorded in LastBranchFromIP, LastBranchToIP and these registers are moved into LastExceptionFromIP, LastExceptionToIP when an exception occures.
; For us, the most valuable are LastExceptionFromIP, LastExceptionToIP because they are updated only when an exception occures (usually bug in the program). LastBranchFromIP and LastBranchToIP are updated very frequently (and thus destroyed if you don't manage to catch and read them very early, it is not so easy because simple calling of any procedure to save them changes=destroys these registers just before saving them...).
; Note that debug exception (single step F7, hardware breakpoint, int01) is an "exotic" exception because it disables DebugCtlMSR.LBR, DebugCtlMSR.BTF (as it also disables RFLAGS.TF bit, DR7.GD bit), so in this case (using hardware breakpoints), the most valuable are LastBranchFromIP and LastBranchToIP (and LastExceptionFromIP as well LastExceptionToIP are useless because they aren't updated).
;
; DebugCtlMSR.BTF changes the behavior of the rFLAGS.TF bit. When this bit is cleared to 0 (normal, most common setting), rFLAGS.TF bit controls instruction single stepping (normal behavior). When this bit is set to 1, the rFLAGS.TF bit controls single stepping on control transfers (branch instruction, interrupt, exception) - single step doesn't occur on every instruction, but only on control transfers ("bigger single steps"). By this way the single-step mechanism is allowed to do single step only on control transfers, rather than single step every instruction.
; Debuggers can use this capability to perform a "coarse" single step across blocks of code (bound by control transfers) (DebugCtlMSR.BTF=1, rFLAGS.TF=1), and then, as the problem search is narrowed, switch into a "fine" single-step mode on every instruction (DebugCtlMSR.BTF=0, rFLAGS.TF=1).
; note for beginners, single step in fdbg is Action -> Trace into (F7)


; so, plays for beginners:

; play No. 0:
; 1. Debuggee -> Open executable (Ctrl-E) -> a00.exe
; 2. Control -> Toggle DebugCtlMSR.LBR
; 3. Action -> Run (F9)
; now Log window shows you recorded LastExceptionFromIP=0000000100001026h LastExceptionToIP=0000000100001040h which are memory address some steps before exception occured
; LastBranchFromIP, LastBranchToIP are destroyed very frequently = very soon = at exception handler in OS kernel because these registers changes at every branching instruction, you can see some exotic addresses as FFFFF80001xxxxxx which are addresses in kernel
; 4. go to Log and select value LastExceptionFromIP=xxxxxxxxxxxxxxxxh with mouse (without the "h" terminator !) and press Ctrl+C (copy) - e.g. it is 0000000100001026 in our case
; 5. Debuggee -> Program reset (Ctrl-R)
; 6. Control -> Toggle DebugCtlMSR.LBR
; 7. Action -> Execute To (Alt-F9)
; 8. Ctrl-V (Paste) -> Done
; 9. now you can see LastExceptionFromIP=0000000100001009h, LastExceptionToIP=0000000100001020h
; repeating steps 4-9 you can "trace backward" which is more painful than very easy tracing forward (just pressing F7 or F8)
; note that if you enabled Rambo -> CC collateral damage, then hardware breakpoints are used instead of software breakpoints so you must grab LastBranchFromIP instead of LastExceptionFromIP when triggering hardware breakpoint
; please keep in mind that this methods isn't any ultimate method, e.g. the buggy procedure may be called few times without causing any exception and then with faulty input parameter the procedure makes an exception (bug) at the end.

; play No. 1:
; 1. Debuggee -> Open executable (Ctrl-E) -> a00.exe
; 2. Control -> Toggle DebugCtlMSR.BTF
; 3. Action -> Trace into (F7)
; 4. Action -> Trace into (F7)
; (you can see that steps are "bigger" = occures only on branching instruction, not on every instruction)
; (now we change it back)
; 5. Control -> Toggle DebugCtlMSR.BTF
; (now single steps are "normal" = occures at every instruction)
; 6. Action -> Trace into (F7)
; 7. Action -> Trace into (F7)


; compiled with FASM 1.66

format PE64 GUI at 100000000h
entry start


include '%fasminc%\win64a.inc'


section '.text' code readable executable

start:
	sub	rsp,8*(4+11)

	mov	ecx,2
	call	dement

	add	rsp,8*(4+11)
	xor	eax,eax
	ret

align 16
dement:
	sub	rsp,8*(4+3)

L0:	dec	ecx
	call	retard
	loop	L0

	add	rsp,8*(4+3)
	ret

align 16
retard:
	mov	edx,10h
	xor	eax,eax
;	div	ecx
	mov	eax,[rcx]
	ret
