; antidebug sample:
; the process runs itself for the second instance so 2 identical processes run
; the parent process becomes the debugger of the child process
; proof of concept of hardware breakpoint protection

; what can we do with that?
; 1. Rambo -> IsDebuggerPresent fight must be set ON
; 2. it is possible to debug only parent process, the child process cannot be correctly debugged, because parent debug loop is responsible for decrypting child

; don't be worried, it is possible to debug both parent and child using HDBG which is a project where FDBG is ported to cooperate with hypervisor instead of calling operating system so HDBG is capable to debug simultaneously both parent and child.
; This is because HDBG driver has its own (its own = OS independent)
; - exception interception / handling (before OS is notified about exceptions) / injecting on demand
; - debug handlers (no need to call OS debug APIs, hdbg talks directly with hypervisor)
; - thread_context recording/modifying
; - process memory reading/writing
; - halting thread causing exception
; the above HDBG features allow to debug both parent and child simultaneously, so you are able to see the memory of the child process while parent is decrypting it


MaxProcesses			=	1000h
MaxThreads			=	1000h
if MaxProcesses > MaxThreads
it may work even it is impossible to have more processes than threads
redundant space MaxProcesses-MaxThreads won't be usefull
end if


include	'%fasminc%\win64a.inc'
include	'win64.inc'

format PE64 GUI at (1 shl 32) on 'nul'
entry start


section '.code' code readable executable writeable	; writeable for enabling self-modifying code

start:

a	=	0			; number of pushed qwords
b	=	4			; number of qwords reserved for API
c	=	6			; number of qwords for API params (CreateProcess uses 10 input params)
d	=	(sizeof.PROCESS_INFORMATION64+7)/8	; stack frame in qwords
e	=	(sizeof.STARTUPINFO64+7)/8
f	=	(b+c+d+e) and 1		; align the following structure at 10h
g	=	(sizeof.CONTEXT64+7)/8
z	=	(a+b+c+d+e+f+g+1) and 1	; align stack 10h
	sub	rsp,8*(b+c+d+e+f+g+z)

virtual at rsp+(b+c)*8
ProcessInfo64	PROCESS_INFORMATION64
end virtual

virtual at rsp+(b+c+d)*8
StartupInfo64	STARTUPINFO64
end virtual

virtual at rsp+(b+c+d+e+f)*8
ChildSelfContext64	CONTEXT64
end virtual

; We are going to create one new identical process.
; Both parent and child processes start at the same entrypoint.
; Parent will become the debugger of the child.
; We can differentiate between parent and child by a few methods:
; 1. Run the parent without any command line parameter and then run the child with some command line parameter and differentiate both after processing command line returned by GetCommandLineA.
; 2. Check IsDebuggerPresent API. If the return value=0 then it is the parent, if nonzero then it is the child.
; 3. Enumerating processes (the hardest way).
; 4. If every process owes its window, then we can use FindWindow (very easy to do). None of our processes have any windows, so we can't use this method.

; We use the choice No. 2.

	call	[IsDebuggerPresent]
; Return Value
; If the current process is running in the context of a debugger, the return value is nonzero.
; If the current process is not running in the context of a debugger, the return value is zero.
	or	eax,eax
	jnz	child

; hardcoded in win64:
; db 65h,48h,8Bh,04h,25h,30h,00h,00h,00h	gs mov	rax,[dword 30h]
; db 48h,8Bh,48h,60h				mov	rcx,[rax+60h]
; db 0Fh,0B6h,41h,02h				movzx eax,byte [rcx+02]

;	push	60h
;	pop	rax
;	mov	rax,[gs:rax]
;	cmp	byte [rax+2],0
;	jnz	child

; Even harder and less transparent way, we use a trick - pages are aligned at 1000h, so AL after mov eax,[fs:18h] is 0
;	push	60h
;	pop	rax
;	mov	rcx,[gs:rax]		; AL will be 0 after completing the instruction
;	cmp	[rcx+2],al
;	jnz	child

parent:
; Launch a second copy of the process (= the child), the parent will become the debugger of the child.

	lea	rcx,[StartupInfo64]
	mov	[rcx + STARTUPINFO64.cb],sizeof.STARTUPINFO64
	call	[GetStartupInfoA]

; Obtain our command line so we can launch ourselves again.
	call	[GetCommandLineA]
	push	rax
	pop	rdx

	lea	rax,[ProcessInfo64]
	lea	rcx,[StartupInfo64]
	mov	[rsp+8*(4+5)],rax	; lpProcessInformation 10th argument
	mov	[rsp+8*(4+4)],rcx	; lpStartupInfo 9th argument
	xor	r9,r9			; lpThreadAttributes 4th arg
	xor	ecx,ecx			; lpApplicationName 1st arg
	mov	[rsp+8*(4+3)],r9	; lpCurrentDirectory 8th arg
	mov	[rsp+8*(4+2)],r9	; lpEnvironment 7th arg
	mov	dword [rsp+8*(4+1)],DEBUG_PROCESS; or DEBUG_ONLY_THIS_PROCESS	; dwCreationFlags 6th arg
; DEBUG_ONLY_THIS_PROCESS If not set and the calling process is being debugged, the new process becomes another process being debugged by the calling process's debugger. If the calling process is not a process being debugged, no debugging-related actions occur.
	mov	[rsp+8*(4+0)],cl	; bInheritHandles 5th arg
	xor	r8,r8			; lpProcessAttributes 3rd arg
;	lea	rdx,[rdx]		; lpCommandLine 2nd arg
	call	[CreateProcessA]
	or	eax,eax
	jz	parent_epi		; We don't want to notify hackers about anything... so we just finish without any message.
					; Or is it better to say anything? The above API may fail in normal circumstances too (but extremely rarely).

	call	debug_loop

	mov	rcx,[ProcessInfo64.hThread]
	call	[CloseHandle]
	mov	rcx,[ProcessInfo64.hProcess]
	call	[CloseHandle]

parent_epi:
	add	rsp,8*(b+c+d+e+f+g+z)
	ret


align 10h
debug_loop:

a	=	1			; return address from procedure is currently qword [rsp]
	push	rbx rbp 0		; 0 is counter for number of processes and threads
b	=	3			; number of pushed qwords

c	=	4			; number of qwords reserved for API
d	=	6			; space to call API with 10 input params
e	=	sizeof.DEBUG_EVENT64	; stack frame in bytes
f	=	(e+7)/8			; stack frame in qwords
g	=	(c+d+f) and 1		; align stack 10h, the CONTEXT64 requires to be aligned at dqword
h	=	sizeof.CONTEXT64
i	=	(h+7)/8
z	=	(a+b+c+d+f+g+i) and 1	; align stack 16
	sub	rsp,8*(c+d+f+g+i+z)

virtual at rsp+8*(c+d)
dbg_event	DEBUG_EVENT64
end virtual

virtual at rsp+8*(c+d+f+g)
Context64	CONTEXT64
end virtual

virtual at rsp+8*(c+d+f+g+i+z)
processes_count	dd	?	; it is already 0 thanks to push 0 there
threads_count	dd	?	; it is already 0 thanks to push 0 there
end virtual


align 10h
debug_loop_L0:

if INFINITE = -1
	or	edx,-1
else
	mov	edx,INFINITE
end if
	lea	rcx,[dbg_event]
	call	[WaitForDebugEvent]
; If the function succeeds, the return value is nonzero.
	or	eax,eax
	jz	debug_loop_epi

	mov	eax,[dbg_event.dwDebugEventCode]
	cmp	eax,number_of_DebugEvents
	jnc	debug_loop_epi		; latest RIP_EVENT = 9, but we are counting from 0
					; so we have 10 events, event No. 0 is undefined
	lea	rcx,[DebugEventProceduresTables]
	jmp	qword [rcx+rax*8]

ContinueDebug_DBG_CONTINUE:
	mov	r8d,DBG_CONTINUE
ContinueDebug_ContinueStatus:
	mov	edx,[dbg_event.dwThreadId]
	mov	ecx,[dbg_event.dwProcessId]
	call	[ContinueDebugEvent]
	or	eax,eax
	jz	debug_loop_epi

; Exit if there is no one process or no one thread left
	mov	eax,[processes_count]
	or	eax,eax
	jz	debug_loop_epi		; No one process? Then exit debug loop.
	mov	eax,[threads_count]
	or	eax,eax
	jnz	debug_loop_L0
; No one thread? Then exit debug loop.

;	xor	eax,eax			; exit code = 0 not necessary, eax=0 every time at this address

debug_loop_epi:
	add	rsp,8*(c+d+f+g+i+z+1)	; +1 for remove the push 0 at the prologue
	pop	rbp rbx
	ret

align 10h
Procedure_EXCEPTION_DEBUG_EVENT:
	cmp	[dbg_event.Exception.ExceptionRecord.ExceptionCode],EXCEPTION_SINGLE_STEP
;	jz	Procedure_EXCEPTION_SINGLE_STEP
;	jmp	ContinueDebug_DBG_CONTINUE
	jnz	ContinueDebug_DBG_CONTINUE

;align 10h
;Procedure_EXCEPTION_SINGLE_STEP:

; single step occured (as a result of trap_flag=1 or as a result of hitting debug register(s) breakpoint(s) or as a result of icebp instruction (int01)
; we can determine what happened by examining the DR6 (N.B. more than 1 of the 3 above ways may occure at the same time)

; see documentation about PID2hProcess, nonstandard input and output parameters
	mov	eax,[dbg_event.dwProcessId]
	call	PID2hProcess
	jc	Procedure_EXCEPTION_SINGLE_STEP_epi
	push	rdx
	pop	rbx
; rbx = hProcess

; see documentation about TID2hThread, nonstandard input and output parameters
	mov	eax,[dbg_event.dwThreadId]
	call	TID2hThread
	jc	Procedure_EXCEPTION_SINGLE_STEP_epi
	push	rdx
	pop	rbp
; rbp = hThread

	lea	rdx,[Context64]
	mov	[rdx+CONTEXT64.ContextFlags],CONTEXT_DEBUG_REGISTERS or CONTEXT_CONTROL
; N.B. while on windows 2008 server R2 x64 there is enough to read CONTEXT_DEBUG_REGISTERS, under windows 2003 server x64 both CONTEXT_DEBUG_REGISTERS + CONTEXT_CONTROL must be used to obtain debug registers correctly
;
; http://msdn.microsoft.com/en-us/library/community/history/ms680632%28v=vs.85%29.aspx?id=1
; SetThreadContext Function (Windows)
; Click the links below to view previous versions of "Windows Vista or prior x64 version does mysterious behavior"
; Modified on: 3/16/10 1:36 PM  |  By: GreenCat
; Windows Vista or prior x64 version does mysterious behavior
; Even if CONTEXT_CONTROL is not specified for ContextFlags, SetThreadContext changes the CS Segment register.
; Thread will crash after ResumeThread.
; To evade this problem, even if you are unnecessary, CONTEXT_CONTROL specifies to ContextFlags when GetThreadContext() before call SetThreadContext(), and call SetThreadContext().
; Reference:
; http://d.hatena.ne.jp/egggarden/20091123/1258974861 (Dead link)

	push	rbp
	pop	rcx
	call	[GetThreadContext]
; If the function succeeds, the return value is nonzero.
	or	eax,eax
	jz	Procedure_EXCEPTION_SINGLE_STEP_epi

;; check whether single step occured as a result of enabled flags.TF (Trap Flag)
;if DR6_BS > 7
;	test	byte [Context64.Dr6+1],1 shl (DR6_BS-8)
;	jz	Procedure_EXCEPTION_SINGLE_STEP_finish
;else
;	bt	[Context64.Dr6],DR6_BS
;	jnc	Procedure_EXCEPTION_SINGLE_STEP_finish
;end if

; check whether debug exception occured as a result of debug 0 breakpoint
if DR6_B0 < 8
	test	byte [Context64.Dr6],1 shl DR6_B0
	jz	Procedure_EXCEPTION_SINGLE_STEP_epi
else
	bt	[Context64.Dr6],DR6_B0
	jnc	Procedure_EXCEPTION_SINGLE_STEP_epi
end if

	lea	rax,[rsp+8*(4+1)]
	mov	[rsp+8*(4+0)],rax	; lpNumberOfBytesRead	address of number of bytes read
	mov	r9d,4			; nSize			number of bytes to read
	lea	r8,[rsp+8*(4+2)]	; lpBuffer		address of buffer to place read data
	mov	rdx,[Context64.Dr0]	; lpBaseAddress		address to start reading
	push	rbx
	pop	rcx			; hProcess		handle of the process whose memory is read
	call	[ReadProcessMemory]
; If the function succeeds, the return value is nonzero.
; If the function fails, the return value is zero.
	or	eax,eax
	jz	Procedure_EXCEPTION_SINGLE_STEP_epi
; second check, 4 bytes had to be read
	cmp	dword [rsp+8*(4+1)],4
	jnz	Procedure_EXCEPTION_SINGLE_STEP_epi

; simple decryptor
	rol	dword [rsp+8*(4+2)],21

	lea	rax,[rsp+8*(4+1)]
	mov	[rsp+8*(4+0)],rax	; lpNumberOfBytesWritten actual number of bytes written
	mov	r9d,4			; nSize			number of bytes to write
	lea	r8,[rsp+8*(4+2)]	; lpBuffer		pointer to buffer to write data to (address of buffer for writing a memory of the child, it holds the new dword to be replaced at the old value of the dword)
	mov	rdx,[Context64.Dr0]	; lpBaseAddress		address to start writing to
	push	rbx
	pop	rcx			; hProcess		handle to process whose memory is written to
	call	[WriteProcessMemory]
; If the function succeeds, the return value is nonzero.
; If the function fails, the return value is zero.
	or	eax,eax
	jz	Procedure_EXCEPTION_SINGLE_STEP_epi
; second check, 4 bytes had to be written
	cmp	dword [rsp+8*(4+1)],4
	jnz	Procedure_EXCEPTION_SINGLE_STEP_epi

Procedure_EXCEPTION_SINGLE_STEP_finish:
	lea	rdx,[Context64]
	mov	[rdx+CONTEXT64.ContextFlags],CONTEXT_DEBUG_REGISTERS
; clear DR6 register
	and	[rdx+CONTEXT64.Dr6],not ( (1 shl DR6_BS) + (1 shl DR6_BD) + (1 shl DR6_B3) + (1 shl DR6_B2) + (1 shl DR6_B1) + (1 shl DR6_B0) )
	add	[rdx+CONTEXT64.Dr0],4	; point to next dword so int01 will hit until repz lodsd ends
	push	rbp
	pop	rcx
	call	[SetThreadContext]
; If the function succeeds, the return value is nonzero.
	or	eax,eax
	jz	Procedure_EXCEPTION_SINGLE_STEP_epi

Procedure_EXCEPTION_SINGLE_STEP_epi:
	jmp	ContinueDebug_DBG_CONTINUE


align 10h
Procedure_CREATE_THREAD_DEBUG_EVENT:
	inc	[threads_count]
	mov	eax,[dbg_event.dwThreadId]
	mov	rdx,[dbg_event.CreateThread.hThread]
	call	store_TID_handle
	mov	eax,[dbg_event.dwProcessId]
	mov	edx,[dbg_event.dwThreadId]
	call	store_PID_TID_pair
	jmp	ContinueDebug_DBG_CONTINUE

align 10h
Procedure_CREATE_PROCESS_DEBUG_EVENT:
	inc	[processes_count]
	inc	[threads_count]			; creating process also creates 1 thread
	mov	eax,[dbg_event.dwProcessId]
	mov	rdx,[dbg_event.CreateProcess.hProcess]
	call	store_PID_handle
	mov	eax,[dbg_event.dwThreadId]
	mov	rdx,[dbg_event.CreateProcess.hThread]
	call	store_TID_handle
	mov	eax,[dbg_event.dwProcessId]
	mov	edx,[dbg_event.dwThreadId]
	call	store_PID_TID_pair
; hFile When the debugger is finished with this file, it should close the handle using the CloseHandle function.
	mov	rcx,[dbg_event.CreateProcess.hFile]
	jrcxz	Procedure_CREATE_PROCESS_DEBUG_EVENT_end
	call	[CloseHandle]
Procedure_CREATE_PROCESS_DEBUG_EVENT_end:
	jmp	ContinueDebug_DBG_CONTINUE

align 10h
Procedure_EXIT_THREAD_DEBUG_EVENT:
	dec	[threads_count]
	mov	eax,[dbg_event.dwThreadId]
	call	remove_TID_handle
;	mov	rcx,rax
;	call	[CloseHandle]
; wrong, ContinueDebugEvent will close the handle (else ContinueDebugEvent will raise EXCEPTION_INVALID_HANDLE)
	mov	eax,[dbg_event.dwProcessId]
	mov	edx,[dbg_event.dwThreadId]
	call	remove_PID_TID_pair
	jmp	ContinueDebug_DBG_CONTINUE

align 10h
Procedure_EXIT_PROCESS_DEBUG_EVENT:
	dec	[processes_count]
	dec	[threads_count]
	mov	eax,[dbg_event.dwProcessId]
	call	remove_PID_handle
;	mov	rcx,rax
;	call	[CloseHandle]
; wrong, ContinueDebugEvent will close the handle (else ContinueDebugEvent will raise EXCEPTION_INVALID_HANDLE)
	mov	eax,[dbg_event.dwThreadId]
	call	remove_TID_handle
;	mov	rcx,rax
;	call	[CloseHandle]
; wrong, ContinueDebugEvent will close the handle (else ContinueDebugEvent will raise EXCEPTION_INVALID_HANDLE)
	mov	eax,[dbg_event.dwProcessId]
	mov	edx,[dbg_event.dwThreadId]
	call	remove_PID_TID_pair
	jmp	ContinueDebug_DBG_CONTINUE

align 10h
Procedure_LOAD_DLL_DEBUG_EVENT:
; hFile When the debugger is finished with this file, it should close the handle using the CloseHandle function.
	mov	rcx,[dbg_event.LoadDll.hFile]
	jrcxz	Procedure_LOAD_DLL_DEBUG_EVENT_end
	call	[CloseHandle]
Procedure_LOAD_DLL_DEBUG_EVENT_end:


;	jmp	ContinueDebug_DBG_CONTINUE

align 10h
Procedure_UNLOAD_DLL_DEBUG_EVENT:
;	jmp	ContinueDebug_DBG_CONTINUE

align 10h
Procedure_OUTPUT_DEBUG_STRING_EVENT:
;	jmp	ContinueDebug_DBG_CONTINUE

align 10h
Procedure_RIP_EVENT:
;	jmp	ContinueDebug_DBG_CONTINUE

align 10h
Procedure_none:
	jmp	ContinueDebug_DBG_CONTINUE

align 10h
store_PID_handle:
; input: RAX ProcessID RDX ProcessHandle
	push	rdi rcx rax
	lea	rdi,[proc_thrd_data.proc_handle]
	xor	eax,eax
	mov	ecx,MaxProcesses
	cld
	repnz scasq
	stc
	jnz	store_PID_handle_end
	mov	rax,[rsp+8*0]
	mov	[rdi-8],rax
	mov	[rdi-8 + MaxProcesses*8],rdx
	clc
store_PID_handle_end:
	pop	rax rcx rdi
	ret

align 10h
remove_PID_handle:
	push	rdi rcx
	lea	rdi,[proc_thrd_data.proc_handle]
	mov	ecx,MaxProcesses
	cld
	repnz scasq
	stc
	jnz	remove_PID_handle_end
	xor	ecx,ecx
;	mov	rax,[rdi-8 + MaxProcesses*8]
	mov	[rdi-8],rcx
	mov	[rdi-8 + MaxProcesses*8],rcx
	clc
remove_PID_handle_end:
	pop	rcx rdi
	ret

align 10h
store_TID_handle:
; in:	RAX ThreadID
;	RDX ThreadHandle
	push	rdi rcx rax
	lea	rdi,[proc_thrd_data.thrd_handle]
	xor	eax,eax
	mov	ecx,MaxThreads
	cld
	repnz scasq
	stc
	jnz	store_TID_handle_end
	mov	rax,[rsp+8*0]
	mov	[rdi-8],rax
	mov	[rdi-8 + MaxThreads*8],rdx
	clc
store_TID_handle_end:
	pop	rax rcx rdi
	ret

align 10h
remove_TID_handle:
; out: success CF=0 RAX=hThread
;      failure CF=1
	push	rdi rcx
	lea	rdi,[proc_thrd_data.thrd_handle]
	mov	ecx,MaxThreads
	cld
	repnz scasq
	stc
	jnz	remove_TID_handle_end
	xor	ecx,ecx
;	mov	rax,[rdi-8 + MaxThreads*8]	; get hThread
	mov	[rdi-8],rcx
	mov	[rdi-8 + MaxThreads*8],rcx
	clc
remove_TID_handle_end:
	pop	rcx rdi
	ret

align 10h
store_PID_TID_pair:
; in:	EAX ProcessID
;	EDX ThreadId
	push	rdi rcx rax
	lea	rdi,[proc_thrd_data.pid_tid]
	xor	eax,eax
a = MaxThreads
if a < MaxProcesses
a = MaxProcesses
end if
	mov	ecx,a
	cld
	repnz scasd
	stc
	jnz	store_PID_TID_pair_end
	mov	eax,[rsp+8*0]
	mov	[rdi-4],eax
	mov	[rdi-4 + a*4],edx
	clc
store_PID_TID_pair_end:
	pop	rax rcx rdi
	ret

align 10h
remove_PID_TID_pair:
; in:	EAX ProcessID
;	EDX ThreadId
; it is easier to scan TIDs because the same PID should have more TIDs, but every TID is unique
	push	rdi rcx
	lea	rdi,[proc_thrd_data.pid_tid + a*4]
	xchg	edx,eax
	mov	ecx,MaxThreads
	cld
	repnz scasd
	stc
	jnz	remove_PID_TID_pair_end
	cmp	[rdi-4 - a*4],edx
	stc
	jnz	remove_PID_TID_pair_end
	xor	ecx,ecx
	mov	[rdi-4],ecx
	mov	[rdi-4 - a*4],ecx
	clc
remove_PID_TID_pair_end:
	xchg	edx,eax
	pop	rcx rdi
	ret

align 4
PID2hProcess:
; in:	EAX ProcessId
; out:	RDX hProcess
; note, this can be done by kernel32.OpenProcess, but then we need extra CloseHandle...
; note, invert operation is kernel32.GetProcessId
	push	rdi rcx
	lea	rdi,[proc_thrd_data.proc_handle]
	mov	ecx,MaxProcesses
	cld
	repnz scasq
	stc
	jnz	PID2hProcess_end
	mov	rdx,[rdi-8 + MaxProcesses*8]
	clc
PID2hProcess_end:
	pop	rcx rdi
	ret

align 4
TID2hThread:
; in:	EAX ThreadId
; out:	RDX hThread
; note, this can be done by kernel32.OpenThread, but then we need extra CloseHandle...
; note, invert operation is kernel32.GetThreadId
; The GetThreadId function retrieves the thread identifier of the specified thread.
;
; DWORD GetThreadId(
;   HANDLE Thread
; );
;
; Parameters:
; Thread 
; [in] Handle to the thread. The handle must have the THREAD_QUERY_INFORMATION access right. For more ; information about access rights, see Thread Security and Access Rights. 
; Return Values:
; If the function succeeds, the return value is the thread identifier of the specified thread.
; If the function fails, the return value is zero. To get extended error information, call GetLastError.

	push	rdi rcx
	lea	rdi,[proc_thrd_data.thrd_handle]
	mov	ecx,MaxThreads
	cld
	repnz scasq
	stc
	jnz	TID2hThread_end
	mov	rdx,[rdi-8 + MaxThreads*8]
	clc
TID2hThread_end:
	pop	rcx rdi
	ret

; +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

align 10h
child:
	xor	r9,r9			; lpParameter. We don't need it.
	lea	r8,[ThrdStart]		; lpStartAddress
	xor	edx,edx			; dwStackSize. If 0 is specified, the stack size defaults to the same size as that of the primary thread of the process.
	xor	ecx,ecx			; lpThreadAttributes.Pointer to a SECURITY_ATTRIBUTES structure that determines whether the returned handle can be inherited by child processes. If lpThreadAttributes is NULL, the handle cannot be inherited.
	mov	[rsp+8*(4+1)],rdx	; lpThreadId. We don't need it, debug loop gets PID.
	mov	dword [rsp+8*(4+0)],CREATE_SUSPENDED	; dwCreationFlags. If the CREATE_SUSPENDED flag is specified, the thread is created in a suspended state, and will not run until the ResumeThread function is called. If this value is zero, the thread runs immediately after creation.
	call	[CreateThread]
; If the function succeeds, the return value is a handle to the new thread.
; If the function fails, the return value is NULL.
	or	rax,rax
	mov	[rsp+8*(4+0)],rax
	jz	child_epi
; N.B. we created a thread to modify its registers, if the thread is running the behaviour of modifying its registers is unpredicable
; fortunately the thread starts to run only after ResumeThread and then WaitForSingleObject, the thread is currently not running so we can change its registers

	lea	rdx,[ChildSelfContext64]
	mov	[rdx+CONTEXT64.ContextFlags],CONTEXT_ALL
	push	rax
	pop	rcx
	call	[GetThreadContext]
; If the function succeeds, the return value is nonzero.
	or	eax,eax
	jz	child_epi

; set hardware breakpoint (repz lodsd will trigger it and address will be increased by 4 every hw bp hit)
	lea	rdx,[ChildSelfContext64]
	mov	[rdx+CONTEXT64.ContextFlags],CONTEXT_ALL
; N.B. while on windows 2008 server R2 x64 there is enough to read CONTEXT_DEBUG_REGISTERS, under windows 2003 server x64 both CONTEXT_ALL must be used to write debug registers correctly (hence it was necessary to obtain ALL registers before writing them back with updated debug registers)
;
; http://msdn.microsoft.com/en-us/library/community/history/ms680632%28v=vs.85%29.aspx?id=1
; SetThreadContext Function (Windows)
; Click the links below to view previous versions of "Windows Vista or prior x64 version does mysterious behavior"
; Modified on: 3/16/10 1:36 PM  |  By: GreenCat
; Windows Vista or prior x64 version does mysterious behavior
; Even if CONTEXT_CONTROL is not specified for ContextFlags, SetThreadContext changes the CS Segment register.
; Thread will crash after ResumeThread.
; To evade this problem, even if you are unnecessary, CONTEXT_CONTROL specifies to ContextFlags when GetThreadContext() before call SetThreadContext(), and call SetThreadContext().
; Reference:
; http://d.hatena.ne.jp/egggarden/20091123/1258974861 (Dead link)

	mov	[rdx+CONTEXT64.Dr7],000F0401h	; LEN0=11b (for 4 bytes size, address must be dword aligned), R/W0=11b (data read or data write), L0=1
; bits 15, 14 set to 0, bits 12, 11 set to 0
; bit 10 set to 1

if (crypted_start and 11b) <> 0
Hey dude, you must align crypted_start at dword boundary
end if
	lea	rcx,[crypted_start]
	mov	[rdx+CONTEXT64.Dr0],rcx
;	and	[rdx+CONTEXT64.Dr1],0
;	and	[rdx+CONTEXT64.Dr2],0
;	and	[rdx+CONTEXT64.Dr3],0
	mov	rcx,[rsp+8*(4+0)]
	call	[SetThreadContext]
; If the function succeeds, the return value is nonzero.
	or	eax,eax
	jz	child_epi

	mov	rcx,[rsp+8*(4+0)]
	call	[ResumeThread]

	mov	edx,INFINITE
	mov	rcx,[rsp+8*(4+0)]
	call	[WaitForSingleObject]
; If the function succeeds, the return value indicates the event that caused the function to return.
; If the function fails, the return value is WAIT_FAILED.

child_epi:
	jmp	parent_epi

align 10h
ThrdStart:
	push	rsi
	sub	rsp,8*(4+4)

	lea	rsi,[crypted_start]
	mov	ecx,crypted_size / 4
	cld
	repz lodsd
; note, there is a small risk that encrypted dword = 0 and in this case repnz scasd terminates prematurely
; This is the reason why we use rol/ror dword,21 which is able to produce encrypted dword 0 only from original dword 0 (it is able to produce result 0 from nothing else than input 0)
; the instruction repz lodsd will generate #DB, which is handled by parent, memory of child is decrypted

align 10h
crypted_start:
;	and	qword [rsp+8*(4+2)],0
	mov	dword [rsp+8*(4+1)],FILE_ATTRIBUTE_NORMAL
	mov	dword [rsp+8*(4+0)],CREATE_ALWAYS
	xor	r9,r9
	xor	r8,r8
	mov	edx,GENERIC_WRITE
	lea	rcx,[filename]
	mov	[rsp+8*(4+2)],r9
	call	[CreateFileA]
	mov	[rsp+8*(4+3)],rax
	push	rax
	pop	rcx
if INVALID_HANDLE_VALUE = -1
	inc	rax
else
	cmp	rax,INVALID_HANDLE_VALUE
end if
	jz	thrd_epi

	and	qword [rsp+8*(4+0)],0
	lea	r9,[rsp+8*(4+1)]
	mov	r8d,filesize
	lea	rdx,[filecontent]
;	mov	rcx,rcx
	call	[WriteFile]
;	or	eax,eax
;	jz	child_close
;	cmp	dword [rsp+8*(4+1)],filesize
;	jnz	child_close

; xor r9,r9
; xor r8,r8
; lea rdx,[msg_success]
; xor ecx,ecx
; call [MessageBoxA]

child_close:
	mov	rcx,[rsp+8*(4+3)]
	call	[CloseHandle]

thrd_epi:
	add	rsp,8*(4+4)
	pop	rsi
	ret

filename	db	'dement.txt',0

filecontent	db	'you were too slow, some files could alredy be infected'
filesize	=	$ - filecontent

virtual
align 4
paddwordzero = $ - $$
end virtual
if paddwordzero <> 0
db paddwordzero dup 0
end if

crypted_size	=	$ - crypted_start

repeat ((crypted_size+3)/4)
load a dword from (crypted_start+(%-1)*4)
store dword ((a shr 21) or (a shl (32-21)) and 0FFFFFFFFh) at (crypted_start+(%-1)*4)
end repeat


section '.data' readable writeable
DebugEventProceduresTables:
			dq	Procedure_none
			dq	Procedure_EXCEPTION_DEBUG_EVENT
			dq	Procedure_CREATE_THREAD_DEBUG_EVENT
			dq	Procedure_CREATE_PROCESS_DEBUG_EVENT
			dq	Procedure_EXIT_THREAD_DEBUG_EVENT
			dq	Procedure_EXIT_PROCESS_DEBUG_EVENT
			dq	Procedure_LOAD_DLL_DEBUG_EVENT
			dq	Procedure_UNLOAD_DLL_DEBUG_EVENT
			dq	Procedure_OUTPUT_DEBUG_STRING_EVENT
			dq	Procedure_RIP_EVENT
number_of_DebugEvents	=	($ - DebugEventProceduresTables)/8
times (EXCEPTION_MAXIMUM_PARAMETERS - RIP_EVENT) dq Procedure_none


a = MaxThreads
if a < MaxProcesses
a = MaxProcesses
end if
proc_thrd_data:
.proc_handle		rq	MaxProcesses*2	; store PID...hProcess
.thrd_handle		rq	MaxThreads*2	; TID...hThread
.pid_tid		rd	a*2		; PID...TID


section '.idata' import data readable writeable
			dd	0,0,0,	RVA kernel_name,	RVA kernel_table
;			dd	0,0,0,	RVA user_name,		RVA user_table
			dd	0,0,0,	0,			0

align 8
kernel_table:
CloseHandle		dq	RVA _CloseHandle
ContinueDebugEvent	dq	RVA _ContinueDebugEvent
CreateFileA		dq	RVA _CreateFileA
CreateProcessA		dq	RVA _CreateProcessA
CreateThread		dq	RVA _CreateThread
GetCommandLineA		dq	RVA _GetCommandLineA
GetStartupInfoA		dq	RVA _GetStartupInfoA
GetThreadContext	dq	RVA _GetThreadContext
IsDebuggerPresent	dq	RVA _IsDebuggerPresent
ReadProcessMemory	dq	RVA _ReadProcessMemory
ResumeThread		dq	RVA _ResumeThread
SetThreadContext	dq	RVA _SetThreadContext
WaitForDebugEvent	dq	RVA _WaitForDebugEvent
WaitForSingleObject	dq	RVA _WaitForSingleObject
WriteFile		dq	RVA _WriteFile
WriteProcessMemory	dq	RVA _WriteProcessMemory
			dq	0

;;user_table:
;MessageBoxA		dq	RVA _MessageBoxA
;			dq	0

; kernel32.dll
_CloseHandle		db	0,0,'CloseHandle',0
_ContinueDebugEvent	db	0,0,'ContinueDebugEvent',0
_CreateFileA		db	0,0,'CreateFileA',0
_CreateProcessA		db	0,0,'CreateProcessA',0
_CreateThread		db	0,0,'CreateThread',0
_GetCommandLineA	db	0,0,'GetCommandLineA',0
_GetStartupInfoA	db	0,0,'GetStartupInfoA',0
_GetThreadContext	db	0,0,'GetThreadContext',0
_IsDebuggerPresent	db	0,0,'IsDebuggerPresent',0
_ReadProcessMemory	db	0,0,'ReadProcessMemory',0
_ResumeThread		db	0,0,'ResumeThread',0
_SetThreadContext	db	0,0,'SetThreadContext',0
_WaitForDebugEvent	db	0,0,'WaitForDebugEvent',0
_WaitForSingleObject	db	0,0,'WaitForSingleObject',0
_WriteFile		db	0,0,'WriteFile',0
_WriteProcessMemory	db	0,0,'WriteProcessMemory',0
;; user32.dll
;_MessageBoxA		db	0,0,'MessageBoxA',0

kernel_name		db	'KERNEL32.DLL',0
;user_name		db	'USER32.DLL',0
