align 16
disassemble_one_instruction:
;input:	rsi point to opcodes
;	rbp address of opcodes in debuggee
;	rdi buffer, max size = disasm_buf
;output:rax opcode size (fits into InstructionMaxSize)
;	fill memory at rdi with disassembled instruction
;	CF=1 if error (instruction_size > InstructionMaxSize)
;
; subroutine doesn't use API, we needn't align16 stack
	push	rcx
	push	rdx
	push	rbx
	push	rsi
	push	rdi

	xor	eax,eax
	cld
	mov	qword [disassemble_prefix],rax
	mov	qword [absolute_RIP_addressing_value],rax
	mov	qword [absolute_RIP_addressing_disassembled_position],rax
;	mov	byte [post_disasm_flag],al
;	mov	byte [post_disasm_action],al
;	mov	byte [post_disasm_mul],al
;	mov	byte [post_disasm_prefix],al
;	mov	byte [seg_prefix_not_flushed],al
	mov	qword [post_disasm_flag],rax	; above 5 in 1 action
						; fivefold action, satisfaction...

	lea	rbx,[disassemble_table_0]
	movzx	eax,byte [rsi]			; grab the first byte of instruction opcode
	call	qword [rbx+rax*8]		; call disasm routine for this byte

	mov	rdi,[absolute_RIP_addressing_disassembled_position]
	mov	rdx,qword [absolute_RIP_addressing_value]
	mov	rcx,rdi
	or	rcx,rdx
	jz	absolute_RIP_addressing_extra_calculation_done
; we calculate address of absolute RIP addressing - we have opcode size in RAX
	push	rax
	add	rax,rdx
	add	rax,rbp
	or	byte [post_disasm_flag],post_disasm_flag_addr0
	mov	qword [post_disasm_addr0],rax
	call	hexa_64_rax
	pop	rax
absolute_RIP_addressing_extra_calculation_done:

; this shouldn't happen, but to be sure...
	mov	ecx,InstructionMaxSize
	cmp	ecx,eax
	cmovc	eax,ecx

	pop	rdi
	pop	rsi
	pop	rbx
	pop	rdx
	pop	rcx
	ret

process_undefined_opcode_begin_string:
	push	rax
	mov	rax,'illegal '
	stosq
	pop	rax
	ret

align 10h
check_lahf_sahf:
; check whether LAHF / SAHF supported
; in:	none
; out:	[supported_lahf_sahf]=1 LAHF/SAHF supported
;	[supported_lahf_sahf]=0 LAHF/SAHF are illegal
	push	rax rcx rdx rbx
	mov	eax,80000000h
	cpuid				; destroys RBX !!!
	cmp	eax,80000001h
	jc	check_lahf_sahf_epi	; we can not cpuid with eax=80000001h
	mov	eax,80000001h
	cpuid				; Software can determine a processors physical-address width by executing CPUID with 80000008H in EAX. The 
; ECX bit 0 returned by CPUID function 8000_0001h to verify that the processor supports LAHF / SAHF in 64-bit mode
	shr	ecx,1
	cmc				; complement Carry Flag
check_lahf_sahf_epi:
	setnc	[supported_lahf_sahf]
	pop	rbx rdx rcx rax
	ret

align 16
search_disassemble_prefix_REX:
; return in rdx REX prefix if there is any, else Carry Flag and rdx=0
;
; The REX prefix must immediately precede the opcode byte(s).
; bit    name   description  
; 7..4   n/a    0100b (yields 40..4Fh range)  
; 3      W      0 = default operand size
;               1 = 64bit operand size   
; 2      R      mod R/M byte reg field extension  
; 1      X      SIB byte index field extension  
; 0      B      mod R/M byte r/m field extension
;               SIB byte base field extension
;               opcode byte reg field extension   
;
	push	rcx
	mov	edx,dword [disassemble_prefix]	; grab maximum 4 prefix bytes allowed by AMD64
	bsr	ecx,edx
	and	cl,11111000b
	shr	edx,cl
	pop	rcx
	or	edx,edx
	jz	search_disassemble_prefix_REX_not_present_carry_ret
	test	dl,01000000b
	jz	search_disassemble_prefix_REX_not_present
	test	dl,10110000b
	jnz	search_disassemble_prefix_REX_not_present
	clc
	ret
search_disassemble_prefix_REX_not_present:
	xor	edx,edx
search_disassemble_prefix_REX_not_present_carry_ret:
	stc
	ret

align 16
search_disassemble_prefix_66:
; return Zero Flag if 66h prefix found
	push	rax
	mov	eax,dword [disassemble_prefix]
	cmp	al,66h
	jz	search_disassemble_prefix_66_end
	cmp	ah,66h
	jz	search_disassemble_prefix_66_end
	shr	eax,16
	cmp	al,66h
	jz	search_disassemble_prefix_66_end
	cmp	ah,66h
search_disassemble_prefix_66_end:
	pop	rax
	ret

align 16
search_disassemble_prefix_67:
; return Zero Flag if 67h prefix found
	push	rax
	mov	eax,dword [disassemble_prefix]
	cmp	al,67h
	jz	search_disassemble_prefix_67_end
	cmp	ah,67h
	jz	search_disassemble_prefix_67_end
	shr	eax,16
	cmp	al,67h
	jz	search_disassemble_prefix_67_end
	cmp	ah,67h
search_disassemble_prefix_67_end:
	pop	rax
	ret

align 16
search_disassemble_prefix_F2:
; return Zero Flag if F2h prefix found
	push	rax
	mov	eax,dword [disassemble_prefix]
	cmp	al,0F2h
	jz	search_disassemble_prefix_F2_end
	cmp	ah,0F2h
	jz	search_disassemble_prefix_F2_end
	shr	eax,16
	cmp	al,0F2h
	jz	search_disassemble_prefix_F2_end
	cmp	ah,0F2h
search_disassemble_prefix_F2_end:
	pop	rax
	ret

align 16
search_disassemble_prefix_F3:
; return Zero Flag if F3h prefix found
	push	rax
	mov	eax,dword [disassemble_prefix]
	cmp	al,0F3h
	jz	search_disassemble_prefix_F3_end
	cmp	ah,0F3h
	jz	search_disassemble_prefix_F3_end
	shr	eax,16
	cmp	al,0F3h
	jz	search_disassemble_prefix_F3_end
	cmp	ah,0F3h
search_disassemble_prefix_F3_end:
	pop	rax
	ret

align 16
find_first_no_prefix_byte:
; returns CF=0 and the first no prefix byte in DL
; returns CF=1 if there isn't any prefix byte in the instruction
	xor	ecx,ecx
AMDPad16
find_first_no_prefix_byte_begin:
	inc	ecx
	cmp	cl,InstructionMaxPrefixSize
	jnbe	find_first_no_prefix_invalid_instruction
	mov	dl,byte [rsi+rcx*1]
	cmp	dl,66h
	jz	find_first_no_prefix_byte_begin
	cmp	dl,67h
	jz	find_first_no_prefix_byte_begin
	cmp	dl,0F2h
	jz	find_first_no_prefix_byte_begin
	cmp	dl,0F3h
	jz	find_first_no_prefix_byte_begin
	cmp	dl,40h
	jc	find_first_no_prefix_have_it
	cmp	dl,50h
	jc	find_first_no_prefix_byte_begin
find_first_no_prefix_have_it:
	clc
	ret
find_first_no_prefix_invalid_instruction:
	stc
	ret

; 06 PUSH ES undefined in AMD64 long mode
process_undefined_opcode_push_es:
	mov	rax,'push es'
	jmp	process_opcode_common_end_undefined_push_pop_seg

; 07 POP ES undefined in AMD64 long mode
process_undefined_opcode_pop_es:
	mov	rax,'pop es'
	jmp	process_opcode_common_end_undefined_push_pop_seg

; 0E PUSH CS undefined in AMD64 long mode
process_undefined_opcode_push_cs:
	mov	rax,'push cs'
	jmp	process_opcode_common_end_undefined_push_pop_seg

; 16 PUSH SS undefined in AMD64 long mode
process_undefined_opcode_push_ss:
	mov	rax,'push ss'
	jmp	process_opcode_common_end_undefined_push_pop_seg

; 17 POP SS undefined in AMD64 long mode
process_undefined_opcode_pop_ss:
	mov	rax,'pop ss'
	jmp	process_opcode_common_end_undefined_push_pop_seg

; 1E PUSH DS undefined in AMD64 long mode
process_undefined_opcode_push_ds:
	mov	rax,'push ds'
	jmp	process_opcode_common_end_undefined_push_pop_seg

; 1F POP DS undefined in AMD64 long mode
process_undefined_opcode_pop_ds:
	mov	rax,'pop ds'
process_opcode_common_end_undefined_push_pop_seg:
	call	process_undefined_opcode_begin_string
	jmp	process_opcode_common_end_55

; 00-05 ADD
process_opcode_add_0:
; 08-0D OR
process_opcode_or_0:
; 10-15 ADC
process_opcode_adc_0:
; 18-1D SBB
process_opcode_sbb_0:
; 20-25 AND
process_opcode_and_0:
; 28-2D SUB
process_opcode_sub_0:
; 30-35 XOR
process_opcode_xor_0:
; 38-3D CMP
process_opcode_cmp_0:
; first store instruction (add or adc sbb and sub xor cmp) followed by one space
	mov	ecx,eax
	shr	eax,3
	lea	rbx,[process_opcode_add_or_adc_sbb_and_sub_xor_cmp_table]
	mov	eax,[rbx+rax*4]
	stosd
	xor	eax,eax
	cmp	byte [rdi-2],' '	; in case of 'or  ' string we decrease 2 spaces to 1
	setz	al
	sub	rdi,rax			; subtract 1 in case of 'or  ' string, else subtract 0

	bt	ecx,2
	jc	process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator
	and	cl,11b
	lea	rbx,[process_opcode_big_fucking_gun_table]
	call	qword [rbx+rcx*8]
	ret

; instructions begin byte 04 05 0C 0D 14 15 1C...
; note that 06 07 0E 0F 16 17 1E... are handled above, not here
process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator:
; 0445          add al,45
; 6605457E      add ax,7E45
; 0532010000    add eax,00000132
; 480578063012  add rax,12300678
	bt	ecx,0
	jnc	process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator_byte
	call	search_disassemble_prefix_66
	jz	process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator_word
process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator_dword:
process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator_qword:
	mov	eax,'eax,'
	mov	ecx,'rax,'
	call	search_disassemble_prefix_REX
	jc	process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator_dword_qword_L3
	bt	edx,3		; 64bit operand size
	cmovc	eax,ecx
process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator_dword_qword_L3:
	stosd
	mov	eax,dword [rsi+1]
	shl	rax,32
	call	hexa_64_rax
	mov	byte [rdi+8],0
	mov	eax,5		; opcode size
	ret

process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator_word:
	mov	eax,'ax,'
	stosd
	dec	rdi
	mov	eax,dword [rsi+1]
	shl	rax,32 + 16
	call	hexa_64_rax
	mov	byte [rdi+4],0
	mov	eax,3		; opcode size
	ret

process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator_byte:
	mov	eax,'al,'
	stosd
	dec	rdi
	mov	al,byte [rsi+1]
	shl	rax,32+16+8
	call	hexa_64_rax
	mov	byte [rdi+2],0
	mov	eax,2		; opcode size
	ret

process_opcode_big_fucking_gun:

process_opcode_big_fucking_gun_mem_reg_b:
	mov	byte [post_disasm_action],post_disasm_byte

; 0000 - 003F			0000		add [rax],al
; 004000 - 007FFF		004060		add [rax+60],al
; 008000000000 - 00BFFFFFFFFF	008090230010	add [rax+10002390],al
; 00C0 - 00FF			00C0		add al,al
	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table]
	call	qword [rbx+rax*8]
;0000           add [rax],al
;004012         add [rax+12],al
;008065FE0501   add [rax+0105FE65],al
;00C0           add al,al

	push	rax		; opcode size

	mov	al,','
	stosb

	call	process_opcode_big_fucking_gun_reg_muthafucker
	mov	byte [rdi],0

	pop	rax		; opcode size
	ret

process_opcode_big_fucking_gun_mem_reg_d:
	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
	call	qword [rbx+rax*8]
;0000           add [rax],al
;004012         add [rax+12],al
;008065FE0501   add [rax+0105FE65],al
;00C0           add al,al

	push	rax		; opcode size

	mov	al,','
	stosb

	call	process_opcode_big_fucking_gun_reg_muthafucker_wdq
	mov	byte [rdi],0

	pop	rax		; opcode size
	ret

process_opcode_big_fucking_gun_reg_mem_b:
	mov	byte [post_disasm_action],post_disasm_byte
	call	process_opcode_big_fucking_gun_reg_muthafucker

	mov	al,','
	stosb

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table]
	call	qword [rbx+rax*8]
; note that function return opcode size in RAX
	mov	byte [rdi],0
	ret

process_opcode_big_fucking_gun_reg_mem_d:
	call	process_opcode_big_fucking_gun_reg_muthafucker_wdq

	mov	al,','
	stosb

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
	call	qword [rbx+rax*8]
; note that function return opcode size in RAX
	mov	byte [rdi],0
	ret

process_opcode_big_fucking_gun_mem_memb_memd_reg:
process_opcode_big_fucking_gun_mem:
; 0000          add [rax],al
; 000430        add [rsi+rax],al
; 000530000000  add [rip+00000030],al
; 0006          add [rsi],al
; 0008          add [rax],cl
; 003F          add [rdi],bh
; 670000        add [eax],al
	mov	al,'['
	stosb
	call	process_opcode_big_fucking_gun_mem_universal
; function return opcode size in RAX
	mov	byte [rdi],']'
	scasb
	mov	byte [rdi],0	; don't increase RDI !!! This allow to continue e.g. by imul (6b240467 imul esp,[rsp+rax*1],67)
	ret

process_opcode_big_fucking_gun_memb:
; 004030    add [rax+30],al
; 00443040  add [rax+rsi+40],al
; 00446840  add [rax+rbp*2+40],al
; 004568    add [rbp+68],al
; 007F68    add [rdi+68],bh

	mov	al,'['
	stosb
	call	process_opcode_big_fucking_gun_mem_universal
; function return opcode size in RAX
	push	rax
;	mov	al,'+'
;	stosb
	mov	byte [rdi],'+'
	scasb
	movsx	rax,byte [rsi+rax*1-1]
	or	byte [post_disasm_flag],post_disasm_flag_addr0
	mov	qword [post_disasm_addr0],rax
;
;0044c07b         add     [rax+rax*8+7b],al
;
	shl	rax,32+16+8
	call	hexa_64_rax_sign
	add	rdi,2
	mov	al,']'
	stosb
	mov	byte [rdi],0	; don't increase RDI
	pop	rax
	ret

process_opcode_big_fucking_gun_memd:
; 008068400000   add [rax+00004068],al
; 00BF68400010   add [rdi+10004068],bh
; 008480c10f0000 add [rax+rax*4+00000fc1],al
	mov	al,'['
	stosb
	call	process_opcode_big_fucking_gun_mem_universal
; function return opcode size in RAX
	push	rax

	mov	byte [rdi],'+'
	scasb

	movsxd	rax,dword [rsi+rax*1-4]
	or	byte [post_disasm_flag],post_disasm_flag_addr0
	mov	qword [post_disasm_addr0],rax
	shl	rax,32
	call	hexa_64_rax_sign
	add	rdi,8
	mov	al,']'
	stosb

	mov	byte [rdi],0	; don't increase RDI

	pop	rax
	ret

process_opcode_big_fucking_gun_reg:
	movzx	eax,byte [rsi+1]
	and	al,111b
;
; 00c0             add al,al
; 4000c2           add dl,al
; 4000c6           add sil,al
; 4100c1           add r9b,al
; 00FF             add bh,bh
;
; gpr_8           al cl dl bl ah ch dh bh
; gpr_8_old_ext   al cl dl bl spl bpl sil dil
; gpr_8_new       r8b r9b r10b r11b r12b r13b r14b r15b
;
process_opcode_big_fucking_gun_reg_common_begin:
	lea	rbx,[gpr_8]
	call	search_disassemble_prefix_REX
	setnc	cl
	movzx	ecx,cl
	shl	ecx,3+2		; ECX*8*4
	add	rbx,rcx		; point to gpr_8_old_ext if REX prefix
				; if no REX, point to gpr_8
; We have prefix, so we must use gpr_8_old_ext or gpr_8_new. Both have 8 members by 4 bytes.
; Tables are gpr_8 (8*4 bytes), gpr_8_old_ext (8*4 bytes), gpr_new (8*4 bytes)
	bt	edx,0		; REX.B bit
	setc	cl		; EDX=0 in case of no REX so we don't change rbx=gpr_8
	movzx	ecx,cl
	shl	ecx,3+2
	add	rbx,rcx		; point to gpr_8_new if REX.B bit set

	mov	eax,[rbx+rax*4]
	stosd
; remove zeros on the end of the string
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	byte [rdi],0	; don't increase rdi

	mov	eax,2		; opcode size
	ret

process_opcode_big_fucking_gun_reg_muthafucker:
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	al,3
;
;00c0             add     al,al
;4000c0           add     al,al
;4000e0           add     al,spl
;4400c0           add     al,r8b
;
; gpr_8           al cl dl bl ah ch dh bh
; gpr_8_old_ext   al cl dl bl spl bpl sil dil
; gpr_8_new       r8b r9b r10b r11b r12b r13b r14b r15b
	lea	rbx,[gpr_8]
	call	search_disassemble_prefix_REX
	setnc	cl
	movzx	ecx,cl
	shl	ecx,3+2		; ECX*8*4
	add	rbx,rcx		; point to gpr_8_old_ext if REX prefix
				; if no REX, point to gpr_8
; we have prefix, so we must use gpr_8_old_ext or gpr_8_new both has 8 members by 4 bytes
; table is gpr_8 (8*4 bytes), gpr_8_old_ext (8*4 bytes), gpr_new (8*4 bytes)
	bt	edx,2		; REX.R
	setc	dl
	shl	edx,3+2		; rdx*8*4  ->  no dx but r10w , no eax but r8d , no rcx but r9
	add	rbx,rdx
	jmp	process_opcode_big_fucking_gun_reg_muthafucker_common_end

process_opcode_big_fucking_gun_reg_muthafucker_common_end:
	mov	eax,dword [rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	byte [rdi],0	; don't increase rdi
	mov	eax,2		; opcode size

	ret

process_opcode_big_fucking_gun_reg_muthafucker_wdq:
; 660100  add [rax],ax
;   0100  add [rax],eax
; 480100  add [rax],rax
; 440100  add [rax],r8d
; 4C0100  add [rax],r8
;   0100  add [rax],eax
;   0108  add [rax],ecx
;   0110  add [rax],edx
;   0118  add [rax],ebx
;   0120  add [rax],esp
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	al,3
process_opcode_big_fucking_gun_reg_muthafucker_wdq_common_begin:
	mov	byte [post_disasm_action],post_disasm_dword
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	setz	cl
	sub	byte [post_disasm_action],cl	; change 3=post_disasm_dword to 2=post_disasm_word
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size
	cmovc	rbx,rcx
	jnc	process_opcode_big_fucking_gun_reg_muthafucker_wdq_common_begin_L0
	mov	byte [post_disasm_action],post_disasm_qword
process_opcode_big_fucking_gun_reg_muthafucker_wdq_common_begin_L0:
	bt	edx,2		; REX.R
	setc	dl
	shl	edx,3+2		; rdx*8*4 ->  no dx but r10w , no eax but r8d , no rcx but r9
	add	rbx,rdx
	jmp	process_opcode_big_fucking_gun_reg_muthafucker_common_end

process_opcode_big_fucking_gun_reg_d:
	mov	byte [post_disasm_action],post_disasm_dword
	movzx	eax,byte [rsi+1]
	and	al,111b
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	setz	cl
	sub	byte [post_disasm_action],cl	; change 3=post_disasm_dword to 2=post_disasm_word
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size
	cmovc	rbx,rcx
	jnc	process_opcode_big_fucking_gun_reg_d_L0
	mov	byte [post_disasm_action],post_disasm_qword
process_opcode_big_fucking_gun_reg_d_L0:
	bt	edx,0		; REX.B
	setc	dl
	shl	edx,3+2		; rdx*8*4  ->  no dx but r10w , no eax but r8d , no rcx but r9
	add	rbx,rdx
	jmp	process_opcode_big_fucking_gun_reg_muthafucker_common_end

process_opcode_big_fucking_gun_mem_universal:
	movzx	eax,byte [rsi+1]
	and	al,111b

; 000424          add [rsp],al - bad: [rsp + rsp*1],al
; 00442408        add [rsp+08],al
; 008480c10f0000  add [rax+rax*4+00000fc1],al
;
; 00049F          add [rdi+rbx*4],al
; 000425a9020040  add [400002a9],al
;
; 000420          add [rax],al
; 000424
; 000426
; 000427          add [rdi],al
; 000460          add [rax],al
; 000467          add [rdi],al
; 0004A0          add [rax],al
; 0004A7          add [rdi],al
; 0004E0          add [rax],al
; 0004E7          add [rdi],al
	cmp	al,4
	jz	process_opcode_big_fucking_gun_mem_universal_SIB
	cmp	al,5
	jz	process_opcode_big_fucking_gun_mem_universal_RIP_addressing
	xor	ebx,ebx		; 0 for gpr_64, 16 for gpr_32
	call	search_disassemble_prefix_67
	setz	bl
	shl	bl,4
	add	al,bl		; not gpr_64 but grp_32
	call	search_disassemble_prefix_REX
	bt	edx,0
	setc	bl
	lea	eax,[rax+rbx*8]	; not [rax] but [r8]. Not [ecx] but [r9d]
	mov	dword [post_disasm_reg0],eax
	lea	rbx,[gpr_64]
	mov	eax,dword [rbx+rax*4]
	stosd
	or	byte [post_disasm_flag],post_disasm_flag_reg0
	shr	eax,16
	setz	bl
	shr	eax,8
	setz	al
	add	al,bl
	sub	rdi,rax

;0000                           add       [rax],al
;004020                         add       [rax+20],al
;008030010000                   add       [rax+00000130],al
;00C0                           add       al,al
; calculate opcode size:
	movzx	eax,byte [rsi+1]
	shr	eax,6			; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	eax,[rax+rax*2]		; eax*3   0->0   1->3   2->6   3->9
	mov	ebx,2
	or	eax,eax
	cmovz	eax,ebx			; 0*3->2
	cmp	eax,3*3
	cmovz	eax,ebx			; 3*3->2

	ret

process_opcode_big_fucking_gun_mem_universal_SIB:
; 000400          add [rax+rax],al
; 00041f          add [rdi+rbx],al
; 00042508010000  add [00000108],al
; 000430          add [rax+rsi],al
; 000480          add [rax+rax*4],al
; 0004a0          add [rax],al
; 0004a7          add [rdi],al
; 0004a8          add [rax+rbp*4],al
;
; 000420          add [rax],al
; 000424          add [rsp],al
; 000426          add [rsi],al
; 000427          add [rdi],al
; 000460          add [rax],al
; 000467          add [rdi],al
; 0004A0          add [rax],al
; 0004A7          add [rdi],al
; 0004E0          add [rax],al
; 0004E7          add [rdi],al
; 000425 !!!
; 0000000000401000  00042548000c24  add [240c0048],al
;
;0004C0          add [rax*8+rax],al
;0044C07B        add [rax*8+rax+7B],al
;0084C034120000  add [rax*8+rax+00001234],al

	movzx	eax,byte [rsi+2]

; fixed misdisassembled opcodes like:
;db 80h,7ch,25h,04h,0b8h
;cmp byte [r13+r12*2+04],-48h
;cmp byte [r13+r12*1+04],-48h
;807D0802		cmp byte [rbp+08],02
;807C25044F		cmp byte [rbp+04],4F
;43807C6504B8		cmp byte [r13+r12*2+04],-48
;40807C6504B8		cmp byte [rbp+04],-48
;803C2504B841807D	cmp byte [8041B804],7D
;41807C25044F		cmp byte [r13+04],4F
;43807D0802		cmp byte [r13+08],02
;43807C25044F		cmp byte [r13+r12*1+04],4F
;43803C2504B8FFC090	cmp byte [C0FFB804+r12*1],-70

	call	search_disassemble_prefix_REX
	bt	edx,1			; REX.X
	jc	process_opcode_big_fucking_gun_mem_universal_no_absolute_addressing

	mov	cl,al
	and	cl,00111111b
	cmp	cl,25h
	jnz	process_opcode_big_fucking_gun_mem_universal_no_absolute_addressing

	test	byte [rsi+1],11000000b
	jz	process_opcode_big_fucking_gun_mem_universal_absolute_addressing

; this was previous mistake, now it is replaced by above correct code:
;	mov	cl,al
;	and	cl,00111111b
;	cmp	cl,25h
;	jz	process_opcode_big_fucking_gun_mem_universal_absolute_addressing

process_opcode_big_fucking_gun_mem_universal_no_absolute_addressing:
	and	al,111b

;0044dd8e         add     [rbp+rbx*8-72],al
;000404           add     [rsp+rax],al
;00440404         add     [rsp+rax+04],al
;0084048e8f45ff   add     [rsp+rax-00ba7072],al
;000404           add     [rsp+rax],al
;0004058f841066   add     [6610848f+rax],al
;000406           add     [rsi+rax],al
;008404c18fc78f   add     [rsp+rax+8fc78fc1],al
;008405488fc066   add     [rbp+rax+66c08f48],al
;0084068fc0678f   add     [rsi+rax+8f67c08f],al
;00668f           add     [rsi-71],ah
;00418f           add     [rcx-71],al
;00040500418fc0   add     [c08f4100+rax],al
;00440501         add     [rbp+rax+0x1],al
;008405268d8088   add     [rbp+rax+0x88808d26],al

	mov	cl,al
	xor	ebx,ebx		; 0 for gpr_64, 16 for gpr_32
	call	search_disassemble_prefix_67
	setz	bl
	shl	bl,4
	add	al,bl		; not gpr_64 but grp_32
	call	search_disassemble_prefix_REX
	bt	edx,0
	setc	bl
	lea	eax,[rax+rbx*8]	; not [rax] but [r8]. Not [ecx] but [r9d]

; bad:	cmp	al,5	; 000425a9020040  add [400002a9],al
; bad:	jz	process_opcode_big_fucking_gun_mem_universal_SIB_mem
; right:
	cmp	cl,5
	jnz	process_opcode_big_fucking_gun_mem_universal_SIB_no_trubble
	cmp	byte [rsi+1],40h
	jnc	process_opcode_big_fucking_gun_mem_universal_SIB_no_trubble

; here we go in case of e.g. 000405.... and we must replace rbp with dword value:
	push	rax
	movsxd	rax,dword [rsi+3]
	mov	qword [post_disasm_addr0],rax
	or	byte [post_disasm_flag],post_disasm_flag_addr0
	shl	rax,32
	call	hexa_64_rax
	pop	rax
	scasq
; and we must increase opcode size by 4
	mov	eax,3+4
	jmp	process_opcode_big_fucking_gun_mem_universal_SIB_common_end

process_opcode_big_fucking_gun_mem_universal_SIB_no_trubble:
	mov	dword [post_disasm_reg0],eax
	lea	rbx,[gpr_64]
	mov	eax,dword [rbx+rax*4]
	stosd
	or	byte [post_disasm_flag],post_disasm_flag_reg0
	shr	eax,16
	setz	bl
	shr	eax,8
	setz	al
	add	al,bl
	sub	rdi,rax
	mov	eax,3		; opcode size

;0004c0           add     [rax+rax*8],al
;0044c07b         add     [rax+rax*8+0x7b],al
;0084C034120000  add [rax*8+rax+00001234],al;	setz	bl

	mov	cl,byte [rsi+1]
;	and	cl,11000000b
;
;	mov	edx,4
;	cmp	cl,40h
;	cmovz	eax,edx
;
;	mov	edx,7
;	cmp	cl,80h
;	cmovz	eax,edx
; faster way, transform 0-3F->0 40-7F->1 80-BF->4 (Note that C0-FF is handled in another subroutine 00C0 add al,al...)
	shr	cl,6
	mov	dl,cl
	and	dl,00000010b
	add	dl,cl
	add	al,dl

;4F8D2420	lea r12,[r8]	->	should be	lea r12,[r8+r12*1]
	call	search_disassemble_prefix_REX
	bt	edx,1		; REX.X
	jc	process_opcode_big_fucking_gun_mem_universal_SIB_common_end

; This is handled in another subroutine: 00042508010000  add [00000108],al
; another example: 000425a9020040 add [400002a9],al
; we won't go here in this case 000425...
; in next cases we skip to +...*...],...
; 000420          add [rax],al
; 000424          add [rsp],al
; 000426          add [rsi],al
; 000427          add [rdi],al
	mov	bl,byte [rsi+2]
	sub	bl,20h
	and	bl,00111111b
	cmp	bl,8
	jc	process_opcode_big_fucking_gun_mem_universal_SIB_terminate

process_opcode_big_fucking_gun_mem_universal_SIB_common_end:
	push	rax		; opcode size
	mov	al,'+'
	stosb
	movzx	eax,byte [rsi+2]
	shr	al,3
	and	al,111b


	xor	ebx,ebx		; 0 for gpr_64, 16 for gpr_32
	call	search_disassemble_prefix_67
	setz	bl
	shl	bl,4
	add	al,bl		; not gpr_64 but grp_32
	call	search_disassemble_prefix_REX
	bt	edx,1	; REX.X bit
	setc	bl
	lea	eax,[rax+rbx*8]	; not [rax] but [r8]. Not [ecx] but [r9d]
	mov	dword [post_disasm_reg1],eax
	lea	rbx,[gpr_64]
	mov	eax,dword [rbx+rax*4]
	stosd
	or	byte [post_disasm_flag],post_disasm_flag_reg1
	shr	eax,16
	setz	bl
	shr	eax,8
	setz	al
	add	al,bl
	sub	rdi,rax

	mov	al,'*'
	stosb
	movzx	ecx,byte [rsi+2]
	shr	cl,3+3
	mov	byte [post_disasm_mul],cl
	mov	al,'0'	; '0' = 30h so low nibble = 0 and we can use bit set directly:
	bts	eax,ecx	; convert '0'-> '1,2,4,8' depending on value in ecx (ecx is in range from 0 to 3)
	stosb
	pop	rax		; opcode size
process_opcode_big_fucking_gun_mem_universal_SIB_terminate:
	ret

process_opcode_big_fucking_gun_mem_universal_SIB_mem:
;000405a9020040  add [400002a9+rax*1],al
;00040da9020040  add [400002a9+rcx*1],al
;000415a9020040  add [400002a9+rdx*1],al
;00041da9020040  add [400002a9+rbx*1],al
;000425a9020040  add [400002a9],al
;00042da9020040  add [400002a9+rbp*1],al
;000435a9020040  add [400002a9+rsi*1],al
;00043da9020040  add [400002a9+rdi*1],al
;000445a9020040  add [400002a9+rax*2],al
;00044da9020040  add [400002a9+rcx*2],al
;000455a9020040  add [400002a9+rdx*2],al
;00045da9020040  add [400002a9+rbx*2],al
;000465a9020040  add [400002a9],al
;00046da9020040  add [400002a9+rbp*2],al
;000475a9020040  add [400002a9+rsi*2],al
;00047da9020040  add [400002a9+rdi*2],al
;000485a9020040  add [400002a9+rax*4],al
;00048da9020040  add [400002a9+rcx*4],al
;000495a9020040  add [400002a9+rdx*4],al
;00049da9020040  add [400002a9+rbx*4],al
;0004a5a9020040  add [400002a9],al
;0004ada9020040  add [400002a9+rbp*4],al

	movsxd	rax,dword [rsi+3]
	mov	qword [post_disasm_addr0],rax
	or	byte [post_disasm_flag],post_disasm_flag_addr0
	shl	rax,32
	call	hexa_64_rax
; ?????????????????????????????????????
; ???????? hexa_64_rax_sign ???????????
; ?????????????????????????????????????

	scasq

	call	search_disassemble_prefix_REX
	bt	edx,1	; REX.X bit
	setc	cl
	mov	al,byte [rsi+2]
	and	al,00111000b
	shr	al,3
	cmp	al,4
	setnz	al
	or	al,cl
	mov	eax,7		; opcode size
	jnz	process_opcode_big_fucking_gun_mem_universal_SIB_common_end
; no REX.X bit set and bit 5. of [rsi+2]=1
; absolute addressing - use dword address. Qword absolute addressing isn't possible.
; We can absolute address at offset 0-FFFFFFFF but not above FFFFFFFF because dword limit.
;000425a9020040  add [400002a9],al
;000465a9020040  add [400002a9],al
;0004a5a9020040  add [400002a9],al
;000425e9020040  add [400002a9],al
	ret

process_opcode_big_fucking_gun_mem_universal_RIP_addressing:
;000530000000  add [rip+00000030],al
;00457E        add [rbp+7E],al
;00857E100000  add [rbp+0000107E],al
;00C5          add ch,al
	test	byte [rsi+1],11000000b
	jnz	process_opcode_big_fucking_gun_mem_universal_RIP_addressing_rbp_r13

; 000530000000  add [rip+00000030],al
;	mov	eax,dword [rsi+2]
	movsxd	rax,dword [rsi+2]
; 0000000078D5A2F5   FF15556EFEFF call qword [0000000078D41150]
; in this case mov eax,dword [rsi+2] produce mistake:
; 0000000078D5A2F5   FF15556EFEFF call qword [0000000178D41150]

; we calculate address later - after return from disassembling when we have opcode size
	mov	qword [absolute_RIP_addressing_value],rax
	mov	qword [absolute_RIP_addressing_disassembled_position],rdi
	add	rdi,16		; reserve size for store address
	mov	eax,6		; opcode size, it will be increased to 7 for 3DNow! instructions
	ret

process_opcode_big_fucking_gun_mem_universal_RIP_addressing_rbp_r13:
;00457E        add [rbp+7E],al
;00857E100000  add [rbp+0000107E],al
	xor	ebx,ebx			; 0=rbp, 16=r13
	call	search_disassemble_prefix_67
	setz	bl
	shl	bl,4
	call	search_disassemble_prefix_REX
	bt	edx,0	; REX.B bit
	setc	dl
	lea	eax,[rbx+rdx*8+5]	; rbp as well r13 is 5th register from begin
	mov	dword [post_disasm_reg0],eax
	lea	rbx,[gpr_64]
	mov	eax,dword [rbx+rax*4]
	stosd
	or	byte [post_disasm_flag],post_disasm_flag_reg0
	shr	eax,16
	setz	bl
	shr	eax,8
	setz	al
	add	al,bl
	sub	rdi,rax

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	eax,[rax+rax*2]		; eax*3
	ret

process_opcode_big_fucking_gun_mem_universal_absolute_addressing:
;d8842500010000   fadd    dword ptr [rbp+00000100]
;d8846500020000   fadd    dword ptr [rbp+00000200]
;d884a500030000   fadd    dword ptr [rbp+00000300]
;d884e500040000   fadd    dword ptr [rbp+00000400]
;d8042500000000   fadd    dword ptr [00000000]
;d8442500         fadd    dword ptr [rbp]
;d8842500000000   fadd    dword ptr [rbp]
;d8040500000000   fadd    dword ptr [00000000+rax]

	cmp	byte [rsi+1],44h
	jz	process_opcode_big_fucking_gun_mem_universal_no_absolute_addressing
	cmp	byte [rsi+1],84h
	jz	process_opcode_big_fucking_gun_mem_universal_no_absolute_addressing

	mov	eax,dword [rsi+3]
	or	byte [post_disasm_flag],post_disasm_flag_addr0
	mov	qword [post_disasm_addr0],rax
	shl	rax,32
	call	hexa_64_rax
	scasq
	mov	eax,7
	ret

process_opcode_67_prefix:
	or	byte [post_disasm_prefix],post_disasm_prefix_67
process_opcode_F3_prefix:
process_opcode_F2_prefix:
process_opcode_66_prefix:
;process_opcode_67_prefix:
process_opcode_REX_prefix:
	push	rdi
	push	rax
	xor	eax,eax
	mov	ecx,4		; AMD64 supports up to 4 prefixes in opcode
	lea	rdi,[disassemble_prefix]
	repnz scasb
	pop	rax
	jnz	process_opcode_prefix_not_free_space
process_opcode_prefix_free_space:
	mov	byte [rdi-1],al
	inc	rsi
	pop	rdi
	movzx	eax,byte [rsi]
	call	qword [rbx+rax*8]
	inc	eax		; increase opcode size by 1
	ret
process_opcode_prefix_not_free_space:
	mov	rcx,qword [rdi-4-7]
	shr	dword [rdi-4],8
	mov	byte [rdi-1],al
	inc	rsi
	pop	rdi
	mov	dword [rdi],'db '
	xchg	rax,rcx
	add	rdi,3
	call	hexa_64_rax
	mov	byte [rdi+2],0
;	xchg	ecx,eax
	sub	rsi,rax
	inc	rsi
	mov	eax,1-InstructionMaxPrefixSize	; will be increased to 1 by 4x inc eax
	ret

process_opcode_0F:
	inc	rsi		; point to the next byte of opcode
	movzx	eax,byte [rsi]
	lea	rbx,[process_opcode_0F_table]
	call	qword [rbx+rax*8]
	dec	rsi
	inc	eax
	ret

process_opcode_push_pop_50_5F:
;6651    push  cx
;665d    pop   bp
;664151  push  r9w
;66415d  pop   r13w
;4151    push  r9
;415d    pop   r13
	sub	al,50h		; 50 begin PUSH RAX, 5F end PUSH RDI , F8 begin POP RAX
	mov	ecx,eax
	and	cl,00000111b
;	test	al,8
;	jnz	process_opcode_pop_58_5F
	bt	eax,3
	jc	process_opcode_pop_58_5F
	mov	al,'p'
	stosb
	mov	eax,'ush '
	jmp	process_opcode_push_pop_50_5F_common_end
process_opcode_pop_58_5F:
	mov	eax,'pop '
process_opcode_push_pop_50_5F_common_end:
	stosd

	call	search_disassemble_prefix_66
	jz	process_opcode_push_pop_50_5F_word

	lea	rbx,[gpr_64]	; RAX RCX...
process_opcode_push_pop_50_5F_L4:
	lea	rax,[rbx+8*4]		; R8 R9...
	call	search_disassemble_prefix_REX
process_opcode_push_pop_50_5F_terminate:
	bt	edx,0
	cmovc	rbx,rax			; r8 instead of rax, r9 instead of rbx
	mov	eax,[rbx+rcx*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	byte [rdi],0	; don't increase rdi
	mov	eax,1
	ret

process_opcode_push_pop_50_5F_word:
; 66h,48h - 66h,4Fh push_pop_qword !!!
	call	search_disassemble_prefix_REX
	bt	edx,3			; REX.W bit 64-bit operand size
	jc	process_opcode_push_pop_50_5F_terminate
;664851  push  rcx
;66485d  pop   rbp
;664951  push  r9
;66495d  pop   r13
;664a51  push  rcx
;664a5d  pop   rbp
	
	lea	rbx,[gpr_16]	; RAX RCX...
	jmp	process_opcode_push_pop_50_5F_L4

; 60 PUSHA undefined in AMD64 long mode
process_undefined_opcode_pusha:
	call	process_undefined_opcode_begin_string
	mov	rax,'pusha'
	jmp	process_opcode_common_end_55

; 61 POPA undefined in AMD64 long mode
process_undefined_opcode_popa:
	call	process_undefined_opcode_begin_string
	mov	rax,'popa'
	jmp	process_opcode_common_end_55

; 26 ES:
process_opcode_seg_ES:
	mov	eax,'ES '
	jmp	process_opcode_seg_common_end
; 2E CS:
process_opcode_seg_CS:
	mov	eax,'CS '
	jmp	process_opcode_seg_common_end
; 36 SS:
process_opcode_seg_SS:
	mov	eax,'SS '
	jmp	process_opcode_seg_common_end
; 3E DS:
process_opcode_seg_DS:
	mov	eax,'DS '
	jmp	process_opcode_seg_common_end
; 64 FS:   65 GS:
process_opcode_seg_FS_GS:
	lea	edx,[rax-63h]	; FS->1  (post_disasm_segment_fs) GS->2 (post_disasm_segment_gs)
	add	eax,'FS ' - 64h
	mov	byte [post_disasm_prefix],dl
process_opcode_seg_common_end:
	bts	dword [seg_prefix_not_flushed],0	; only low byte is used
	jnc	process_opcode_seg_common_end_L0
	cmp	byte [rdi-1],' '
	setz	cl
	movzx	ecx,cl
	sub	rdi,rcx
	xor	eax,eax			; we processed 0 bytes at this round
					; 0 is being increased to 1 by inc eax after ret
	mov	byte [rdi],al		; 0-terminator
	ret

process_opcode_seg_common_end_L0:
	stosd
	dec	rdi

	inc	rsi

	movzx	eax,byte [rsi]
	call	qword [rbx+rax*8]

	inc	eax		; increase opcode size by 1
	ret

; 62 BOUND undefined in long mode
;0000000000401000 6200             bound   eax,[rax]
;0000000000401002 622491           bound   esp,[rcx+rdx*4]
;0000000000401005 622549916641     bound   esp,[0000000041a6a154]
;000000000040100b 624025           bound   eax,[rax+0x25]
;000000000040100e 628000204000     bound   eax,[rax+0x402000]
;0000000000401014 62a40200904000   bound   esp,[rdx+rax+0x409000]
;000000000040101b 62a538010000     bound   esp,[rbp+0x138]
process_undefined_opcode_bound:
	call	process_undefined_opcode_begin_string
	mov	rax,'bound '
	stosq
	dec	rdi
	dec	rdi
	jmp	process_opcode_movsxd_common_begin

; 63 MOVSXD
process_opcode_movsxd:
;0000000000401000 6300             movsxd  rax,[rax]
;0000000000401002 632491           movsxd  rsp,[rcx+rdx*4]
;0000000000401005 632549916641     movsxd  rsp,[0000000041a6a154]
;000000000040100b 634025           movsxd  rax,[rax+25]
;000000000040100e 638000204000     movsxd  rax,[rax+00402000]
;0000000000401014 63a40200904000   movsxd  rsp,[rdx+rax+00409000]
;000000000040101b 63a538010000     movsxd  rsp,[rbp+00000138]
;0000000000401000 6300             movsxd  rax,[rax]
;0000000000401002 632491           movsxd  rsp,[rcx+rdx*4]
;0000000000401005 632549916641     movsxd  rsp,[0000000041a6a154]
;000000000040100b 634025           movsxd  rax,[rax+25]
;000000000040100e 638000204000     movsxd  rax,[rax+00402000]
;0000000000401014 63a40200904000   movsxd  rsp,[rdx+rax+0x409000]
;000000000040101b 63a538010000     movsxd  rsp,[rbp+00000138]
;0000000000401021 66630480         movsxd  rax,[rax+rax*4]
;0000000000401025 67630480         movsxd  rax,[eax+eax*4]
;0000000000401029 6667630480       movsxd  rax,[eax+eax*4]
;000000000040102e 6766630480       movsxd  rax,[eax+eax*4]
;0000000000401033 666741630480     movsxd  rax,[r8d+eax*4]
;0000000000401039 666742630480     movsxd  rax,[eax+r8d*4]
;000000000040103f 666743630480     movsxd  rax,[r8d+r8d*4]
;0000000000401045 666744630480     movsxd  r8,[eax+eax*4]
;000000000040104b 666745630480     movsxd  r8,[r8d+eax*4]
;0000000000401051 666746630480     movsxd  r8,[eax+r8d*4]
;0000000000401057 666747630480     movsxd  r8,[r8d+r8d*4]
;000000000040105d 666748630480     movsxd  rax,[eax+eax*4]
;0000000000401063 666749630480     movsxd  rax,[r8d+eax*4]
;0000000000401069 66674a630480     movsxd  rax,[eax+r8d*4]
;000000000040106f 66674b630480     movsxd  rax,[r8d+r8d*4]
;0000000000401075 66674c630480     movsxd  r8,[eax+eax*4]
;000000000040107b 66674d630480     movsxd  r8,[r8d+eax*4]
;0000000000401081 66674e630480     movsxd  r8,[eax+r8d*4]
;0000000000401087 66674f630480     movsxd  r8,[r8d+r8d*4]
;6667630480       movsxd  rax,[eax+eax*4]
;666740630480     movsxd  rax,[eax+eax*4]

	mov	rax,'movsxd '
	stosq
	dec	rdi
	mov	byte [post_disasm_action],post_disasm_dword

process_opcode_movsxd_common_begin:
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	al,3
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
	cmovc	rbx,rcx
	bt	edx,2		; REX.R
	setc	dl
	shl	edx,3+2
	add	rbx,rdx		; not rax but r8, not eax but r8d ...

	mov	eax,dword [rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	al,','
	stosb

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
;	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
	lea	rbx,[process_Ed_table]
	call	qword [rbx+rax*8]
; return opcode size
	ret

; 68h push dword sign extended
process_opcode_push_dword:
	mov	eax,'push'
	stosd
	mov	al,' '
	stosb
;	mov	eax,dword [rsi+1]
;	cdqe
	movsxd	rax,dword [rsi+1]
	call	hexa_64_rax	;hexa_64_rax_sign_negative
	mov	byte [rdi+16],0

	mov	qword [post_disasm_addr0],rax
	or	byte [post_disasm_flag],post_disasm_flag_direct
	mov	byte [post_disasm_action],post_disasm_addr

	mov	eax,5
	ret

; 6Ah push byte sign extended
process_opcode_push_byte:
	mov	eax,'push'
	stosd
	mov	al,' '
	stosb
	movsx	rax,byte [rsi+1]
	call	hexa_64_rax_sign_negative
	mov	byte [rdi+16],0
	mov	eax,2
	ret

; 69 imul by dword , 6B imul by byte
process_opcode_imul_69:
process_opcode_imul_6B:
process_opcode_imul_69_6B:
;69000b000000     imul eax,[rax],0000000b
;6bb40006010000d2 imul esi,[rax+rax+00000106],ffffffd2
;6bb500020000c1   imul esi,[rbp+00000200],ffffffc1
;6b24050801000091 imul esp,[00000108+rax],ffffff91
;6b240467         imul esp,[rsp+rax],67
;6b2508010000f2   imul esp,[00401130],fffffff2
;69C00B000000     imul eax,eax,0000000B
	mov	eax,'imul'
	stosd
	mov	al,' '
	stosb
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	mov	byte [post_disasm_action],post_disasm_dword
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	setz	dl
	sub	byte [post_disasm_action],dl	; post_disasm_dword -> post_disasm_word
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3			; REX.W
	cmovc	rbx,rcx
	bt	edx,2			; REX.R
	setc	cl
	movzx	ecx,cl
	shl	ecx,3+2
	add	rbx,rcx			; not rax but r8, not eax but r8d ...
	bt	edx,3			; REX.W
	jnc	process_opcode_imul_69_6B_L0
	mov	byte [post_disasm_action],post_disasm_qword
process_opcode_imul_69_6B_L0:
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	al,','
	stosb

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
	call	qword [rbx+rax*8]
; return opcode size
; return opcode size of first 2 members

	mov	byte [rdi],','
	scasb

; process last value:
	mov	edx,dword [rsi+rax*1]	; grab value

; add 1 if 6B opcode, add 4 if 69 opcode
	mov	ebx,1
	mov	ecx,4
	cmp	byte [rsi],69h
	cmovz	ebx,ecx

	add	rax,rbx		; add 1 or 4 to opcode size

	mov	cl,8*8
	shl	bl,3		; BL*8
	sub	cl,bl

	push	rax
	xchg	edx,eax
	shl	rax,cl
	call	hexa_64_rax_sign_negative
	pop	rax

	shr	bl,2
	add	rdi,rbx
	mov	byte [rdi],0

	ret

; 6Ch insb
; 6Dh insd
; 6Eh outsb
; 6Fh outsw
; 66 6D insw
; 66 6F outsw
process_opcode_insb_outsb:
	and	al,00000011b	;	sub	al,6Ch
	lea	rbx,[instructions_insb_outsb_table]
	xor	ecx,ecx
	mov	edx,1
	test	al,1		; test bit 0.
	cmovz	edx,ecx		; e.g. db 66h 6Ch = insb    db 66h 6Dh = insw

	call	search_disassemble_prefix_66
	cmovz	ecx,edx
	shr	eax,cl		;	shr	rax,cl
	shl	ecx,3+2		; ecx*8*4
	add	rbx,rcx
	mov	rax,[rbx+rax*8]
	stosq
	mov	eax,1
	ret

;70-7F JO JNO JB JNB JZ JNZ JBE JNBE    JS JNS JP JNP JL JNL JLE JNLE
process_opcode_jump_condition:
;	movzx	eax,byte [esi]
; eax = byte [rsi] now
	and	al,00001111b
	mov	[post_disasm_cond_jump_type],al
	lea	rbx,[instructions_jump_conditional_table]
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	al,' '
	stosb
	movsx	rax,byte [rsi+1]
	lea	rax,[rbp + rax*1 + 2]
	call	hexa_64_rax
	mov	byte [rdi+16],0

	mov	qword [post_disasm_addr0],rax
	or	byte [post_disasm_flag],post_disasm_flag_direct or post_disasm_flag_cond_jump
	mov	byte [post_disasm_action],post_disasm_addr

	mov	eax,2
	ret

process_opcode_80_83:
;800008        add byte [rax],08
;810030010000  add dword [rax],00000130
;820028        add byte [rax],28
;830030        add dword [rax],00000030
;800888        or  byte [rax],88
;801005        adc byte [rax],05
;80180c        sbb byte [rax],0c
;802064        and byte [rax],64
;802815        sub byte [rax],15
;803078        xor byte [rax],78
;80381d        cmp byte [rax],1d
;80405844      add byte [rax+58],44
;80405844      add byte [rax+58],44
;6680405844    add byte [rax+58],44
;6780405844    add byte [eax+58],44
;4180405844    add byte [r8+58],44
;674180405844  add byte [r8d+58],44
;8004258d00666748 add byte ptr [6766008d],48
;80056667496800   add byte [00000000688977b2],00
;80844a8d0066674b add byte [rdx+rcx*2+6766008d],4b
;8084a5674c7d0066 add byte [rbp+007d4c67],66
;6681006641       add word [rax],4166
;668100b841       add word [rax],41b8
;666781000f0c     add word [eax],0c0f
;4c810048b84800   add qword [rax],0048b848
;4881006b030549   add qword [rax],4905036b
;4081000c24486b   add dword [rax],6b48240c
;49830005         add qword [r8],05
;80C008           add al,08
;80C008           add al,08
;6681C00100       add ax,0001
;4081C021830562   add eax,62058321
;4881C080100574   add rax,74051080
;664181C06441     add r8w,4164
;664981C07880381D add r8,1D388078

; sub al,80 isn't necessary, we remove highest bit by AND operation
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	al,3
	lea	rbx,[process_opcode_add_or_adc_sbb_and_sub_xor_cmp_table]
	mov	eax,dword [rbx+rax*4]
	stosd
;	xor	eax,eax
;	cmp	byte [rdi-2],' '
;	setz	al
;	sub	rdi,rax
; this is faster than above 4 instructions:
	cmp	byte [rdi-2],' '+1
	sbb	rdi,0

process_opcode_80_83_common_begin:
	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_80_83_skip_store_operand_size
	mov	ebx,'byte'
	mov	byte [post_disasm_action],post_disasm_byte
	bt	dword [rsi],0
	jnc	process_opcode_80_83_store_operand_size
	mov	ebx,'word'
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
; REX.W overwrites 66 prefix
	jc	process_opcode_80_83_skip_prefix_66

	mov	byte [post_disasm_action],post_disasm_word
	call	search_disassemble_prefix_66
	jz	process_opcode_80_83_store_operand_size
process_opcode_80_83_skip_prefix_66:
	mov	byte [post_disasm_action],post_disasm_dword
	mov	al,'d'
	mov	cl,'q'
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
	cmovc	eax,ecx
	adc	byte [post_disasm_action],0	; post_disasm_dword -> post_disasm_qword
	stosb
process_opcode_80_83_store_operand_size:
	xchg	ebx,eax
	stosd
	mov	al,' '
	stosb
process_opcode_80_83_skip_store_operand_size:

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_80_83_table]
	call	qword [rbx+rax*8]
; 800020          add [rax],+20
; 80403010        add [rax+30],+10
; 80802010400160  add [rax+01401020],+60
; 80C008          add al,08
; 6681C0EC78      add ax,78EC
; 81C083EC7848    add eax,4878EC83
; 4881C0EC784837  add rax,+374878EC

	push	rax		; opcode size

	mov	byte [rdi],','	; we mustn't destroy eax !!!
	scasb

; we need eax to get value at [rsi+rax] in this subroutine:
; we have CL=0 for reg8, CL=1 for reg16, CL=2 for reg32, CL=3 for reg64
; CL is returned from process_opcode_80_83_reg
	call	process_opcode_80_83_value
; returns value size (1,2,4) in ebx
	mov	byte [rdi],0

	pop	rax		; opcode size
	add	eax,ebx		; add value size
	ret

process_opcode_80_83_reg:
; 80C008          add al,08
; 6681C0EC78      add ax,78EC
; 81C083EC7848    add eax,4878EC83
; 4881C0EC784837  add rax,+374878EC
; 4981C0EC784837  add r8,+374878EC
	movzx	eax,byte [rsi+1]
	and	al,00000111b

	bt	dword [rsi],0
;	jnc	process_opcode_big_fucking_gun_reg_common_begin
	jc	process_opcode_80_83_reg_wdq

	lea	rbx,[gpr_8]
	call	search_disassemble_prefix_REX
	setnc	cl
	movzx	ecx,cl
	shl	ecx,3+2		; ECX*8*4
	add	rbx,rcx		; point to gpr_8_old_ext if REX prefix
				; if no REX, point to gpr_8
; We have prefix, so we must use gpr_8_old_ext or gpr_8_new. Both have 8 members by 4 bytes.
; Tables are gpr_8 (8*4 bytes), gpr_8_old_ext (8*4 bytes), gpr_new (8*4 bytes)
	bt	edx,0		; REX.B bit
	setc	cl		; EDX=0 in case of no REX so we don't change rbx=gpr_8
	movzx	ecx,cl
	shl	ecx,3+2
	add	rbx,rcx		; point to gpr_8_new if REX.B bit set
	mov	cl,-1
	jmp	process_opcode_80_83_reg_have_it

process_opcode_80_83_reg_wdq:
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	mov	cl,0		; reg16
	jz	process_opcode_80_83_reg_wdq_L8
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
	cmovc	rbx,rcx
	mov	cl,1		; reg32
	adc	cl,0		; reg64
process_opcode_80_83_reg_wdq_L8:
	call	search_disassemble_prefix_REX
	bt	edx,0		; REX.B
	setc	dl
	movzx	edx,dl
	shl	edx,3+2
	add	rbx,rdx
process_opcode_80_83_reg_have_it:
	inc	cl
; cl=0 reg8, cl=1 reg16, cl=2 reg32, cl=3 reg64
	mov	eax,dword [rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0
	mov	byte [rdi],0	; 0-terminator, don't increase rdi
	mov	eax,2		; opcode size
	ret

process_opcode_80_83_value:
; we have CL=0 for reg8, CL=1 for reg16, CL=2 for reg32, CL=3 for reg64
; CL is returned from process_opcode_80_83_reg

; 1. get value size (byte/word/dword)
	movzx	ebx,byte [rsi]	; zeroing upper parts of EBX too
	and	bl,00000011b
	cmp	bl,1		; value size 1 byte
; 80 mov reg8,val8
; 81 mov reg32,val32
; 82 mov reg8,val8
; 83 mov reg32,val8
	mov	bl,1
	jnz	process_opcode_80_83_value_have_size

; now we have 81 mov regx,valx where regx<>reg8 , valx<>val8
; so we check for reg16, reg32, reg64
	mov	bl,4		; value size 1 dword
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size this overwrite 66 prefix
	jc	process_opcode_80_83_value_have_size
	mov	dl,2		; value size 1 word
	call	search_disassemble_prefix_66
	cmovz	ebx,edx
process_opcode_80_83_value_have_size:
	movzx	edx,byte [post_disasm_action]
	sub	dl,post_disasm_byte
	cmp	byte [rsi+1],11000000b
	cmovc	ecx,edx		; [memory], not register

; opcode 8xYZ
; x=0,1,2,3
; YZ >= 0C0h
; bl=1 for value8, bl=2 for value16, bl=4 for value32
; for bl>1 we have CL=0 for reg8, CL=1 for reg16, CL=2 for reg32, CL=3 for reg64
	cmp	cl,0
;	jz	process_opcode_80_83_value_easy
	jnz	process_opcode_80_83_value_a_bit_difficult

process_opcode_80_83_value_easy:
	mov	eax,[rsi+rax*1]
process_opcode_80_83_value_done:
	mov	cl,8
	sub	cl,bl
	shl	cl,3		; CL*8
	shl	rax,cl
;
;	call	hexa_64_rax
;	mov	byte [rdi],0
;	ret
;
;	call	hexa_64_rax_sign_negative
;	lea	rdi,[rdi+rbx*2]
;;	xor	ecx,ecx
;;	or	rax,rax
;;	sets	cl
;;	add	rdi,rcx
;; in case of negative value we store e.g. -01 not only FF
;; this isn't good in case of cmp...,0CCh where 0CCh is breakpoint instruction, not negative value...
;; not good in case:
;; and dword [ThreadContext + ThreadContext_flags_offset],11111111111111111111111011111111b
;; disassembled as ...,-00000101

	call	hexa_64_rax
	lea	rdi,[rdi+rbx*2]
	mov	byte [rdi],0
	ret

process_opcode_80_83_value_a_bit_difficult:
; e.g.
; 6683C0FF add  ax,FFFF
;   83C0FD add eax,FFFFFFFD
; 4883C8BA add rax,FFFFFFFFFFFFFFBA
	xchg	edx,eax
	movsx	rax,byte [rsi+rdx*1]
	cmp	bl,1
	jz	process_opcode_80_83_value_a_bit_difficult_value_done
	movsx	rax,word [rsi+rdx*1]
	cmp	bl,2
	jz	process_opcode_80_83_value_a_bit_difficult_value_done
	movsxd	rax,dword [rsi+rdx*1]
process_opcode_80_83_value_a_bit_difficult_value_done:
; we want short for for possitive values, e.g. 7Fh,
	or	rax,rax
	jns	process_opcode_80_83_value_done
; we want long for for negative values, e.g. 80h=FFFFFFFFFFFFFF80h
; ebx=1 for value8, ebx=2 for value16, ebx=4 for value32
; CL=0 for reg8, CL=1 for reg16, CL=2 for reg32, CL=3 for reg64
	xor	edx,edx
	bts	edx,ecx
	mov	cl,8
	sub	cl,dl
	shl	cl,3		; CL*8
	shl	rax,cl
	call	hexa_64_rax
	lea	rdi,[rdi+rdx*2]
	mov	byte [rdi],0
	ret

process_opcode_test_xchg_84_87:
; 8400           test [rax],al
; 8500           test [rax],eax
; 854010         test [rax+0x10],eax
; 66854010       test [rax+0x10],ax
; 6748854824     test [eax+0x24],rcx
; 4985d3         test r11,rdx
; 67418510       test [r8d],edx
; 666785048c     test [esp+ecx*4],ax
; 4084C4         test spl,al
; 4184C4         test r12b,al
; 8700           xchg [rax],eax
; 874004         xchg [rax+04],eax
; 878000A90200   xchg [rax+0002A900],eax
; 87C0           xchg eax,eax
; 872400         xchg [rax+rax*1],esp
; 872500A90200   xchg [rip+0002A900],esp    rip must be recalculated to absolute, e.g.:
;                0000000000401000:  872500a90200  xchg [000000000042b906],esp
	bt	eax,1
	mov	eax,'test'
	mov	ebx,'xchg'
	cmovc	eax,ebx
	stosd
	mov	al,' '
	stosb

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_80_83_table]
	call	qword [rbx+rax*8]
; return opcode size

	mov	byte [rdi],','
	scasb

process_opcode_84_87_register:
	push	rax
	bt	dword [rsi],0
	jc	process_opcode_84_87_register_wdq
process_opcode_84_87_register_b:
	mov	byte [post_disasm_action],post_disasm_byte
	lea	rbx,[gpr_8]
	lea	rcx,[gpr_8 + 8*4]
	call	search_disassemble_prefix_REX
	cmovnc	rbx,rcx
	lea	rcx,[gpr_8 + 8*4 + 8*4]
	bt	edx,2		; REX.R
	cmovc	rbx,rcx
process_opcode_84_87_reg_have_it:
	movzx	eax,byte [rsi + 1]
	and	al,00111000b
	shr	eax,3
	mov	eax,[rbx+rax*4]
	stosd

	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	byte [rdi],0	; don't increase rdi
	pop	rax
	ret

process_opcode_84_87_register_wdq:
	mov	byte [post_disasm_action],post_disasm_dword
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	setz	dl
	sub	byte [post_disasm_action],dl	; post_disasm_dword -> post_disasm_word
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
	cmovc	rbx,rcx
	jnc	process_opcode_84_87_register_wdq_L0
	mov	byte [post_disasm_action],post_disasm_qword
process_opcode_84_87_register_wdq_L0:
	bt	edx,2		; REX.R
	setc	dl
	shl	edx,3+2
	add	rbx,rdx
	jmp	process_opcode_84_87_reg_have_it

; 88-8B
process_opcode_mov_88_8B:
	sub	al,88h
	xchg	eax,ecx
	mov	eax,'mov '
	stosd
	lea	rbx,[process_opcode_big_fucking_gun_table]

;;; !;	lea	rax,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]

	call	qword [rbx+rcx*8]
	ret

process_opcode_mov_8C:
;8c0c24           mov [rsp],cs
;8c0423           mov [rbx],es
;8c0424           mov [rsp],es
;8c0c24           mov [rsp],cs
;8c1424           mov [rsp],ss
;8c446440         mov [rsp+40],es
;8c00             mov [rax],es
;8c00             mov [rax],es
;8c01             mov [rcx],es
;8c02             mov [rdx],es
;8c03             mov [rbx],es
;8c0400           mov [rax+rax],es
;8c448548         mov [rbp+rax*4+48],es
;000000000040100f 8c0585d36741     mov [0000000041a7e39a],es
;8c06             mov [rsi],es
;8c07             mov [rdi],es
;8c08             mov [rax],cs
;8c2484           mov [rsp+rax*4],fs
;0000000000401020 8c25e0448410     mov [0000000010c45506],fs
;8c84f84d85d348   mov [rax+rdi*8+48d3854d],es
;8ca5788d001d     mov [rbp+1d008d78],fs
;8CC0             mov ax,es
;8CD0             mov ax,ss
;8CD1             mov cx,ss
	mov	ecx,eax
	mov	eax,'mov '
	stosd

	mov	byte [post_disasm_action],post_disasm_word

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_8C_table]
	call	qword [rbx+rax*8]

	push	rax		; opcode size

	mov	byte [rdi],','	; we mustn't destroy eax !!!
	scasb

	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	lea	rbx,[segment_registers]
	mov	eax,[rbx+rax*2]
	stosw

	mov	byte [rdi],0

	pop	rax		; opcode size
	ret

process_opcode_8C_reg:
	movzx	eax,byte [rsi+1]
	and	al,00000111b
	jmp	process_opcode_80_83_reg_wdq

process_opcode_mov_8E:
	mov	ecx,eax
	mov	eax,'mov '
	stosd

	mov	byte [post_disasm_action],post_disasm_word

	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	lea	rbx,[segment_registers]
	mov	eax,[rbx+rax*2]
	stosw

	mov	al,','
	stosb

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_8C_table]
	call	qword [rbx+rax*8]
; return opcode size

	mov	byte [rdi],0

	ret

; 8D LEA
process_opcode_lea:
;0000000000401000 8d00             lea eax,[rax]
;0000000000401002 8d20             lea esp,[rax]
;0000000000401004 8d2478           lea esp,[rax+rdi*2]
;0000000000401007 8d2523010000     lea esp,[0000000000401130]
;000000000040100d 8d26             lea esp,[rsi]
;000000000040100f 8d8088000000     lea eax,[rax+00000088]
;0000000000401015 8da48c04010000   lea esp,[rsp+rcx*4+00000104]
;000000000040101c 8da508020000     lea esp,[rbp+00000208]
;668d00      lea ax,[rax]
;66678d00    lea ax,[eax]
;678d00      lea eax,[eax]
;8d00        lea eax,[rax]
;6667408d00  lea ax,[eax]
;6667418d00  lea ax,[r8d]
;6667448d00  lea r8w,[eax]
;6667458d00  lea r8w,[r8d]
;6667488d00  lea rax,[eax]
;6667498d00  lea rax,[r8d]
;66674a8d00  lea rax,[eax]
;66674b8d00  lea rax,[r8d]
;66674c8d00  lea r8,[eax]
	mov	eax,'lea '
	stosd
	jmp	process_opcode_big_fucking_gun_reg_mem_d

process_opcode_pop_8F:
	mov	eax,'pop '
	stosd

;	cmp	byte [rsi+1],11000000b
;	jnc	process_opcode_pop_8F_operand_size_done
; not necessary, 8FC0-8FFF invalid opcode

	mov	byte [post_disasm_action],post_disasm_word
	mov	rax,'qword '
	mov	rcx,'word '
	xor	edx,edx
	call	search_disassemble_prefix_66
	setnz	dl
	cmovz	rax,rcx

	mov	qword [rdi],rax
	lea	rdi,[rdi+5+rdx*1]
	add	dl,dl
	add	byte [post_disasm_action],dl	; post_disasm_word -> post_disasm_qword

;process_opcode_pop_8F_operand_size_done:
	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_8F_table]
	call	qword [rbx+rax*8]

	mov	byte [rdi],0

	ret

process_opcode_8F_reg:
	movzx	eax,byte [rsi+1]
	and	al,00000111b
	lea	rbx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,0		; REX.B
	setc	cl
	movzx	ecx,cl
	add	rbx,rcx
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0
	mov	eax,2		; opcode size
	ret

; NOP, PAUSE, XCHG R8,RAX
process_opcode_90:
	cmp	dword [disassemble_prefix],0F3h
	jz	process_opcode_90_pause
	call	search_disassemble_prefix_REX
	jc	process_opcode_90_nop
	bt	edx,0
	jc	process_opcode_91_to_97		; xchg r8d,eax \ xchg r8,rax
process_opcode_90_nop:
	mov	eax,'nop'
	stosd
	mov	eax,1
	ret
process_opcode_90_pause:
	mov	eax,'paus'
	stosd
	mov	al,'e'
	stosb
	mov	byte [rdi],0
	mov	eax,1
	ret

; XCHG reg (16,32,64),accumulator
process_opcode_91_to_97:
; 6691    xchg cx,ax
; 91      xchg ecx,eax
; 4891    xchg rcx,rax
; 664191  xchg r9w,ax
; 4191    xchg r9d,eax
; 4991    xchg r9,rax
	mov	eax,'xchg'
	stosd
	mov	al,' '
	stosb
	movzx	eax,byte [rsi]
	and	al,00000111b
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size
	cmovc	rbx,rcx
	bt	edx,0		; REX.B
	setc	dl
	shl	edx,3+2		; rdx*8*4  ->  no dx but r10w , no eax but r8d , no rcx but r9

	mov	ecx,[rbx]	; grab 'ax' or 'eax' or 'rax' to ecx

	add	rbx,rdx

	mov	eax,[rbx+rax*4]
	stosd

	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	al,','
	stosb

	xchg	ecx,eax
	stosd

	jmp	process_opcode_common_end_55

; 98-99
process_opcode_convert_b_w_d_q_o:
;98               cwde
;99               cdq
;4898             cdqe
;4899             cqo
;6698             cbw
;6699             cwd
	sub	al,98h
	xor	edx,edx
	mov	ecx,2
	mov	ebx,4
	call	search_disassemble_prefix_REX
	bt	edx,3			; REX.W bit 64-bit operand size
	cmovc	edx,ecx
	call	search_disassemble_prefix_66
	cmovz	edx,ebx
	lea	rbx,[instruction_convert_b_w_d_q_o_table]
	add	eax,edx
	mov	eax,[rbx+rax*4]
	stosd
	mov	al,0
	stosb
	mov	eax,1
	ret

; 9A undefined in long mode e.g. 9A070040003300 call 0033:00400007
process_opcode_undefined_call_far_9A:
	call	process_undefined_opcode_begin_string
	mov	eax,'call'
	stosd
	mov	al,' '
	stosb
	mov	eax,[rsi+5]
	shl	rax,32+16
	call	hexa_64_rax
	scasd
	mov	al,':'
	stosb
	mov	eax,[rsi+1]
	shl	rax,32
	call	hexa_64_rax
	scasq
	xor	eax,eax
	stosb
	mov	eax,7
	ret

; 9B wait
process_opcode_wait:
	mov	eax,'wait'
	stosd
	xor	eax,eax
	stosb			; zero terminated string
	inc	eax		; opcode size = 1
	ret

; 9C pushf
process_opcode_pushf:
; 669c  pushfw
; 9c    pushfd
; 489c  pushfq
	mov	rax,'pushf'
	mov	rcx,'w' * 10000000000h
process_common_end_9C_9D_L0:
	xor	ebx,ebx
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
process_common_end_9C_9D:
	or	rax,rbx
process_common_end_9C_9F:
	stosq
	mov	eax,1
	ret

; 9D POPF
process_opcode_popf:
	mov	eax,'popf'
	mov	rcx,'w' * 100000000h
	jmp	process_common_end_9C_9D_L0

; 9E SAHF
process_opcode_sahf:
	cmp	[supported_lahf_sahf],0
	jnz	process_opcode_sahf_L0
	call	process_undefined_opcode_begin_string
process_opcode_sahf_L0:
	mov	eax,'sahf'
	jmp	process_common_end_9C_9F

; 9F LAHF
process_opcode_lahf:
	cmp	[supported_lahf_sahf],0
	jnz	process_opcode_lahf_L0
	call	process_undefined_opcode_begin_string
process_opcode_lahf_L0:
	mov	eax,'lahf'
	jmp	process_common_end_9C_9F

; A0-A3
process_opcode_mov_accumulator_absolute:
	bt	eax,1
	mov	eax,'mov '
	stosd
;A0h mov al,[mem64]
;A1h mov rax,[mem64]
;A2h mov [mem64],al
;A3h mov [mem64],rax 
	jc	process_opcode_mov_accumulator_absolute_mem_reg
process_opcode_mov_accumulator_absolute_reg_mem:
	call	process_opcode_mov_accumulator_absolute_reg
	mov	al,','
	stosb
	call	process_opcode_mov_accumulator_absolute_mem
	xor	eax,eax
	stosb
;	mov	al,9
; fixed 2006, april, 1st
; 48a13000000000000000 mov rax,[0000000000000030]
; 656748a130000000     mov rax,gs:[00000030]
	call	search_disassemble_prefix_67
	setnz	al
	movzx	eax,al
	lea	eax,[4+rax*4 + 1]	; transform 0->4 , 1->8 (0=dword, 1=qword)
					; 1=self opcode size (A0, A1, A2, A3)
	ret

process_opcode_mov_accumulator_absolute_mem_reg:
	call	process_opcode_mov_accumulator_absolute_mem
	mov	al,','
	stosb
	call	process_opcode_mov_accumulator_absolute_reg
	mov	al,0
	stosb
	call	search_disassemble_prefix_67
	setnz	al
	movzx	eax,al
	lea	eax,[4+rax*4 + 1]	; transform 0->4 , 1->8 (0=dword, 1=qword)
					; 1=self opcode size (A0, A1, A2, A3)
	ret

process_opcode_mov_accumulator_absolute_reg:
	mov	byte [post_disasm_action],post_disasm_byte
	mov	ecx,2		; AL as well AX have 2 bytes
	mov	eax,'al'
	bt	dword [rsi],0
	jnc	process_opcode_mov_accumulator_absolute_reg_store_accumulator
	mov	byte [post_disasm_action],post_disasm_word
	mov	eax,'ax'
	call	search_disassemble_prefix_66
	jz	process_opcode_mov_accumulator_absolute_reg_store_accumulator
	dec	ecx		; EAX as well RAX have 3 bytes
	mov	byte [post_disasm_action],post_disasm_dword
	mov	eax,'eax'
	mov	ebx,'rax'
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W bit 64 bit operand size
	cmovc	eax,ebx
	adc	byte [post_disasm_action],0	; change post_disasm_dword -> post_disasm_qword
process_opcode_mov_accumulator_absolute_reg_store_accumulator:
	stosd
	sub	rdi,rcx
	ret

process_opcode_mov_accumulator_absolute_mem:
	mov	al,'['
	stosb
;	mov	rax,qword [rsi+1]
; fixed 2006, april, 1st
; 48a13000000000000000 mov rax,[0000000000000030]
; 656748a130000000     mov rax,gs:[00000030]
;	lea	rax,[rsi+1]
;	lea	rcx,[rsi+1-4]
;	call	search_disassemble_prefix_67
;	cmovz	rax,rcx
;	mov	edx,16
;	mov	ecx,8
;	cmovz	edx,ecx
;	movsxd	ecx,dword [rsi+1]
;	mov	rax,qword [rax]
;	cmovz	rcx,rax
;	setnz	bl
;	call	hexa_64_rax
;;	add	rdi,16
;	lea	rdi,[rdi+rdx*1]
;	mov	al,']'
;	stosb
;	ret
	mov	rax,qword [rsi+1]
	call	search_disassemble_prefix_67
	setz	cl
	setnz	dl
	shl	cl,3+2
	movzx	edx,dl
	shl	rax,cl
	call	hexa_64_rax
	shr	rax,cl
	or	byte [post_disasm_flag],post_disasm_flag_addr0
	mov	qword [post_disasm_addr0],rax
	lea	rdi,[rdi+8+rdx*8]
	mov	al,']'
	stosb
	ret

; A4-A7   A4 movsb  A5 movsd  A6 cmpsb  A7 cmpsd
process_opcode_string_operations_0:
	or	byte [post_disasm_flag],post_disasm_flag_rdi or post_disasm_flag_rsi
	sub	al,0A4h
	lea	rbx,[process_opcode_string_operations_table_0]
	jmp	process_opcode_string_operations_common_begin

process_opcode_test_accumulator_A8_A9:
;	sub	al,0A8h		; isn't needed, process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator test only bit 0 of RCX and ignore remainder of CL
	mov	ecx,eax
	mov	eax,'test'
	stosd
	mov	al,' '
	stosb
	jmp	process_opcode_add_or_adc_sbb_and_sub_xor_cmp_accumulator

; AA-AF   AA stosb  AB stosd  AC lodsb  AD lodsd  AE scasb  AF scasd
process_opcode_string_operations_1:
	sub	al,0AAh

	mov	bl,post_disasm_flag_rsi
	mov	dl,post_disasm_flag_rdi
	lea	ecx,[rax-2]
	shr	cl,1
	cmovz	edx,ebx
	or	byte [post_disasm_flag],dl
	lea	rbx,[process_opcode_string_operations_table_1]
process_opcode_string_operations_common_begin:
	mov	ecx,eax
	shr	eax,1
	mov	eax,[rbx+rax*4]
	stosd
	test	cl,00000001b
	jnz	process_opcode_string_operations_no_b
	mov	al,'b'
	mov	bl,post_disasm_byte
process_opcode_string_operations_common_end:
	mov	byte [post_disasm_action],bl
	stosb
	mov	word [rdi],'32'
	call	search_disassemble_prefix_67
	setz	al
	movzx	eax,al
	lea	rdi,[rdi+rax*2]
	mov	byte [rdi],0
	mov	eax,1
	ret

process_opcode_string_operations_no_b:
	mov	al,'d'
	mov	bl,post_disasm_dword
	call	search_disassemble_prefix_66
	mov	dl,'w'
	mov	cl,post_disasm_word
	cmovz	eax,edx
	cmovz	ebx,ecx
	call	search_disassemble_prefix_REX
	bt	edx,3			; REX.W bit 64-bit operand size
	mov	dl,'q'
	mov	cl,post_disasm_qword
	cmovc	eax,edx
	cmovc	ebx,ecx
	jmp	process_opcode_string_operations_common_end

process_opcode_mov_B0_B7:
; B0-B7
; B000   mov al,0
; B400   mov ah,0
; 41B100 mov r9b,0
; 40B400 mov spl,0
; 41B400 mov r12b,0
	mov	eax,'mov '
	stosd
	lea	rbx,[gpr_8]
	lea	rax,[rbx+8*4]
	lea	rcx,[rax+8*4]
	call	search_disassemble_prefix_REX
	cmovnc	rbx,rax
	bt	edx,0		; REX.B
	cmovc	rbx,rcx
	movzx	eax,byte [rsi]
	sub	al,0B0h
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	al,','
	stosb
	mov	eax,[rsi+1]
	shl	rax,32+16+8
	call	hexa_64_rax
	scasb
	scasb
	mov	byte [rdi],0
	mov	eax,2
	ret

process_opcode_mov_B8_BF:
; 66B84320               mov ax,2043
; 6641BA2425             mov r10w,2524
; B841B7F500             mov eax,00F5B741
; 41BF0C242501           mov r15d,0125240C
; 48B848000C24486B0305   mov rax,05036B48240C0048
; 49B848000C24486B0305   mov r8,05036B48240C0048
; 6640b8486b             mov ax,6b48
; 6648B8486b873821436587 mov rax,8765432138876b48
	mov	eax,'mov '
	stosd
	lea	rbx,[gpr_32]
	lea	rax,[gpr_16]
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_66
	cmovz	rbx,rax
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
	cmovc	rbx,rcx
	bt	edx,0
	setc	al
	movzx	eax,al
	shl	eax,3+2		; eax*8*4 size of halftable
	add	rbx,rax		; not eax but r8d, not rcx but r9, not bp but r13w
	movzx	eax,byte [rsi]
	sub	al,0B8h
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	al,','
	stosb

	mov	ebx,4		; default value size = 4 (doubleword)
	lea	ecx,[rbx-2]	; 66 prefix value size = 2 (word)
	call	search_disassemble_prefix_66
	cmovz	ebx,ecx
	mov	cl,8		; 48 prefix value size = 8 (quadword)
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
	cmovc	ebx,ecx
	mov	rax,[rsi+1]
	mov	cl,8
	sub	cl,bl
	shl	cl,3		; CL*8
	shl	rax,cl
	call	hexa_64_rax
	lea	rdi,[rdi+rbx*2]
	mov	byte [rdi],0
	lea	eax,[rbx+1]	; opcode size = 1 + value size
	ret

; C2 RET word    C3 RET
process_opcode_return:
	sub	al,0C2h
	xchg	ecx,eax
	mov	eax,'ret '
	stosd
process_opcode_return_common_end:
	jrcxz	process_opcode_return_value
	xor	eax,eax
	mov	[rdi-1],al
	inc	eax		; eax=1
	ret

process_opcode_return_value:
;	movsx	rax,word [rsi+1]
	mov	eax,dword [rsi+1]
	shl	rax,32 + 16
	call	hexa_64_rax
	mov	byte [rdi+4],0
	mov	eax,3
	ret

process_undefined_opcode_LES_LDS_C4_C5:
	call	process_undefined_opcode_begin_string
	bt	eax,0
	setc	cl
	mov	eax,'les '
	sub	ah,cl		; change 'les ' to 'lds '
	stosd
	jmp	process_opcode_big_fucking_gun_reg_mem_d

process_opcode_mov_C6_C7:
;c60000          mov byte [rax],00
;66c7008d41      mov word [rax],418d
;c700cbad2523    mov dword [rax],2325adcb
;67c7008d268d70  mov dword [eax],708d268d
	sub	al,0C6h
	xchg	ecx,eax
	mov	eax,'mov '
	stosd
	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_mov_C6_C7_skip_operand_size
process_opcode_mov_C6_C7_common_begin:
	mov	eax,'byte'
	mov	byte [post_disasm_action],post_disasm_byte
	cmp	cl,0
	jz	process_opcode_mov_C6_C7_store_operand_size
	mov	byte [post_disasm_action],post_disasm_word
	call	search_disassemble_prefix_66
	jz	process_opcode_mov_C6_C7_store_operand_word
	mov	al,'d'
	mov	bl,'q'
	mov	byte [post_disasm_action],post_disasm_dword
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size
	cmovc	eax,ebx
	adc	byte [post_disasm_action],0	; change post_disasm_dword -> post_disasm_qword
	stosb			; prepare fist letter of 'dword' or 'qword'
process_opcode_mov_C6_C7_store_operand_word:
	mov	eax,'word'
process_opcode_mov_C6_C7_store_operand_size:
	stosd
	mov	al,' '
	stosb
process_opcode_mov_C6_C7_skip_operand_size:

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_80_83_table]
	call	qword [rbx+rax*8]
; note only first 2 procedures of process_opcode_80_83_table will be used 

	push	rax		; opcode size

	mov	byte [rdi],','	; we mustn't destroy eax !!!
	scasb

; we need eax to get value at [rsi+rax*1] in this subroutine:
	call	process_opcode_mov_C6_C7_value
; return value size (1,2,4) in ebx
	mov	byte [rdi],0

	pop	rax		; opcode size
	add	eax,ebx		; add value size
	ret

process_opcode_mov_C6_C7_value:
; 1. get value size (byte/word/dword)
	mov	ebx,1		; size 1 byte
	bt	dword [rsi],0
	jnc	process_opcode_mov_C6_C7_value_have_size
	mov	ebx,4		; size 1 dword
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size overwrites 66 prefix
	jc	process_opcode_mov_C6_C7_value_have_size
	mov	ecx,2		; size 1 word
	call	search_disassemble_prefix_66
	cmovz	ebx,ecx
process_opcode_mov_C6_C7_value_have_size:
;	mov	eax,[rsi+rax*1]
; correction for instruction like 48C7C0FFFFFFFF mov rax,FFFFFFFFFFFFFFFF
	movsxd	rax,dword [rsi+rax*1]
; if REX.W and rax is negative and ebx=4 put result 8 into BL
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size
;	jnc	process_opcode_mov_C6_C7_value_do_it
; without jump:
	setc	cl
	movzx	ecx,cl
	lea	ecx,[rcx*4+4]	; CL*4+4
;	mov	cl,8		; size 1 qword
	cmp	bl,4
	cmovnz	ecx,ebx		; overwrite ecx=8 with original ebx if original ebx isn't 4
	or	rax,rax
	cmovs	ebx,ecx		; move 8 into bl in case negative RAX (Sign flag)
; correction for 48C7C0FFFFFFFF mov rax,FFFFFFFFFFFFFFFF (old bad was mov rax,FFFFFFFF) done !
;
;process_opcode_mov_C6_C7_value_do_it:
	mov	cl,8
	sub	cl,bl
	shl	cl,3		; CL*8
	shl	rax,cl
	call	hexa_64_rax
	lea	rdi,[rdi+rbx*2]
; we must restore old BL=4 in case of BL=8 because we processed 4 opcode bytes, no 8
	mov	cl,4
	cmp	bl,8
	cmovz	ebx,ecx
	mov	byte [rdi],0
	ret
; using hexa_64_rax_sign_negative is faster
;process_opcode_mov_C6_C7_value_have_size:
;	mov	eax,[rsi+rax*1]
;	mov	cl,8
;	sub	cl,bl
;	shl	cl,3		; CL*8
;	shl	rax,cl
;	call	hexa_64_rax_sign_negative	; instead of call hexa_64_rax
;	lea	rdi,[rdi+rbx*2]
;	mov	byte [rdi],0
;	ret

; CA RETF word    CB RETF
process_opcode_return_far:
	sub	al,0CAh
	xchg	ecx,eax
	mov	eax,'retf'
	stosd
	mov	al,' '
	stosb
	jmp	process_opcode_return_common_end

; C8 ENTER
process_opcode_enter:
	mov	rax,'enter '
	stosq
	dec	rdi
	dec	rdi
	mov	eax,[rsi+1]
	shl	rax,32+16
	call	hexa_64_rax
	add	rdi,4
	mov	al,','
	stosb
	mov	eax,[rsi+3]
	shl	rax,32+16+8
	call	hexa_64_rax
	mov	byte [rdi+2],0
	mov	eax,4
	ret

; C9 LEAVE
process_opcode_leave:
	mov	rax,'leave'
	jmp	process_opcode_common_end_55

; CC  int3
process_opcode_int3:
	mov	eax,'int3'
	jmp	process_opcode_common_end_55

; CD  int
process_opcode_int:
	mov	eax,'int '
	stosd
	mov	al,[rsi+1]
	shl	rax,32+16+8	; found by Alorent from board.flatassembler.net, thanks you !!!	old bad: shr	rax,32+16+8
	call	hexa_64_rax
	mov	byte [rdi+2],0
	mov	eax,2
	ret

; CE  INTO note, AMD64 long mode undefined opcode
process_opcode_into:
	call	process_undefined_opcode_begin_string
	mov	eax,'into'
	jmp	process_opcode_common_end_55

; CF  iret
process_opcode_iret:
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size
	mov	rax,'iretq'
	mov	ecx,'iret'
	cmovnc	rax,rcx
	jmp	process_opcode_common_end_55

process_opcode_shift_rotate_C0_C1_D0_D3:
;C0-C1 D0-D3: ROL ROR RCL RCR SHL SHR SAL* SAR 
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	lea	rbx,[instructions_shift_rotate]
	mov	eax,[rbx+rax*4]
	stosd

	cmp	byte [rsi+1],0C0h	; Will we shift/rotate registers instead of memory ?
	jnc	process_opcode_shift_rotate_C0_C1_D0_D3_skip_operand_size
	mov	ebx,'byte'
	mov	byte [post_disasm_action],post_disasm_byte
	bt	dword [rsi],0
	jnc	process_opcode_shift_rotate_C0_C1_D0_D3_store_operand_size
	mov	ebx,'word'
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
; REX.W overwrites 66 prefix
	jc	process_opcode_shift_rotate_C0_C1_D0_D3_skip_prefix_66
	mov	byte [post_disasm_action],post_disasm_word
	call	search_disassemble_prefix_66
	jz	process_opcode_shift_rotate_C0_C1_D0_D3_store_operand_size
process_opcode_shift_rotate_C0_C1_D0_D3_skip_prefix_66:
	mov	byte [post_disasm_action],post_disasm_dword
	mov	al,'d'
	mov	cl,'q'
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
	cmovc	eax,ecx
	adc	byte [post_disasm_action],0	; change post_disasm_dword -> post_disasm_qword
	stosb
process_opcode_shift_rotate_C0_C1_D0_D3_store_operand_size:
	xchg	ebx,eax
	stosd
	mov	al,' '
	stosb
process_opcode_shift_rotate_C0_C1_D0_D3_skip_operand_size:

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_80_83_table]
	call	qword [rbx+rax*8]
; return opcode size in rax

	mov	byte [rdi],','	; we mustn't destroy eax !!!
	scasb

	mov	dl,byte [rsi]
	sub	dl,0C0h
	xor	ebx,ebx
	mov	ecx,1
	cmp	dl,1
	jbe	process_opcode_shift_rotate_C0_C1_D0_D3_value

process_opcode_shift_rotate_C0_C1_D0_D3_no_value:
	bt	edx,1
	jc	process_opcode_shift_rotate_C0_C1_D0_D3_CL

process_opcode_shift_rotate_C0_C1_D0_D3_1:
	mov	byte [rdi],'1'
	scasb
	mov	byte [rdi],0
	ret

process_opcode_shift_rotate_C0_C1_D0_D3_CL:
	mov	byte [rdi],'c'
	scasb
	mov	byte [rdi],'l'
	scasb
	mov	byte [rdi],0
	ret

process_opcode_shift_rotate_C0_C1_D0_D3_value:
	push	rax
	mov	eax,[rsi+rax*1]
	shl	rax,8*7
	call	hexa_64_rax
	scasb
	scasb
	mov	byte [rdi],0
	pop	rax		; opcode size
	inc	eax		; opcode size
	ret

; 27 DAA undefined in AMD64 long mode
process_undefined_opcode_daa:
; 2F DAS undefined in AMD64 long mode
process_undefined_opcode_das:
; 37 AAA undefined in AMD64 long mode
process_undefined_opcode_aaa:
; 3F AAS undefined in AMD64 long mode
process_undefined_opcode_aas:
	sub	al,27h
	shr	eax,1
	lea	rbx,[process_undefined_opcode_daa_das_aaa_aas_table]
	mov	eax,[rbx+rax*1]
	jmp	process_undefined_opcode_aam_aad_common_end
; D4 AAM
process_undefined_opcode_aam:
	mov	eax,'aam '
	jmp	process_undefined_opcode_aam_aad_common_end
; D5 AAD
process_undefined_opcode_aad:
	mov	eax,'aad '
process_undefined_opcode_aam_aad_common_end:
	call	process_undefined_opcode_begin_string
	stosd
	mov	al,byte [rsi+1]
	shl	rax,32+16+8
	call	hexa_64_rax
	mov	byte [rdi+2],0
	mov	eax,2
	ret

; D6 SETALC
process_undefined_opcode_setalc:
	call	process_undefined_opcode_begin_string
	mov	rax,'setalc'
	jmp	process_opcode_common_end_55

; D7 XLAT   67D7 xlat byte [ebx]
process_opcode_xlat:
	mov	byte [post_disasm_action],post_disasm_xlat
	or	byte [post_disasm_flag],post_disasm_flag_reg0
	mov	rax,'xlat byt'
	stosq
	mov	eax,'e ['
	stosd
	dec	rdi
	mov	eax,'rbx]'
	mov	ecx,'ebx]'
	call	search_disassemble_prefix_67
	cmovz	eax,ecx
	jmp	process_opcode_common_end_55

process_opcode_FPU_D8:
	lea	rbx,[opcode_FPU_D8_instructions]
	movzx	ecx,byte [rsi+1]
	and	cl,00111000b
	mov	rax,[rbx+rcx*1]
	mov	[rdi],rax
	bsr	rcx,rax		; find highest nonzero bit in rax and store result into rcx
	add	cl,7
	shr	ecx,3		; ecx/8 because each byte has 8 bits
	add	rdi,rcx

	lea	rcx,[rdi]
	mov	eax,'st0,'
	stosd
	cmp	byte [rsi+1],0C0h
	cmovc	rdi,rcx
process_opcode_FPU_D8_common_begin:
	jnc	process_opcode_FPU_D8_skip_operand_size
process_opcode_FPU_D8_put_operand_size:
	mov	rax,'dword '
	mov	byte [post_disasm_action],post_disasm_fpu_dword
	stosq
	dec	rdi
	dec	rdi
process_opcode_FPU_D8_skip_operand_size:

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_FPU_D8_table]
	call	qword [rbx+rax*8]
	ret

process_opcode_D8_FPU_register:
	mov	ax,'st'
	stosw
	mov	al,[rsi+1]
	and	al,00000111b
	add	al,'0'
	stosb
process_opcode_FPU_D8_return:
	mov	byte [rdi],0	; 0-terminator
	mov	eax,2		; opcode size
	ret

process_opcode_FPU_D9:
	lea	rbx,[opcode_FPU_D9_instructions_00_BF]
	movzx	eax,byte [rsi+1]
	cmp	al,0C0h
	jnc	process_opcode_FPU_D9_C0_FF
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax		; find highest nonzero bit in rax and store result in rax
	add	al,7
	shr	eax,3		; eax/8 because each byte has 8 bits
	add	rdi,rax
	bt	dword [rsi+1],5
	cmc			; CoMplement Carry flag
	jmp	process_opcode_FPU_D8_common_begin
	; note that only first 3 subroutines from process_opcode_FPU_D8_table will be used !

process_opcode_FPU_D9_C0_FF:
	lea	rbx,[rbx+8*8]
	cmp	al,0E0h
	jnc	process_opcode_FPU_D9_E0_FF
process_opcode_FPU_D9_C0_DF:
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	movzx	eax,byte [rsi+1]
	mov	bl,al
	and	bl,11111000b
	cmp	bl,0D0h
	jnz	process_opcode_D8_FPU_register
process_opcode_FPU_D9_D0_D8:	; fnop
	dec	rdi
	jmp	process_opcode_FPU_D8_return

process_opcode_FPU_D9_E0_FF:
	lea	rbx,[rbx+8*4]
	and	al,00011111b
	mov	rax,[rbx+rax*8]
process_opcode_FPU_D9_common_end:
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	jmp	process_opcode_FPU_D8_return

process_opcode_FPU_DA:
	lea	rbx,[opcode_FPU_DA_instructions]
	movzx	eax,byte [rsi+1]
	cmp	al,0C0h
	jnc	process_opcode_FPU_DA_C0_FF
process_opcode_FPU_DA_00_BF:
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax

	mov	rax,'dword '
	mov	byte [post_disasm_action],post_disasm_fpu_sig_dword
	stosq
	dec	rdi
	dec	rdi

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_FPU_D8_table]	; only first 3 subroutines will be used
	call	qword [rbx+rax*8]
	ret

process_opcode_FPU_DA_C0_FF:
	lea	rbx,[rbx+8*8]
	cmp	al,0E0h
	jnc	process_opcode_FPU_DA_E0_FF
process_opcode_FPU_DA_C0_DF:
	and	al,00111000b
	mov	rax,[rbx+rax*1]
process_opcode_FPU_DA_C0_DF_common_begin:
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax

process_opcode_FPU_DA_C0_DF_common_begin_2:
	mov	eax,'st0,'
	stosd
	mov	ax,'st'
	stosw
	mov	al,[rsi+1]
	and	al,00000111b
	add	al,'0'
	stosb
	jmp	process_opcode_FPU_D8_return

process_opcode_FPU_DA_E0_FF:
	cmp	al,0E9h
	mov	rax,'fucompp'
	mov	ebx,'f???'
	mov	ecx,1
	mov	edx,4
	cmovnz	rax,rbx
	cmovnz	ecx,edx
	stosq
	sub	rdi,rcx
	jmp	process_opcode_FPU_D8_return

process_opcode_FPU_DB:
	lea	rbx,[opcode_FPU_DB_instructions]
	movzx	eax,byte [rsi+1]
	cmp	al,0C0h
	jnc	process_opcode_FPU_DB_C0_FF
process_opcode_FPU_DB_00_BF:
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax

	mov	rax,'dword '
	mov	rbx,'tbyte '
	mov	cl,post_disasm_fpu_sig_dword
	mov	dl,post_disasm_fpu_tbyte
	bt	dword [rsi+1],5
	cmovc	rax,rbx
	cmovc	ecx,edx
	stosq
	mov	byte [post_disasm_action],cl
	dec	rdi
	dec	rdi

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_FPU_D8_table]	; only first 3 subroutines will be used
	call	qword [rbx+rax*8]
	ret

process_opcode_FPU_DB_C0_FF:
	lea	rbx,[rbx+8*8]
	cmp	al,0E0h
	jnc	process_opcode_FPU_DB_E0_FF
process_opcode_FPU_DB_C0_DF:
	and	al,00111000b
	mov	rax,[rbx+rax*1]
process_opcode_FPU_DB_C0_DF_common_begin:
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	mov	al,' '
	stosb
	jmp	process_opcode_FPU_DA_C0_DF_common_begin_2
process_opcode_FPU_DB_E0_FF:
	cmp	al,0E8h
	jnc	process_opcode_FPU_DB_E8_FF
process_opcode_FPU_DB_E0_EF:
	lea	rbx,[rbx+8*4]
	and	al,00000111b
	mov	rax,[rbx+rax*8]
process_opcode_FPU_DB_E0_EF_common_begin:
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	mov	al,' '
	stosb
	jmp	process_opcode_FPU_D8_return
process_opcode_FPU_DB_E8_FF:
	cmp	al,0F8h
	jnc	process_opcode_FPU_DB_F8_FF
process_opcode_FPU_DB_E8_F7:
	bt	eax,4
	mov	rax,'fucomi '
	mov	rbx,'fcomi '
	cmovc	rax,rbx
	jmp	process_opcode_FPU_DA_C0_DF_common_begin
process_opcode_FPU_DB_F8_FF:
	cmp	al,0FCh
	mov	rax,'frint2'
	mov	ebx,'f???'
	cmovnz	rax,rbx
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	jmp	process_opcode_FPU_D8_return

process_opcode_FPU_DC:
	lea	rbx,[opcode_FPU_DC_instructions]
	movzx	eax,byte [rsi+1]
	cmp	al,0C0h
	jnc	process_opcode_FPU_DC_C0_FF
process_opcode_FPU_DC_00_BF:
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax

	mov	rax,'qword '
	mov	byte [post_disasm_action],post_disasm_fpu_qword
	stosq
	dec	rdi
	dec	rdi

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_FPU_D8_table]	; only first 3 subroutines will be used
	call	qword [rbx+rax*8]
	ret

process_opcode_FPU_DC_C0_FF:
	lea	rbx,[rbx+8*8]
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax

	mov	eax,'st0,'
	stosd
	mov	al,[rsi+1]
	and	al,00000111b
	add	[rdi-2],al
	movzx	eax,byte [rsi+1]
	and	al,11110000b
	cmp	al,0D0h
	mov	al,0
	setz	al
	sub	rdi,rax
	or	rax,rax
	jnz	process_opcode_FPU_D8_return	; fcom st(1) fcomp st(3)...

	mov	eax,'st0'
	stosd
	dec	rdi
	mov	eax,2
	ret

process_opcode_FPU_DD:
	lea	rbx,[opcode_FPU_DD_instructions]
	movzx	eax,byte [rsi+1]
	cmp	al,0C0h
	jnc	process_opcode_FPU_DD_C0_FF
process_opcode_FPU_DD_00_BF:
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax

	mov	rax,'qword '
	mov	[rdi],rax
;	mov	eax,6
;	mov	ebx,0
	xor	ebx,ebx
	lea	eax,[rbx+6]
	mov	cl,post_disasm_fpu_qword
	mov	dl,post_disasm_nothing
	bt	dword [rsi+1],5
	cmovc	eax,ebx
	cmovc	ecx,edx
	add	rdi,rax
	mov	byte [post_disasm_action],cl

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_FPU_D8_table]	; only first 3 subroutines will be used
	call	qword [rbx+rax*8]
	ret

process_opcode_FPU_DD_C0_FF:
	lea	rbx,[rbx+8*8]
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	cmp	byte [rsi+1],0F0h
	jnc	process_opcode_FPU_DD_C0_FF_common_end

	mov	eax,'st0'
	stosd
	dec	rdi
	movzx	eax,byte [rsi+1]
	and	al,00000111b
	add	byte [rdi-1],al
process_opcode_FPU_DD_C0_FF_common_end:
	mov	eax,2
	ret

process_opcode_FPU_DE:
	lea	rbx,[opcode_FPU_DE_instructions]
	movzx	eax,byte [rsi+1]
	cmp	al,0C0h
	jnc	process_opcode_FPU_DE_C0_FF
process_opcode_FPU_DE_00_BF:
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax

	mov	eax,'word'
	mov	byte [post_disasm_action],post_disasm_fpu_sig_word
	stosd
	mov	al,' '
	stosb

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_FPU_D8_table]	; only first 3 subroutines will be used
	call	qword [rbx+rax*8]
	ret

process_opcode_FPU_DE_C0_FF:
	lea	rbx,[rbx+8*8]
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	bl,byte [rsi+1]
	and	bl,11111000b
	cmp	bl,0D8h
	jz	process_opcode_FPU_DE_D9
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	mov	eax,'st0'
	stosd
	dec	rdi
	mov	al,byte [rsi+1]
	and	al,00000111b
	add	byte [rdi-1],al
	mov	eax,',st0'
	stosd
	xor	eax,eax
	mov	bl,byte [rsi+1]
	and	bl,11111000b
	cmp	bl,0D0h
	mov	ebx,4
	cmovz	eax,ebx
	sub	rdi,rax
process_opcode_FPU_DE_end:
	mov	byte [rdi],0
	mov	eax,2
	ret

process_opcode_FPU_DE_D9:
	mov	bl,byte [rsi+1]
	cmp	bl,0D9h
	mov	ebx,'f???'
	cmovnz	rax,rbx
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	mov	eax,2
	ret

process_opcode_FPU_DF:
	lea	rbx,[opcode_FPU_DF_instructions]
	movzx	eax,byte [rsi+1]
	cmp	al,0C0h
	jnc	process_opcode_FPU_DF_C0_FF
process_opcode_FPU_DF_00_BF:
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax

	mov	rax,'tbyte '
	mov	rbx,'qword '
	mov	cl,post_disasm_fpu_bcd
	mov	dl,post_disasm_fpu_sig_qword
	bt	dword [rsi+1],3
	cmovc	rax,rbx
	cmovc	ecx,edx
	mov	bl,byte [rsi+1]
	and	bl,00111111b
	cmp	bl,20h
	mov	rbx,'word '
	mov	dl,post_disasm_fpu_sig_word
	cmovc	rax,rbx
	cmovc	ecx,edx
	mov	[rdi],rax
	mov	byte [post_disasm_action],cl
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_FPU_D8_table]	; only first 3 subroutines will be used
	call	qword [rbx+rax*8]
	ret

process_opcode_FPU_DF_C0_FF:
	lea	rbx,[rbx+8*8]
	and	al,00111000b
	mov	rax,[rbx+rax*1]
	mov	bl,byte [rsi+1]
	and	bl,11111000b
	cmp	bl,0E0h
	jnc	process_opcode_FPU_DF_E0_FF
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
process_opcode_FPU_DF_C0_FF_common_end:
	mov	eax,'st0'
	stosd
	dec	rdi
	mov	al,[rsi+1]
	and	al,00000111b
	add	[rdi-1],al
process_opcode_FPU_DF_C0_FF_common_end_2:
	mov	byte [rdi],0
	mov	eax,2
	ret
process_opcode_FPU_DF_E0_FF:
	cmp	bl,0E0h
	jz	process_opcode_FPU_DF_E0
	cmp	bl,0F8h
	jz	process_opcode_FPU_DF_E0_common_end
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	mov	eax,'st0,'
	stosd
	jmp	process_opcode_FPU_DF_C0_FF_common_end
process_opcode_FPU_DF_E0:
	mov	ebx,'f???'
	cmp	byte [rsi+1],0E0h
	cmovnz	rax,rbx
process_opcode_FPU_DF_E0_common_end:
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	jmp	process_opcode_FPU_DF_C0_FF_common_end_2

; E0 LOOPNZ
; E1 LOOPZ
; E2 LOOP
; prefix 67 loop ecx, without prefix loop rcx
; loop cx invalid in AMD64 long mode - loop cx to RIP=401023h would cause loop to RIP=001023h
process_opcode_loops:
	sub	al,0E0h
	lea	rbx,[process_opcode_loops_table]
	xchg	ecx,eax
	mov	rax,[rbx+rcx*8]
	stosq
	sub	rdi,rcx
	mov	al,'q'
	mov	cl,'d'
	call	search_disassemble_prefix_67
	cmovz	eax,ecx
	setz	dl
	movzx	edx,dl
	mov	byte [rdi-2],al
	movsx	rax,byte [rsi+1]
	lea	rax,[rax+rdx*1+2]
	lea	rax,[rax+rbp*1]		; smaller instruction than [rbp+rax*1+0]
	call	hexa_64_rax
	mov	byte [rdi+16],0
	mov	qword [post_disasm_addr0],rax
	or	byte [post_disasm_flag],post_disasm_flag_direct
	mov	byte [post_disasm_action],post_disasm_addr
	mov	eax,2
	ret

; E3 JECXZ/JRCXZ
; note that JCXZ make mess in AMD64 long mode.
; E.g. RIP=401056h and JCXZ 401078h make jump to 001078h not to 401078h !
process_opcode_jcxz:
	lea	rbx,[process_opcode_jcxz_table]
	xor	ecx,ecx
	call	search_disassemble_prefix_67
	setz	cl
	mov	rax,[rbx+rcx*8]
	stosq
	lea	rdi,[rdi-2]
	movsx	rax,byte [rsi+1]
	add	rax,rcx
	lea	rax,[rbp + rax*1 + 2]
	call	hexa_64_rax
	mov	byte [rdi+16],0
	mov	qword [post_disasm_addr0],rax
	or	byte [post_disasm_flag],post_disasm_flag_direct
	mov	byte [post_disasm_action],post_disasm_addr
	mov	eax,2
	ret

; E4-E7  E4 in al  E5 in eax  E6 out al  E7 out eax
process_opcode_ports_val:
	bt	dword [rsi],1
	jc	process_opcode_ports_val_out
process_opcode_ports_val_in:
	mov	eax,'in '
	stosd
	dec	rdi
	bt	dword [rsi],0
	jc	process_opcode_ports_val_in_d
process_opcode_ports_val_in_b:
	mov	eax,'al,'
	stosd
	dec	rdi
	jmp	process_opcode_ports_val_in_common_end

process_opcode_ports_val_in_d:
	cmp	byte [disassemble_prefix],66h
	jz	process_opcode_ports_val_in_w
	mov	eax,'eax,'
	stosd
	jmp	process_opcode_ports_val_in_common_end

process_opcode_ports_val_in_w:
	mov	eax,'ax,'
	stosd
	dec	rdi

process_opcode_ports_val_in_common_end:
	mov	al,byte [rsi+1]
	bswap	rax
	call	hexa_64_rax
	mov	byte [rdi+2],0
	mov	eax,2
	ret

process_opcode_ports_val_out:
	mov	eax,'out '
	stosd
	mov	al,byte [rsi+1]
	bswap	rax
	call	hexa_64_rax
	bt	dword [rsi],0
	jc	process_opcode_ports_val_out_d
process_opcode_ports_val_out_b:
	mov	eax,',al'
	jmp	process_opcode_ports_val_out_common_end

process_opcode_ports_val_out_d:
	cmp	byte [disassemble_prefix],66h
	jz	process_opcode_ports_val_out_w
	mov	eax,',eax'
	jmp	process_opcode_ports_val_out_common_end

process_opcode_ports_val_out_w:
	mov	eax,',ax'

process_opcode_ports_val_out_common_end:
	mov	dword [rdi+2],eax
	mov	byte [rdi+6],0
	mov	eax,2
	ret

; E8 call
process_opcode_call:
	mov	al,'c'
	stosb
	mov	eax,'all '
process_opcode_call_common_end:
	stosd
	movsxd	rax,dword [rsi+1]
	lea	rax,[rbp + rax*1 + 5]
	call	hexa_64_rax
	mov	byte [rdi+16],0
	mov	qword [post_disasm_addr0],rax
	or	byte [post_disasm_flag],post_disasm_flag_direct
	mov	byte [post_disasm_action],post_disasm_addr
	mov	eax,5
	ret

; E9 jmp near
process_opcode_jmp_near:
	mov	eax,'jmp '
	jmp	process_opcode_call_common_end

; EA jmp far
process_opcode_jmp_far:
	mov	eax,'jmp '
	stosd
	mov	ax,word [rsi+1+4]
	shl	rax,48
	call	hexa_64_rax
	scasd
	mov	al,':'
	stosb
	mov	eax,dword [rsi+1]
	shl	rax,32
	call	hexa_64_rax
	mov	byte [rdi+8],0
	mov	eax,1+4+2
	ret

; EB jmp short
process_opcode_jmp_short:
	mov	eax,'jmp '
process_opcode_jmp_short_common_end:
	stosd
process_opcode_jmp_short_common_end_L5:
	movsx	rax,byte [rsi+1]
	lea	rax,[rbp + rax*1 + 2]
	call	hexa_64_rax
	mov	byte [rdi+16],0
	mov	qword [post_disasm_addr0],rax
	or	byte [post_disasm_flag],post_disasm_flag_direct
	mov	byte [post_disasm_action],post_disasm_addr
	mov	eax,2
	ret

; EC-EF
process_opcode_ports_dx:
	bt	dword [rsi],1
	jc	process_opcode_ports_dx_out

process_opcode_ports_dx_in:
	mov	eax,'in '
	stosd
	dec	rdi
	bt	dword [rsi],0
	jc	process_opcode_ports_dx_in_d
process_opcode_ports_dx_in_b:
	mov	eax,'al,'
	stosd
	dec	rdi
	jmp	process_opcode_ports_dx_in_common_end

process_opcode_ports_dx_in_d:
	cmp	byte [disassemble_prefix],66h
	jz	process_opcode_ports_dx_in_w
	mov	eax,'eax,'
	stosd
	jmp	process_opcode_ports_dx_in_common_end

process_opcode_ports_dx_in_w:
	mov	eax,'ax,'
	stosd
	dec	rdi

process_opcode_ports_dx_in_common_end:
	mov	dword [rdi],'dx'
	mov	eax,1
	ret

process_opcode_ports_dx_out:
	mov	eax,'out '
	stosd
	mov	eax,'dx,'
	stosd
	dec	rdi
	bt	dword [rsi],0
	jc	process_opcode_ports_dx_out_d
process_opcode_ports_dx_out_b:
	mov	eax,'al'
	jmp	process_opcode_ports_dx_out_common_end

process_opcode_ports_dx_out_d:
	cmp	byte [disassemble_prefix],66h
	jz	process_opcode_ports_dx_out_w
	mov	eax,'eax'
	jmp	process_opcode_ports_dx_out_common_end

process_opcode_ports_dx_out_w:
	mov	eax,'ax'

process_opcode_ports_dx_out_common_end:
	stosd
	mov	eax,1
	ret

process_opcode_lock_F0:
	mov	eax,'lock'
	stosd
	jmp	process_opcode_repnz_repz_common_end

; F1 icebp int1
process_opcode_int1:
	mov	eax,'int1'
	jmp	process_opcode_common_end_55

; F2 REPNZ
process_opcode_repnz:
	call	find_first_no_prefix_byte
	jc	process_opcode_F2_prefix
	cmp	dl,0Fh
	jz	process_opcode_F2_prefix

	xor	edx,edx
	bsr	edx,dword [disassemble_prefix]
	add	dl,7
	shr	dl,3
; DL = number of prefixes preceeding byte at [rsi]
	add	dl,cl
	cmp	dl,InstructionMaxPrefixSize
	jnbe	process_opcode_F2_prefix	; illegal instruction, exceedes InstructionMaxPrefixSize

process_opcode_repnz_no_prefix:
	mov	eax,'repn'
	stosd
	mov	al,'z'
	stosb
	jmp	process_opcode_repnz_repz_common_end

; F3 REPZ
process_opcode_repz:
	call	find_first_no_prefix_byte
	jc	process_opcode_F3_prefix
	cmp	dl,0Fh
	jz	process_opcode_F3_prefix
	cmp	byte [rsi+1],90h
	jz	process_opcode_F3_prefix	; F390 PAUSE instruction

	xor	edx,edx
	bsr	edx,dword [disassemble_prefix]
	add	dl,7
	shr	dl,3
; DL = number of prefixes preceeding byte at [rsi]
	add	dl,cl
	cmp	dl,InstructionMaxPrefixSize
	jnbe	process_opcode_F3_prefix	; illegal instruction, exceedes InstructionMaxPrefixSize

	mov	eax,'repz'
	stosd
process_opcode_repnz_repz_common_end:
	mov	al,' '
	stosb

	inc	rsi

	inc	rbp		; absolute address for calculate offset in opcode

	movzx	eax,byte [rsi]
	call	qword [rbx+rax*8]

	dec	rbp

	inc	eax		; increase opcode size by 1
	ret

process_opcode_F4_F5:
;	movzx	eax,byte [rsi]
	lea	rbx,[instructions_F4_F5]
	sub	al,0F4h		; F4 HLT, F5 CMC
	jmp	process_opcode_common_end_52

process_opcode_F6_F7:
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	lea	rbx,[group3_instruction_table]
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	mov	al,' '
	stosb

	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_F6_F7_skip_operand_size
	mov	ebx,'byte'
	mov	byte [post_disasm_action],post_disasm_byte
	bt	dword [rsi],0
	jnc	process_opcode_F6_F7_store_operand_size
	mov	ebx,'word'
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
; REX.W overwrites 66 prefix
	jc	process_opcode_F6_F7_skip_prefix_66
	mov	byte [post_disasm_action],post_disasm_word
	call	search_disassemble_prefix_66
	jz	process_opcode_F6_F7_store_operand_size
process_opcode_F6_F7_skip_prefix_66:
	mov	byte [post_disasm_action],post_disasm_dword
	mov	al,'d'
	mov	cl,'q'
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
	cmovc	eax,ecx
	adc	byte [post_disasm_action],0	; change post_disasm_dword -> post_disasm_qword
	stosb
process_opcode_F6_F7_store_operand_size:
	xchg	ebx,eax
	stosd
	mov	al,' '
	stosb
process_opcode_F6_F7_skip_operand_size:

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_F6_F7_table]
	call	qword [rbx+rax*8]

	mov	cl,byte [rsi+1]
	and	cl,00111000b
	cmp	cl,10h
	jc	process_opcode_F6_F7_test

	ret

process_opcode_F6_F7_test:
	push	rax
	mov	byte [rdi],','
	scasb
	mov	eax,[rsi+rax*1]
	movzx	ebx,byte [rsi]
; we transform F6 -> 1 , F7 -> 4
	mov	ecx,1
	sub	bl,0F6h
	shl	ebx,2
	cmovz	ebx,ecx
	mov	cl,2		; word size for 66 prefix for opcode F7 (66F7)
	cmovz	ecx,ebx
	call	search_disassemble_prefix_66
	cmovz	ebx,ecx

	mov	cl,8
	sub	cl,bl
	shl	cl,3		; CL*8
	shl	rax,cl
	call	hexa_64_rax
	lea	rdi,[rdi+rbx*2]
	mov	byte [rdi],0

	pop	rax		; opcode size
	add	rax,rbx
	ret

process_opcode_F6_F7_register:
	bt	dword [rsi],0
	jc	process_opcode_F6_F7_register_wdq
process_opcode_F6_F7_register_b:
	lea	rbx,[gpr_8]
	lea	rcx,[gpr_8 + 8*4]
	call	search_disassemble_prefix_REX
	cmovnc	rbx,rcx
	lea	rcx,[gpr_8 + 8*4 + 8*4]
;	bt	edx,2		; REX.R
; wrong, this is correct:
	bt	edx,0		; REX.B bit
	cmovc	rbx,rcx
process_opcode_F6_F7_reg_have_it:
	movzx	eax,byte [rsi + 1]
	and	al,00000111b
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	byte [rdi],0	; don't increase rdi
	mov	eax,2		; opcode size
	ret

process_opcode_F6_F7_register_wdq:
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W
	cmovc	rbx,rcx
;	bt	edx,2		; REX.R
; wrong, this is correct:
	bt	edx,0		; REX.B bit
	setc	cl
	movzx	ecx,cl
	shl	ecx,3+2
	add	rbx,rcx
	jmp	process_opcode_F6_F7_reg_have_it

process_opcode_set_flag_F8_FD:
	movzx	eax,byte [rsi]
	lea	rbx,[instructions_set_flag_F8_FD]
	sub	al,0F8h		; F8 begin CLC, FD end STD
process_opcode_common_end_52:
	mov	eax,[rbx+rax*4]
process_opcode_common_end_55:
	stosq
process_opcode_common_end_58:
	mov	eax,1		; opcode size = 1 byte
	ret

process_opcode_inc_dec_FE_byte:
	mov	eax,'inc '
	mov	ebx,'dec '
	bt	dword [rsi+1],3
	cmovc	eax,ebx
	stosd

	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_inc_dec_FE_byte_skip_operand_size
	mov	eax,'byte'
	mov	byte [post_disasm_action],post_disasm_byte
	stosd
	mov	al,' '
	stosb
process_opcode_inc_dec_FE_byte_skip_operand_size:

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_80_83_table]
	call	qword [rbx+rax*8]
	ret

process_opcode_FF:
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	mov	ecx,eax
	lea	rbx,[opcode_FF_instruction_table]
	mov	eax,[rbx+rax*4]
	stosd
	shr	eax,24
	setz	al
	sub	rdi,rax
	mov	al,' '
	stosb
	cmp	cl,2
	jc	process_opcode_FF_inc_dec
	cmp	cl,6
	jc	process_opcode_FF_call_jmp

process_opcode_FF_push:
	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_FF_push_skip_operand_size
	xor	eax,eax
	mov	byte [rdi],'q'
	call	search_disassemble_prefix_66
	setnz	al
	add	rdi,rax
	lea	edx,[rax+rax*1+post_disasm_word]	; post_disasm_word -> post_disasm_qword
	mov	byte [post_disasm_action],dl
	mov	eax,'word'
	stosd
	mov	al,' '
	stosb
process_opcode_FF_push_skip_operand_size:
	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_q_w]
	call	qword [rbx+rax*8]
	ret

process_opcode_FF_inc_dec:
	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_FF_inc_dec_skip_operand_size
	mov	al,'d'
	mov	bl,'q'
	call	search_disassemble_prefix_REX
	bt	edx,3
	cmovc	eax,ebx
	mov	dl,post_disasm_dword
	adc	dl,0
	stosb
	cmp	dl,post_disasm_dword
	jnz	process_opcode_FF_inc_dec_L3
	call	search_disassemble_prefix_66
	setz	al
	movzx	eax,al
	sub	rdi,rax
	sub	dl,al
process_opcode_FF_inc_dec_L3:
	mov	eax,'word'
	stosd
	mov	al,' '
	stosb
	mov	byte [post_disasm_action],dl
process_opcode_FF_inc_dec_skip_operand_size:
	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]	; table_d not table_q !!!
	call	qword [rbx+rax*8]
	ret

process_opcode_FF_call_jmp:
	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_FF_push_skip_operand_size
	mov	al,'q'
	cmp	cl,6
	mov	dl,post_disasm_qword
	jz	process_opcode_FF_call_jmp_have_operand_size	; push allways qword
	bt	ecx,0
	jnc	process_opcode_FF_call_jmp_have_operand_size
	mov	al,'f'		; fword (word segment:dword address)
	mov	bl,'d'		; dword (word segment:word address), e.g. 66FF28 jmp dword [rax])
				; note that jmp dword [rax] is illegal instruction (dword address, not as above word seg:word address)
	mov	dl,post_disasm_dword
	call	search_disassemble_prefix_66
	cmovz	eax,ebx
process_opcode_FF_call_jmp_have_operand_size:
	stosb
	mov	byte [post_disasm_action],dl
	mov	eax,'word'
	stosd
	mov	al,' '
	stosb
	jmp	process_opcode_FF_push_skip_operand_size

align 16
process_opcode_big_fucking_gun_reg_q:
	movzx	eax,byte [rsi+1]
	and	al,111b

	lea	rbx,[gpr_64]
process_opcode_big_fucking_gun_reg_q_common:
	call	search_disassemble_prefix_REX
	bt	edx,0		; REX.B		;2		; REX.R
	setc	dl
	shl	edx,3+2		; rdx*8*4  ->  no dx but r10w , no eax but r8d , no rcx but r9
	add	rbx,rdx

	jmp	process_opcode_F6_F7_reg_have_it

process_opcode_reg_q_w:
	movzx	eax,byte [rsi+1]
	and	al,111b

	lea	rbx,[gpr_64]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	jmp	process_opcode_big_fucking_gun_reg_q_common

process_opcode_0F00:
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	lea	rbx,[opcode_0F00_instruction_table]
	mov	eax,[rbx+rax*4]
	stosd
	shr	eax,8*3
	setnz	al
	lea	rdi,[rdi+rax-1]
	mov	al,' '
	stosb
	mov	eax,'word'
	stosd
	mov	al,' '
	stosb

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
	call	qword [rbx+rax*8]

	ret

process_opcode_0F01:
	movzx	eax,byte [rsi+1]
	cmp	al,0F9h
	jz	process_opcode_0F01F9
	cmp	al,0C1h
	jc	process_opcode_0F01_after_vanderpol
	cmp	al,0C4h
	jbe	process_opcode_0F01_vanderpol
	cmp	al,0C8h
	jc	process_opcode_0F01_after_vanderpol
	cmp	al,0C9h
	jbe	process_opcode_0F01_vanderpol1
	cmp	al,0D0h
	jc	process_opcode_0F01_after_vanderpol
	cmp	al,0D1h
	jbe	process_opcode_0F01_vanderpol2
process_opcode_0F01_after_vanderpol:
	and	al,11111000b
	cmp	al,0D8h
	jz	process_opcode_0F01_pacifica
	and	al,00111000b
	shr	eax,3
	cmp	al,7
	jz	process_opcode_0F01_invlpg_swapgs
	lea	rbx,[opcode_0F01_instruction_table]
	mov	eax,[rbx+rax*4]
	stosd

	mov	al,' '
	stosb

process_opcode_0F01_common_end:
	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
	call	qword [rbx+rax*8]
	ret

process_opcode_0F01F9:
	mov	rax,'rdtscp'
	stosq
	mov	eax,2
	ret

process_opcode_0F01_pacifica:
	movzx	eax,byte [rsi+1]
	and	al,00000111b
	lea	rbx,[pacifica_instructions]
	mov	rax,qword [rbx+rax*8]
	stosq
	mov	eax,2
	ret

process_opcode_0F01_vanderpol:
	sub	al,0C1h
	lea	rbx,[vanderpol_instructions]
	mov	rax,qword [rbx+rax*8]
	stosq
	xor	eax,eax
	stosb
	mov	al,2
	ret

process_opcode_0F01_vanderpol1:
	sub	al,0C8h
	lea	rbx,[vanderpol_instructions1]
	mov	rax,qword [rbx+rax*8]
	stosq
	mov	eax,2
	ret

process_opcode_0F01_vanderpol2:
	sub	al,0D0h
	lea	rbx,[vanderpol_instructions2]
	mov	rax,qword [rbx+rax*8]
	stosq
	mov	eax,2
	ret

process_opcode_0F01_invlpg_swapgs:
	cmp	byte [rsi+1],11000000b
	jnc	process_opcode_0F01_swapgs

process_opcode_0F01_invlpg:
	mov	eax,'invl'
	stosd
	mov	eax,'pg '
	stosd
	dec	rdi
	jmp	process_opcode_0F01_common_end

process_opcode_0F01_swapgs:
	mov	eax,'swap'
	stosd
	mov	eax,'gs'
	stosd
	mov	eax,3-1		; -1 because first 0F was skipped by process_opcode_0F routine
	ret

process_opcode_0F02:
	mov	eax,'lar '
process_opcode_0F02_common_begin:
	stosd

	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	al,3
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	setz	cl
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size
	cmovc	rbx,rcx
	bt	edx,2		; REX.R
	setc	dl
	shl	edx,3+2		; rdx*8*4 ->  no dx but r10w , no eax but r8d , no rcx but r9
	add	rbx,rdx
	mov	eax,dword [rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0
	mov	al,','
	stosb

	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_0F02_L3
	mov	eax,'word'
	stosd
	mov	al,' '
	stosb
process_opcode_0F02_L3:

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_w]
	call	qword [rbx+rax*8]

	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_0F02_L7
	mov	byte [post_disasm_action],post_disasm_word
process_opcode_0F02_L7:
;if post_disasm_word=2
;	setc	cl
;	shl	cl,1
;	mov	byte [post_disasm_action],cl
;else
;	jnc	process_opcode_0F02_L7
;	mov	byte [post_disasm_action],post_disasm_word
;process_opcode_0F02_L7:
;end if

	ret

process_opcode_0F03:
	mov	eax,'lsl '
	jmp	process_opcode_0F02_common_begin

process_opcode_0F04_0F07:
	and	al,00000011b
	mov	cl,al
	lea	rbx,[opcode_0F04_0F07_instruction_table]
	mov	rax,[rbx+rax*8]
	stosq
	mov	eax,2-1
	cmp	cl,11b		; sysret ?
	jz	process_opcode_0F04_0F07_sysretq
	ret
process_opcode_0F04_0F07_sysretq:
	call	search_disassemble_prefix_REX
	bt	edx,3		; REX.W 64 bit operand size
	jc	process_opcode_0F04_0F07_sysretq_done
	mov	byte [rdi-2],'0'	; sysret instead of sysretq
process_opcode_0F04_0F07_sysretq_done:
	ret

process_opcode_0F08_0F0C:
	and	al,00000111b
	lea	rbx,[opcode_0F08_0F0C_instruction_table]
	mov	rax,[rbx+rax*8]
	stosq
	mov	eax,2-1
	ret

process_opcode_prefetch_0F0D:
	mov	rax,'prefetch'
	stosq
	mov	al,' '
	stosb
	mov	[rdi],al
	mov	bl,[rsi+1]
	and	bl,00111000b
	cmp	bl,8
	mov	bl,'w'
	mov	ecx,0
	mov	edx,1
	cmovz	eax,ebx
	cmovz	ecx,edx
; make 'prefetchw '
	mov	[rdi-1],al
	add	rdi,rcx

	mov	eax,'byte'
	stosd
	mov	al,' '
	stosb
	xor	eax,eax
	mov	edx,5
	cmp	byte [rsi+1],0C0h
	cmovnc	eax,edx
	sub	rdi,rax		; skip operand size above C0

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table]
	call	qword [rbx+rax*8]
; note that function returns opcode size in RAX
	mov	byte [rdi],0
	ret

process_opcode_femms_0F0E:
	mov	rax,'femms'
	stosq
	mov	eax,2-1
	ret

process_opcode_3DNow_0F0F:
	push	rdi
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_3DNow_table]
	call	qword [rbx+rax*8]
; note that function returns opcode size in RAX
	mov	al,byte [rsi+rax*1]
	lea	rdi,[opcode_0F0F_3DNow_index]
	lea	rdx,[rdi+1]
	mov	ecx,opcode_0F0F_3DNow_number_of_opcodes + 1
	mov	edx,opcode_0F0F_3DNow_number_of_opcodes
	repnz scasb
	sub	edx,ecx
	pop	rdi

	lea	rax,[opcode_0F0F_3DNow]
	mov	rax,[rax+rdx*8]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	mov	al,' '
	stosb

	mov	eax,'mm0,'
	stosd
	mov	al,[rsi+1]
	and	al,00111000b
	shr	al,3
	add	[rdi-2],al

	mov	rax,'qword '
	mov	byte [post_disasm_action],post_disasm_mm_64_sp
	stosq
	dec	rdi
	dec	rdi
	xor	eax,eax
	mov	edx,6
	cmp	byte [rsi+1],0C0h
	cmovnc	eax,edx
	sub	rdi,rax		; skip operand size

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_3DNow_table]
	call	qword [rbx+rax*8]
; note that function return opcode size in RAX
	mov	byte [rdi],0

	inc	eax
	ret

process_opcode_3DNow_reg:
	mov	eax,'mm0'
	stosd
	dec	rdi
	mov	al,[rsi+1]
	and	al,00000111b
	add	[rdi-1],al
	mov	eax,2
	ret

process_opcode_0F10_0F11:
	lea	rbx,[opcode_0F10_0F11_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0F10_0F11_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl

	bt	dword [rsi],0
	setc	al
;	movzx	eax,al
; eax<100h
	lea	rax,[rax+rcx*2 + 4]
	call	qword [rbx+rax*8]
	ret

process_opcode_0F12:
	lea	rbx,[opcode_0F12_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0F12_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl

	mov	al,[rsi+1]
	cmp	al,0C0h
	setc	al
	or	al,cl
	jnz	process_opcode_0F12_no_movhlps
; patch movlps with movhlps
	inc	rdi
	mov	rax,'movhlps '
	mov	[rdi-8],rax
process_opcode_0F12_no_movhlps:

	call	qword [rbx+4*8+rcx*8]
	ret

process_opcode_movlps_movlpd_movhps_movhpd_0F13_0F17:
	mov	rax,'movlps '
	stosq
	dec	rdi
	xor	eax,eax
	mov	ebx,'h' xor 'l'
	bt	dword [rsi],2
	cmovc	eax,ebx
	xor	[rdi-4],eax
	xor	eax,eax
	mov	ebx,'s' xor 'd'
	call	search_disassemble_prefix_66
	cmovz	eax,ebx
	mov	dl,post_disasm_fpu_qword
	mov	bl,post_disasm_mm_64_sp
	cmovz	edx,ebx
	mov	byte [post_disasm_action],dl
	xor	[rdi-2],eax
	call	process_Mq_Vq
	ret

process_opcode_unpcklps_unpcklpd_unpckhps_unpckhpd_0F14_0F15:
	mov	rax,'unpcklps'
	stosq
	mov	al,' '
	stosb
	xor	eax,eax
	mov	ebx,'h' xor 'l'
	bt	dword [rsi],0
	cmovc	eax,ebx
	lea	rcx,[process_Vo_Wq]
	lea	rdx,[process_Vo_Wo]	; it is displayed like oword, but we know that
					; we use only high qword for unpckhpX (X is s or d)
	cmovc	rcx,rdx
	xor	[rdi-4],eax
	xor	eax,eax
	mov	ebx,'s' xor 'd'
	call	search_disassemble_prefix_66
	cmovz	eax,ebx
	setnz	dl
	add	dl,post_disasm_xmm_128_dp
	mov	byte [post_disasm_action],dl
	xor	[rdi-2],eax
	call	rcx
	ret

process_opcode_0F16:
	lea	rbx,[opcode_0F16_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0F16_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl

	mov	al,[rsi+1]
	cmp	al,0C0h
	setc	al
	or	al,cl
	jnz	process_opcode_0F16_no_movlhps
; patch movhps with movlhps
	inc	rdi
	mov	rax,'movlhps '
	mov	[rdi-8],rax
process_opcode_0F16_no_movlhps:
	call	qword [rbx+4*8+rcx*8]
	ret

process_opcode_prefetch_0F18:
	mov	rax,'prefetch'
	mov	edx,'???'
	mov	cl,[rsi+1]
	and	cl,00111000b
	cmp	cl,20h
	cmovnc	rax,rdx
	stosq
	mov	dl,[rsi+1]
	and	dl,00011000b
	shr	dl,3
	mov	eax,'t0 ' - 100h
	add	ah,dl
	or	dl,dl
	mov	edx,'nta '
	cmovz	eax,edx
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table]
	call	qword [rbx+rax*8]

	ret

process_opcode_nop_0F19_0F1F:
	mov	eax,'nop '
	stosd
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
	movzx	eax,byte [rsi+1]
	shr	eax,6
	call	qword [rbx+rax*8]
	ret

process_opcode_mov_0F20_0F27:
; mod field of mod R/M byte should be 11b
; cmp byte [rsi],0C0h
; jnc invalid...
	mov	eax,'mov '
	stosd
	bt	dword [rsi],1
	jc	process_opcode_mov_0F20_0F27_spec_common
process_opcode_mov_0F20_0F27_common_spec:
	call	process_opcode_mov_0F20_0F27_common
	mov	al,','
	stosb
	call	process_opcode_mov_0F20_0F27_spec
	ret
process_opcode_mov_0F20_0F27_spec_common:
	call	process_opcode_mov_0F20_0F27_spec
	mov	al,','
	stosb
	call	process_opcode_mov_0F20_0F27_common
	ret
process_opcode_mov_0F20_0F27_common:
	movzx	eax,byte [rsi+1]
	and	al,00000111b
	call	search_disassemble_prefix_REX
	and	dl,00000001b		; REX.B
	lea	rbx,[gpr_64]
	shl	edx,3+2
	add	rbx,rdx
	mov	eax,[rbx+rax*4]
	stosd
	dec	rdi
; remove zeros on the end of the string, e.g. for R8, R9
	cmp	byte [rdi-1],1
	sbb	rdi,0
	mov	eax,3-1
	ret
process_opcode_mov_0F20_0F27_spec:
	movzx	eax,byte [rsi]
	and	al,00000111b
	lea	rbx,[special_regs]
	mov	eax,[rbx+rax*2]
	stosw
	call	search_disassemble_prefix_REX
	and	dl,00000100b		; leave REX.R bit (bit 2)
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	al,3
	lea	eax,[rax+rdx*2+'0']	; rdx*2 for shift bit 2 to bit 3
	cmp	al,'9'
	jbe	process_opcode_mov_0F20_0F27_spec_done
	mov	byte [rdi],'1'
	scasb
	sub	al,10
process_opcode_mov_0F20_0F27_spec_done:
	stosb
	mov	byte [rdi],0
	mov	eax,3-1
	ret

process_opcode_movaps_movapd_0F28_0F29:
	mov	rax,'movaps '
	mov	rbx,'movapd '
	call	search_disassemble_prefix_66
	cmovz	rax,rbx
	setnz	dl
	add	dl,post_disasm_xmm_128_dp
	mov	byte [post_disasm_action],dl
	lea	rcx,[process_Vo_Wo]
	lea	rdx,[process_Wo_Vo]
	bt	dword [rsi],0
	cmovc	rcx,rdx
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	call	rcx
	ret

process_opcode_0F2A:
	lea	rbx,[opcode_0F2A_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0F2A_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl
	call	qword [rbx + 8*4 + rcx*8]
	ret

process_opcode_movntps_movntpd_0F2B:
	mov	rax,'movntps '
	mov	rbx,'movntpd '
	call	search_disassemble_prefix_66
	cmovz	rax,rbx
	setnz	dl
	add	dl,post_disasm_xmm_128_dp
	mov	byte [post_disasm_action],dl

	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	call	process_Mo_Vo_without_operand_size
	ret

process_opcode_0F2C_0F2D:
	mov	eax,'cvtt'
	stosd
	bt	dword [rsi],0
	setnc	al
	movzx	eax,al
	lea	rdi,[rdi+rax*1-1]		; cvtt for 0F2C, cvt for 0F2D

	lea	rbx,[opcode_0F2C_0F2D_table]

process_opcode_0F2C_0F2D_0F5A_common:
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0F2C_0F2D_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl
	call	qword [rbx + 8*4 + rcx*8]
	ret

process_opcode_ucomiss_ucomisd_0F2E:
	mov	rax,'ucomiss '
	mov	rbx,'ucomisd '
	mov	dl,post_disasm_fpu_dword
	mov	cl,post_disasm_fpu_qword
	call	search_disassemble_prefix_66
	cmovz	rax,rbx
	cmovz	edx,ecx
	mov	byte [post_disasm_action],dl
	stosq
process_opcode_ucomiss_ucomisd_0F2E_common_begin:
	lea	rbx,[process_Vd_Wd]
	lea	rcx,[process_Vq_Wq]
	cmovz	rbx,rcx
	call	rbx
	ret

process_opcode_comiss_comisd_0F2F:
	mov	rax,'comiss '
	mov	rbx,'comisd '
	add	rdi,7
	call	search_disassemble_prefix_66
	cmovz	rax,rbx
	mov	[rdi-7],rax
	jmp	process_opcode_ucomiss_ucomisd_0F2E_common_begin

process_opcode_0F30_0F36:
process_opcode_0F39:
process_opcode_0F3B_0F3F:
	mov	ebx,7
	bt	eax,3
	cmovc	eax,ebx		; for bytes 0F38-0F3F
	and	al,0000111b
	lea	rbx,[opcode_0F30_0F3F_instruction_table]
	mov	rax,[rbx+rax*8]
	stosq
	mov	eax,2-1
	ret

process_opcode_getsec_0F37:
	mov	rax,'getsec'
	stosq
	mov	eax,2-1
	ret

process_opcode_0F38:
; 253667.pdf table A-4
	ret

process_opcode_0F3A:
; 253667.pdf table A-5
	ret

process_opcode_cmov_0F40_0F4F:
	and	al,00001111b
	mov	ecx,eax
	mov	eax,'cmov'
	stosd
	lea	rbx,[instructions_jump_conditional_table + 1]
	mov	eax,[rbx+rcx*4]
	stosd
	dec	rdi
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	al,' '
	stosb

;	call	process_opcode_big_fucking_gun_reg_mem_d
;
;	mov	byte [rdi],0
;	ret
	jmp	process_opcode_big_fucking_gun_reg_mem_d

process_opcode_movmskps_movmskpd_0F50:
	mov	rax,'movmskps'
	mov	rbx,'movmskpd'
	call	search_disassemble_prefix_66
	cmovz	rax,rbx
	setnz	dl
	add	dl,post_disasm_xmm_128_sp
	stosq
	mov	byte [post_disasm_action],dl
	mov	al,' '
	stosb
	call	process_Gd_VRo
	ret

process_opcode_sqrt_0F51:
	lea	rbx,[opcode_sqrt_0F51_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0F51_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl
	call	qword [rbx+8*4+rcx*8]
	ret

;process_opcode_rsqrt_0F52:
;	mov	rax,'rsqrtps '
;	mov	rbx,'rsqrtss '
;	mov	dl,post_disasm_xmm_128_sp
;	mov	cl,post_disasm_fpu_dword
;	call	search_disassemble_prefix_F3
;	cmovz	rax,rbx
;	cmovz	edx,ecx
;	mov	byte [post_disasm_action],dl
;	lea	rbx,[process_Vo_Wo]
;	lea	rcx,[process_Vd_Wd]
;	cmovz	rbx,rcx
;	stosq
;	call	rbx
;	ret
;
;process_opcode_rcp_0F53:
;	mov	rax,'rcpps '
;	mov	rbx,'rcpss '
;	mov	dl,post_disasm_xmm_128_sp
;	mov	cl,post_disasm_fpu_dword
;	call	search_disassemble_prefix_F3
;	cmovz	rax,rbx
;	cmovz	edx,ecx
;	mov	byte [post_disasm_action],dl
;	lea	rbx,[process_Vo_Wo]
;	lea	rcx,[process_Vd_Wd]
;	cmovz	rbx,rcx
;	stosq
;	dec	rdi
;	dec	rdi
;	call	rbx
;	ret

process_opcode_rsqrt_0F52:
	mov	rax,'rsqrtps '
	stosq
	jmp	process_opcode_rsqrt_0F52_rcp_0F53_common

process_opcode_rcp_0F53:
	mov	rax,'rcpps '
	stosq
	lea	rdi,[rdi-2]
process_opcode_rsqrt_0F52_rcp_0F53_common:
	mov	dl,post_disasm_xmm_128_sp
	mov	cl,post_disasm_fpu_dword
	call	search_disassemble_prefix_F3
	setz	al
	cmovz	edx,ecx
	lea	eax,[rax+rax*2]		; transform 0->0 1->3 ('p' xor 's' = 3)
	lea	rbx,[process_Vo_Wo]
	lea	rcx,[process_Vd_Wd]
	mov	byte [post_disasm_action],dl
	cmovz	rbx,rcx
	xor	byte [rdi-3],al		; change 'p'->'s'
	call	rbx
	ret

process_opcode_and_andn_or_xor_0F54_0F57:
	and	al,00000011b
	lea	rbx,[opcode_and_andn_or_xor_0F54_0F57_table]
	mov	eax,[rbx+rax*4]
	mov	[rdi],eax
	bsr	eax,eax
	add	al,7
	shr	eax,3
	add	rdi,rax
	mov	eax,'ps '
	mov	ecx,'pd '
	call	search_disassemble_prefix_66
	cmovz	eax,ecx
	stosd
; although packed single/double in instruction name, these instructions perform logical
; bitwise operations, so we want to know hexa data instead of packed single/double:
	mov	byte [post_disasm_action],post_disasm_xmm_128_hexa
	dec	rdi
	call	process_Vo_Wo
	ret

process_opcode_0F58_0F59_0F5C_0F5F:
	and	al,00000111b
	lea	rbx,[opcode_0F58_0F59_0F5C_0F5F_instructions]
	mov	eax,[rbx+rax*4]
	stosd
	dec	rdi
	lea	rbx,[rbx+8*4]	; = lea rbx,[opcode_0F58_0F59_0F5C_0F5F_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0F58_0F59_0F5C_0F5F_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl
	call	qword [rbx+8*4+rcx*8]
	ret

process_opcode_0F5A:
	lea	rbx,[opcode_0F5A_table]
;	call	process_opcode_mm_instruction_universal
;	lea	rdx,[opcode_0F5A_post_disasm_table]
;	mov	dl,byte [rdx+rcx*1]
;	mov	byte [post_disasm_action],dl
;	call	qword [rbx+8*4+rcx*8]
;	ret
	jmp	process_opcode_0F2C_0F2D_0F5A_common

process_opcode_0F5B:
	lea	rbx,[opcode_0F5B_instructions]
	mov	byte [rdi],'c'
	scasb
	call	process_opcode_mm_instruction_universal
	mov	edx,2-1	; opcode size
	cmp	cl,3
	cmovz	eax,edx
	jz	process_opcode_0F58_0F59_0F5C_0F5F_egzyd
	cmp	cl,0
	mov	dl,post_disasm_xmm_128_sp
	mov	cl,post_disasm_xmm_4_sig_dwords
	cmovz	edx,ecx
	mov	byte [post_disasm_action],dl
	call	process_Vo_Wo
process_opcode_0F58_0F59_0F5C_0F5F_egzyd:
	ret

process_opcode_0F60_0F62:
	and	al,00000011b
	lea	rbx,[opcode_0F60_0F62_instructions]
	mov	byte [rdi],'p'
	scasb
	mov	rax,[rbx+rax*8]
	stosq
	mov	al,' '
	stosb
	lea	rbx,[process_Pq_Qd]
	lea	rcx,[process_Vo_Wq]
	mov	dl,post_disasm_dword
	mov	al,post_disasm_mm_64_hexa
	jmp	process_opcode_0F63_0F67_common_part_2

process_opcode_0F63_0F67:
	sub	al,63h
	lea	rbx,[opcode_0F63_0F67_instructions]
process_opcode_0F63_0F67_common_part_1:
	mov	rax,[rbx+rax*8]
	stosq
	shr	rax,32+16+8
	setz	al
	sub	rdi,rax
	mov	al,' '
	stosb
	lea	rbx,[process_Pq_Qq]
	lea	rcx,[process_Vo_Wo]
	mov	dl,post_disasm_mm_64_hexa
	mov	al,post_disasm_xmm_128_hexa
process_opcode_0F63_0F67_common_part_2:
	call	search_disassemble_prefix_66
	cmovz	edx,eax
	cmovz	rbx,rcx
	mov	byte [post_disasm_action],dl
	call	rbx
	ret

process_opcode_0F68_0F6B:
	and	al,00000011b
	mov	byte [rdi],'p'
	scasb
	lea	rbx,[opcode_0F68_0F6B_instructions]
	jmp	process_opcode_0F63_0F67_common_part_1

process_opcode_punpcklqdq_0F6C:
	mov	rax,'punpcklq'
process_opcode_punpcklqdq_0F6C_common_begin:
	mov	rbx,'p???'
	call	search_disassemble_prefix_66
	cmovnz	rax,rbx
	mov	byte [post_disasm_action],post_disasm_mm_64_hexa
	stosq
	mov	eax,'dq '
	stosd
	dec	rdi
	call	process_Vo_Wq
	ret

process_opcode_punpckhqdq_0F6D:
	mov	rax,'punpckhq'
	jmp	process_opcode_punpcklqdq_0F6C_common_begin

process_opcode_movd_0F6E:
; correct this for REX.W:
; 0Fh,6Eh	MOVQ Pq,Eq
; 66h,0Fh,6Eh	MOVQ Vo,Eq

	mov	eax,'movd'
;	mov	ebx,'movq'	; www.amd.com 26568.pdf MOVD Move Doubleword or Quadword
				; if you want FASM movd xmm13,r10 you have to use movq
				; even movq xmm13,r10 breaks amd rules and may be encoded by movd xmm13,r10
	call	search_disassemble_prefix_REX
	bt	edx,3
;	cmovc	eax,ebx
	stosd
	mov	al,' '
	stosb
	lea	rbx,[process_Pq_Ed]
	lea	rax,[process_Pq_Eq]
	lea	rcx,[process_Vo_Ed]
	lea	rdx,[process_Vo_Eq]
	cmovc	rbx,rax
	cmovc	rcx,rdx
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	mov	byte [post_disasm_action],post_disasm_dword
	call	rbx
	ret

process_opcode_0F6F:
	lea	rbx,[opcode_0F6F_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0F6F_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl
	call	qword [rbx+8*4+rcx*8]
	ret

process_opcode_pshuf_0F70:
	lea	rbx,[opcode_pshuf_0F70_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0F70_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl
	call	qword [rbx+8*4+rcx*8]
	call	process_lb_binary
	ret

process_opcode_pshimw_pshimd_0F71_0F72:
; we check illegal instructions, only registers are allowed, not memory !
; e.g. 0F711003 is illegal, but 0F71D003 is real instruction psrlw mm0,3
	mov	eax,'ps?'
	cmp	byte [rsi+1],11b shl 6	; =0C0h mod field of mod R/M byte should be 11b
	jc	process_opcode_pshimw_pshimd_0F71_0F72_store_instruction
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	lea	rbx,[opcode_pshimw_pshimd_0F71_0F72_table]
	mov	eax,[rbx+rax*4]
process_opcode_pshimw_pshimd_0F71_0F72_store_instruction:
	stosd
	mov	al,'d'
	mov	cl,'w'
	bt	dword [rsi],0	; difference between 71 and 72
	cmovc	eax,ecx
	stosb
	mov	al,' '
	stosb
;	mov	eax,'xmmx'	; hybrid for xmm and mmx
;;	mov	cl,0
;;	mov	dl,8
;	call	search_disassemble_prefix_66
;;	cmovz	ecx,edx
;	setz	cl
;	shl	cl,3
;	shr	eax,cl
;	stosd
;	dec	rdi

	lea	rbx,[process_PRq]
	lea	rcx,[process_VRo]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	call	rbx
	call	process_lb_hexa
	ret

process_opcode_pshimq_0F73:
	mov	eax,'???'
; we check illegal instructions, only registers allowed, not memory !
	cmp	byte [rsi+1],0C0h
	jc	process_opcode_pshimq_0F73_store_instruction
	mov	dl,[rsi+1]
	and	dl,00110000b
	mov	ebx,'psrl'
	mov	ecx,'psll'
	cmp	dl,00010000b
	cmovz	eax,ebx
	cmp	dl,00110000b
	cmovz	eax,ecx
process_opcode_pshimq_0F73_store_instruction:
	stosd
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	lea	rbx,[opcode_pshimq_0F73_table]
	call	search_disassemble_prefix_66
	lea	rcx,[process_PRq]
	lea	rdx,[process_VRo]
	cmovz	rcx,rdx
; change rbx pointing from opcode_pshimq_0F73_table to opcode_pshimq_660F73_table if 66 prefix
	lea	rdx,[rbx+8*2]
	cmovz	rbx,rdx
	movzx	eax,word [rbx+rax*2]
	stosw
	shr	eax,8
;	setnz	al
;	lea	rdi,[rdi+rax*1-1]
	setz	al
	sub	rdi,rax
	mov	al,' '
	stosb
	call	rcx
	call	process_lb_hexa
	ret

process_opcode_pcmpeq_0F74_0F76:
	and	al,00000011b
	lea	rbx,[opcode_pcmpeq_0F74_0F76_table]
	mov	rax,[rbx+rax*8]
	stosq
	lea	rbx,[process_Pq_Qq]
	lea	rcx,[process_Vo_Wo]
	mov	dl,post_disasm_mm_64_hexa
	mov	al,post_disasm_xmm_128_hexa
	call	search_disassemble_prefix_66
	cmovz	edx,eax
	cmovz	rbx,rcx
	mov	byte [post_disasm_action],dl
	call	rbx
	ret

process_opcode_emms_0F77:
	mov	eax,'emms'
	stosd
	mov	byte [rdi],0
	mov	eax,2-1
	ret

process_opcode_vmread_0F78:
	mov	rax,'vmread '
	stosq
	dec	rdi

	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_vmread_0F78_skip_operand_size
	xor	eax,eax
	mov	byte [rdi],'q'
	call	search_disassemble_prefix_66
	setnz	al
	add	rdi,rax
	lea	edx,[rax+rax*1+post_disasm_word]	; post_disasm_word -> post_disasm_qword
	mov	byte [post_disasm_action],dl
	mov	eax,'word'
	stosd
	mov	al,' '
	stosb
process_opcode_vmread_0F78_skip_operand_size:
	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_q_w]
	call	qword [rbx+rax*8]

	push	rax

	mov	al,','
	stosb
	lea	rbx,[gpr_64]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	call	search_disassemble_prefix_REX
	bt	edx,2		; REX.R		;0		; REX.B
	setc	dl
	shl	edx,3+2		; rdx*8*4  ->  no dx but r10w , no eax but r8d , no rcx but r9
	add	rbx,rdx
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	al,3
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0
	mov	byte [rdi],0

	pop	rax

	ret


process_opcode_vmwrite_0F79:
	mov	rax,'vmwrite '
	stosq
process_opcode_vmwrite_0F79_common_end:
	lea	rbx,[gpr_64]
	lea	rcx,[gpr_16]
	call	search_disassemble_prefix_66
	cmovz	rbx,rcx
	call	search_disassemble_prefix_REX
	bt	edx,2		; REX.R		;0		; REX.B
	setc	dl
	shl	edx,3+2		; rdx*8*4  ->  no dx but r10w , no eax but r8d , no rcx but r9
	add	rbx,rdx
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	al,3
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0
	mov	al,','
	stosb
	jmp	process_opcode_FF_push

process_opcode_unknown_0F7A_0F7B:
	and	al,00001111b
	lea	rbx,[table_transform_hexa]
	xlatb
	xchg	ecx,eax
	mov	rax,'db 0F,7'
	stosq
	mov	byte [rdi-1],cl
	mov	byte [rdi],0
	mov	eax,2-1
	ret

process_opcode_haddp_hsubp_0F7C_0F7D:
	and	al,00000001b
	shl	eax,3+2
	lea	rbx,[opcode_haddp_0F7C_instruction]
;	lea	rbx,[rbx+rax*1]		; change opcode_haddp_0F7C_instruction to opcode_hsubp_0F7D_instruction
	add	rbx,rax
	call	process_opcode_mm_instruction_universal
	call	process_Vo_Wo
	mov	byte [post_disasm_action],post_disasm_xmm_128_hexa
	ret

process_opcode_movq_0F7E:
	lea	rbx,[opcode_movq_0F7E_table]
	call	process_opcode_mm_instruction_universal
	mov	al,cl
	mov	dl,3
	cmp	cl,1
	cmovz	eax,edx
	call	search_disassemble_prefix_REX
	bt	edx,3
	cmovc	ecx,eax
	mov	byte [post_disasm_action],post_disasm_mm_64_hexa
	call	qword [rbx+8*4+rcx*8]
	ret

process_opcode_0F7F:
	lea	rbx,[process_opcode_0F7F_table]
	call	process_opcode_mm_instruction_universal
	mov	dl,post_disasm_xmm_128_hexa
	mov	al,post_disasm_mm_64_hexa
	or	ecx,ecx
	cmovz	edx,eax
	mov	byte [post_disasm_action],dl
	call	qword [rbx+8*4+rcx*8]
	ret

process_opcode_jump_condition_long_0F80_0F8F:
	and	al,00001111b
	mov	[post_disasm_cond_jump_type],al
	lea	rbx,[instructions_jump_conditional_table]
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	al,' '
	stosb
	movsxd	rax,dword [rsi+1]
	lea	rax,[rbp + rax*1 + 6]
	call	hexa_64_rax
	add	rdi,16
	mov	byte [rdi],0

	mov	qword [post_disasm_addr0],rax
	or	byte [post_disasm_flag],post_disasm_flag_direct or post_disasm_flag_cond_jump
	mov	byte [post_disasm_action],post_disasm_addr

	mov	eax,5
	ret

process_opcode_set_condition_0F90_0F9F:
	and	al,00001111b
	mov	ecx,eax
	mov	eax,'set'
	stosd
	dec	rdi

	lea	rbx,[instructions_jump_conditional_table + 1]
	mov	eax,[rbx+rcx*4]
	stosd
	dec	rdi
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	al,' '
	stosb

	cmp	byte [rsi+1],11000000b
	jnc	process_opcode_set_condition_0F90_0F9F_skip_operand_size
	mov	eax,'byte'
	mov	byte [post_disasm_action],post_disasm_byte
	stosd
	mov	al,' '
	stosb
process_opcode_set_condition_0F90_0F9F_skip_operand_size:

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table]
	call	qword [rbx+rax*8]

	mov	byte [rdi],0
	ret

process_opcode_push_pop_fs_gs_0FA0_0FA1_0FA8_0FA9:
	xchg	ecx,eax
	mov	eax,'push'
	mov	ebx,'pop'
	bt	ecx,0
	cmovc	eax,ebx
	stosd
	setc	al
	movzx	eax,al
	sub	rdi,rax
	mov	al,' '
	stosb
	mov	eax,'fs'
	bt	ecx,3
	adc	al,0	; change 'fs' to 'gs'
	stosd
	mov	eax,1
	ret

process_opcode_cpuid_0FA2:
	mov	rax,'cpuid'
	stosq
	mov	eax,1
	ret

process_opcode_rsm_0FAA:
	mov	eax,'rsm'
	stosd
	mov	eax,1
	ret

process_opcode_bt_bts_0FA3_0FAB:
	bt	eax,3
	mov	eax,'bts'
	stosd
	setc	al
	movzx	eax,al
	lea	rdi,[rdi-2+rax*1]
	mov	al,' '
	stosb
	jmp	process_opcode_big_fucking_gun_mem_reg_d

process_opcode_shld_shrd_0FA4_0FA5_0FAC_0FAD:
	xchg	ecx,eax
	mov	eax,'shld'
	mov	ebx,'shrd'
	bt	ecx,3
	cmovc	eax,ebx
	stosd
	mov	al,' '
	stosb
	call	process_opcode_big_fucking_gun_mem_reg_d
	push	rax
	mov	byte [rdi],','
	scasb
	mov	al,[rsi+rax*1]
	shl	rax,32+16+8
	call	hexa_64_rax
	mov	eax,[rdi]
	mov	ebx,'cl'
	bt	dword [rsi],0
	cmovc	eax,ebx
	stosw
	mov	byte [rdi],0
	pop	rax
	bt	dword [rsi],0
	setnc	bl
	add	al,bl
	ret

process_opcode_unknown_0FA6:
	mov	rax,'db 0F,A6'
	jmp	process_opcode_unknown_0FA7_continue
process_opcode_unknown_0FA7:
	mov	rax,'db 0F,A7'
process_opcode_unknown_0FA7_continue:
	stosq
	mov	byte [rdi],0
	mov	eax,2-1
	ret

process_opcode_0FAE:
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	cmp	byte [rsi+1],11000000b
	jnc	process_opcode_0FAE_no_xsave_no_xrstor_no_clflush
	cmp	al,38h
	jz	process_opcode_0FAE_clflush
	cmp	al,20h
	jc	process_opcode_0FAE_no_xsave_no_xrstor_no_clflush
	cmp	al,30h
	jc	process_opcode_0FAE_xsave_xrstor
process_opcode_0FAE_no_xsave_no_xrstor_no_clflush:
	lea	rbx,[opcode_0FAE_instruction_table]
	mov	rax,[rbx+rax*1]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	movzx	eax,byte [rsi+1]
	and	al,11111000b
	cmp	al,0E0h
	mov	eax,2		; opcode size
	jnc	process_opcode_0FAE_konyc
	mov	eax,' '
	mov	rbx,' dword '
	mov	dl,post_disasm_nothing
	mov	cl,post_disasm_dword
	bt	dword [rsi+1],4
	cmovc	rax,rbx
	cmovc	edx,ecx
	mov	[rdi],rax
	mov	byte [post_disasm_action],dl
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
process_opcode_0FAE_common_end:
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table]
	call	qword [rbx+rax*8]
process_opcode_0FAE_konyc:
	mov	byte [rdi],0
	ret
process_opcode_0FAE_clflush:
	mov	rax,'clflush '
	stosq
	mov	eax,'byte'
	stosd
	mov	byte [post_disasm_action],post_disasm_byte
	mov	al,' '
	stosb
	jmp	process_opcode_0FAE_common_end

process_opcode_0FAE_xsave_xrstor:
;	sub	al,20h
	test	al,1000b
	mov	rax,'xsave'
	mov	rcx,'xrstor'
	cmovz	rax,rcx
	setz	dl
	movzx	edx,dl
	stosq
	lea	rdi,[rdi-3+rdx*1]
	call	search_disassemble_prefix_REX
	jc	process_opcode_0FAE_xsave_xrstor_go_on
	bt	edx,3		; 64bit operand size
	jnc	process_opcode_0FAE_xsave_xrstor_go_on
	mov	al,'6'
	stosb
	mov	al,'4'
	stosb
process_opcode_0FAE_xsave_xrstor_go_on:
	mov	al,' '
	stosb
	jmp	process_opcode_0FAE_common_end


process_opcode_imul_0FAF:
	mov	eax,'imul'
	stosd
	mov	al,' '
	stosb
;	call	process_opcode_big_fucking_gun_reg_mem_d
;	mov	byte [rdi],0
;	ret
	jmp	process_opcode_big_fucking_gun_reg_mem_d

process_opcode_cmpxchg_0FB0:
	mov	rax,'cmpxchg '
	stosq
	jmp	process_opcode_big_fucking_gun_mem_reg_b
process_opcode_cmpxchg_0FB1:
	mov	rax,'cmpxchg '
	stosq
	jmp	process_opcode_big_fucking_gun_mem_reg_d

process_opcode_lss_0FB2:
	mov	eax,'lss '
	stosd
	jmp	process_opcode_big_fucking_gun_reg_mem_d

process_opcode_lfs_lgs_0FB4_0FB5:
	bt	eax,0
	setc	cl
	mov	eax,'lfs '
	add	ah,cl		; change lfs to lgs
	stosd
	jmp	process_opcode_big_fucking_gun_reg_mem_d

process_opcode_btr_btc_0FB3_0FBB:
	bt	eax,3
	mov	eax,'btr '
	mov	ebx,'btc '
	cmovc	eax,ebx
	stosd
	jmp	process_opcode_big_fucking_gun_mem_reg_d

process_opcode_movzx_movsx_0FB6_0FB7_0FBE_0FBE:
	mov	eax,'movz'
	mov	ebx,'movs'
	bt	dword [rsi],3
	cmovc	eax,ebx
	stosd
	mov	al,'x'
	stosb
	mov	al,' '
	stosb

	call	process_opcode_big_fucking_gun_reg_muthafucker_wdq

	mov	al,','
	stosb

	cmp	byte [rsi+1],0C0h
	jnc	process_opcode_movzx_movsx_0FB6_0FB7_0FBE_0FBE_skip_operand_size
	mov	eax,'byte'
	mov	ebx,'word'
	bt	dword [rsi],0
	cmovc	eax,ebx
	stosd
	mov	al,' '
	stosb
process_opcode_movzx_movsx_0FB6_0FB7_0FBE_0FBE_skip_operand_size:

	movzx	eax,byte [rsi+1]
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table]
	lea	rcx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_w]
	bt	dword [rsi],0
	cmovc	rbx,rcx
	call	qword [rbx+rax*8]
	mov	dl,post_disasm_byte
	bt	dword [rsi],0
	adc	dl,0
	mov	byte [post_disasm_action],dl
	ret

process_opcode_big_fucking_gun_reg_w:
	movzx	eax,byte [rsi+1]
	and	al,111b
	lea	rbx,[gpr_16]
	call	search_disassemble_prefix_REX
	bt	edx,0		; REX.B
	setc	cl
	movzx	ecx,cl
	shl	ecx,3+2
	add	rbx,rcx
	mov	eax,[rbx+rax*4]
	stosd
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0

	mov	byte [rdi],0	; don't increase rdi
	mov	eax,2
	ret

;process_opcode_unknown_0FB8:
;	mov	rax,'db 0F,B8'
;	stosq
;	mov	byte [rdi],0
;	mov	eax,2-1
;	ret
process_opcode_jmpe_popcnt:
	mov	eax,'jmpe'
	mov	rcx,'popcnt '
	call	search_disassemble_prefix_F3
	cmovz	rax,rcx
	stosq
	jz	process_opcode_popcnt
	mov	eax,2-1
	ret
process_opcode_popcnt:
	dec	rdi
	jmp	process_opcode_big_fucking_gun_reg_mem_d


process_opcode_ud2_0FB9:
	mov	eax,'ud2'
	stosd
	dec	rdi
	mov	eax,2-1
	ret

process_opcode_bit_tests_0FBA:
	movzx	eax,byte [rsi+1]
	and	al,00011000b
	shr	eax,3
	lea	rbx,[opcode_bit_tests_0FBA_table]
	mov	eax,[rbx+rax*4]
	stosd
	setnz	al
	movzx	eax,al
	lea	rdi,[rdi-2+rax*1]
	mov	al,' '
	stosb

; operand sizes here, default dword, 66 prefix word, 48 prefix qword
	mov	rax,'dword '
	mov	ecx,6		; size
	mov	rbx,'word '
	mov	edx,5
	call	search_disassemble_prefix_66
	cmovz	rax,rbx
	cmovz	ecx,edx
	mov	rbx,'qword '
	call	search_disassemble_prefix_REX
	bt	edx,3
	mov	edx,6
	cmovc	rax,rbx
	cmovc	ecx,edx
	xor	edx,edx
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx		; no operand size for registers!
	mov	[rdi],rax
	add	rdi,rcx

	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_bit_tests_0FBA_table]
	call	qword [rbx+rax*8]

	push	rax		; opcode size

	mov	byte [rdi],','
	scasb

	mov	al,[rsi+rax*1]
	shl	rax,32+16+8
	call	hexa_64_rax
	scasb
	scasb
	mov	byte [rdi],0
	pop	rax		; opcode size
	inc	eax

	mov	bl,post_disasm_dword
	mov	cl,post_disasm_word
	call	search_disassemble_prefix_66
	cmovz	ebx,ecx
	mov	cl,post_disasm_qword
	call	search_disassemble_prefix_REX
	bt	edx,3
	cmovc	ebx,ecx
	mov	byte [post_disasm_action],bl

	ret

process_opcode_bsf_bsr_0FBC_0FBD:
	bt	eax,0
	mov	eax,'bsf '
	mov	ebx,'bsr '
	cmovc	eax,ebx
	stosd
;	call	process_opcode_big_fucking_gun_reg_mem_d
;	mov	byte [rdi],0
;	ret
	jmp	process_opcode_big_fucking_gun_reg_mem_d

process_opcode_xadd_0FC0:
	mov	eax,'xadd'
	stosd
	mov	al,' '
	stosb
	jmp	process_opcode_big_fucking_gun_mem_reg_b
process_opcode_xadd_0FC1:
	mov	eax,'xadd'
	stosd
	mov	al,' '
	stosb
	jmp	process_opcode_big_fucking_gun_mem_reg_d

process_opcode_0FC2:
	mov	eax,'cmp'
	stosd
	dec	rdi
	push	rdi		; save rdi
	call	process_Vo_Wo	; only for get opcode size
	pop	rdi		; restore rdi
	movzx	eax,byte [rsi+rax*1]
	and	al,00000111b
	lea	rbx,[opcode_0FC2_conditions]
	mov	rax,[rbx+rax*8]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	lea	rbx,[opcode_0FC2_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0FC2_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl
	call	qword [rbx + 8*4 + rcx*8]
	inc	rax		; opcode size
	ret

; 0Fh,C3h MOVNTI Mq,Gq 
process_opcode_movnti_0FC3:
	mov	rax,'movnti '
	mov	ebx,'???'
	cmp	byte [rsi+1],0C0h
	cmovnc	rax,rbx
	stosq
	dec	rdi
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_q]
	call	qword [rbx+rax*8]
; note that 4rd subroutine is called in case of invalid opcode which causes exception when executes
	push	rax	; save opcode size
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_64]
	mov	al,','
	stosb
	call	search_disassemble_prefix_REX
	bt	edx,3
	mov	al,post_disasm_dword
	cmovc	rbx,rcx
	adc	al,0
	mov	byte [post_disasm_action],al
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	al,3
	bt	edx,2			; REX.R
	setc	cl
	movzx	ecx,cl
	shl	ecx,3+2
	add	rbx,rcx			; not rax but r8, not eax but r8d ...
	mov	eax,[rbx+rax*4]
	stosd
	mov	[rdi],eax
	bsr	eax,eax
	add	al,7
	shr	eax,3
	add	rdi,rax
	pop	rax	; restore opcode size
	ret

process_opcode_pinsrw_0FC4:
	mov	rax,'pinsrw '	
	stosq
	dec	rdi
	lea	rcx,[process_Pq_destination]
	lea	rdx,[process_Vo]
	call	search_disassemble_prefix_66
	cmovz	rcx,rdx
	call	rcx
	mov	al,','
	stosb
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_Ed_table]
	call	qword [rbx+rax*8]
	mov	byte [rdi],','
	scasb
	push	rax
	mov	al,byte [rsi+rax*1]
	shl	rax,32+16+8
	call	hexa_64_rax
	scasw
	pop	rax
	inc	eax
	mov	byte [rdi],0
	mov	byte [post_disasm_action],post_disasm_word
	ret

process_opcode_pextrw_0FC5:
	mov	rax,'pextrw '	
	mov	ebx,'???'
	cmp	byte [rsi+1],0C0h
	cmovc	rax,rbx

	stosq
	dec	rdi

	call	process_Gd

	mov	al,','
	stosb

	lea	rcx,[process_PRq]
	lea	rdx,[process_VRo]
	call	search_disassemble_prefix_66
	cmovz	rcx,rdx
	call	rcx

	mov	byte [rdi],','
	scasb

	mov	al,byte [rsi+2]
	shl	rax,32+16+8
	call	hexa_64_rax
	scasw
	mov	byte [rdi],0
	mov	rax,4-1
	ret

process_opcode_shufp_0FC6:
	mov	eax,'shuf'
	stosd
	mov	eax,'ps '
	mov	ebx,'pd '
	mov	dl,post_disasm_xmm_128_sp
	mov	cl,post_disasm_xmm_128_dp
	call	search_disassemble_prefix_66
	cmovz	eax,ebx
	cmovz	edx,ecx
	stosd
	mov	byte [post_disasm_action],dl
	dec	rdi
	call	process_Vo_Wo
	call	process_lb_hexa
	ret

process_opcode_cmpxchg8b_cmpxchg16b_0FC708_0FC70F:
;0Fh,C7h,/1 CMPXCHG8B Mq CMPXCHG16B Mo
	mov	al,byte [rsi+1]
	and	al,00111000b
	cmp	al,8*6
	jz	process_opcode_vmclear_vmptrld_vmxon
	cmp	al,8*7
	jz	process_opcode_vmptrst
	cmp	al,8*1
	mov	rax,'cmpxchg1'
	mov	ebx,'???'
	cmovnz	rax,rbx
	cmp	byte [rsi+1],0C0h
; Not allowed ! The destination operand should be a double quad word in memory.
	cmovnc	rax,rbx
	stosq
	call	search_disassemble_prefix_REX
	bt	edx,3
	mov	dl,'6'
	mov	bl,'8'
	mov	rax,'b qword '
	mov	rcx,'b oword '
	cmovc	ebx,edx
	cmovc	rax,rcx
	mov	dl,post_disasm_dword
	mov	cl,post_disasm_xmm_128_hexa
	cmovc	edx,ecx
	setc	cl
	movzx	ecx,cl
	add	rdi,rcx
	mov	byte [rdi-1],bl
	mov	byte [post_disasm_action],dl
	stosq
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
	call	qword [rbx+rax*8]
; note that only first 3 subroutines are used, not 4th subroutine !
	ret

process_opcode_vmclear_vmptrld_vmxon:
	mov	rax,'vmclear'
	call	search_disassemble_prefix_66
	jz	process_opcode_vanderpol_common_end
	mov	rax,'vmxon'
	call	search_disassemble_prefix_F3
	jz	process_opcode_vanderpol_common_end
	mov	rax,'vmptrld'
process_opcode_vanderpol_common_end:
	stosq
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0
	cmp	byte [rdi-1],1
	sbb	rdi,0
	mov	rax,' qword '
	stosq
	dec	rdi
;	call	process_opcode_big_fucking_gun_mem	; wroooooooong, bbbbbuuuuuuuugggggg !!!!
; correct:
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
	call	qword [rbx+rax*8]
; note that function return opcode size in RAX
	mov	byte [rdi],0
	mov	[post_disasm_action],post_disasm_mm_64_hexa
	ret

process_opcode_vmptrst:
	mov	rax,'vmptrst'
	jmp	process_opcode_vanderpol_common_end


process_opcode_bswap_0FC8_0FCF:
	mov	rax,'bswap '
	stosq
	dec	rdi
	dec	rdi
	movzx	eax,byte [rsi]
	and	al,00000111b
	call	process_opcode_80_83_reg_wdq
	mov	byte [rdi],0
	mov	eax,1
	ret

process_opcode_addsubp_0FD0:
	mov	rax,'addsubp?'
	mov	rcx,'addsubpd'
	call	search_disassemble_prefix_66
	mov	dl,post_disasm_nothing
	mov	bl,post_disasm_xmm_128_dp
	cmovz	rax,rcx
	cmovz	edx,ebx
	mov	rcx,'addsubps'
	mov	bl,post_disasm_xmm_128_sp
	call	search_disassemble_prefix_F2
	cmovz	rax,rcx
	cmovz	edx,ebx
	stosq
	mov	byte [post_disasm_action],dl
	mov	al,' '
	stosb
	call	process_Vo_Wo
	ret

process_opcode_psrl_0FD1_0FD3:
	and	al,00000011b
	lea	rbx,[operand_size_short]
	mov	dl,[rbx+rax*1]
	mov	eax,'psrl'
	stosd
	xchg	edx,eax
	stosb
	mov	al,' '
	stosb
process_opcode_psrl_0FD1_0FD3_common_continue:
	mov	dl,post_disasm_mm_64_hexa
	mov	cl,post_disasm_xmm_128_hexa
	lea	rax,[process_Pq_Qq]
	lea	rbx,[process_Vo_Wo]
	call	search_disassemble_prefix_66
	cmovz	edx,ecx
	cmovz	rax,rbx
	mov	byte [post_disasm_action],dl
;	call	rax
;	ret
	jmp	rax

process_opcode_paddq_0FD4:
	mov	rax,'paddq '
	stosq
	dec	rdi
	dec	rdi
	jmp	process_opcode_psrl_0FD1_0FD3_common_continue

process_opcode_pmullw_0FD5:
	mov	rax,'pmullw '
	stosq
	dec	rdi
	jmp	process_opcode_psrl_0FD1_0FD3_common_continue

process_opcode_0FD6:
	lea	rbx,[opcode_0FD6_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0FD6_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl
	jmp	qword [rbx+8*4+rcx*8]

process_opcode_pmovmskb_0FD7:
	mov	rax,'pmovmskb'
	stosq
	mov	al,' '
	stosb
	lea	rax,[process_Gd_PRq]
	lea	rbx,[process_Gd_VRo]
	call	search_disassemble_prefix_66
	cmovz	rax,rbx
;	call	rax
;	ret
	jmp	rax

process_opcode_0FD8_0FDF:
	lea	rbx,[opcode_0FD8_0FDF_instructions]
process_opcode_0FD8_0FDF_common_begin:
	and	al,00000111b
	mov	rax,[rbx+rax*8]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	mov	al,' '
	stosb
	mov	dl,post_disasm_mm_64_hexa
	mov	cl,post_disasm_xmm_128_hexa
	lea	rax,[process_Pq_Qq]
	lea	rbx,[process_Vo_Wo]
	call	search_disassemble_prefix_66
	cmovz	edx,ecx
	cmovz	rax,rbx
	mov	byte [post_disasm_action],dl
;	call	rax
;	ret
	jmp	rax

process_opcode_0FE0_0FE5:
	lea	rbx,[opcode_0FE0_0FE5_instructions]
	jmp	process_opcode_0FD8_0FDF_common_begin

process_opcode_0FF1_0FF6:
	lea	rbx,[opcode_0FF1_0FF6_instructions]
	jmp	process_opcode_0FD8_0FDF_common_begin

process_opcode_0FE8_0FEF:
	lea	rbx,[opcode_0FE8_0FEF_instructions]
	jmp	process_opcode_0FD8_0FDF_common_begin

process_opcode_0FF8_0FFE:
	lea	rbx,[opcode_0FF8_0FFE_instructions]
	jmp	process_opcode_0FD8_0FDF_common_begin

process_opcode_0FE6:
	mov	al,'c'
	stosb
	lea	rbx,[opcode_0FE6_table]
	call	process_opcode_mm_instruction_universal
	lea	rdx,[opcode_0FE6_post_disasm_table]
	mov	dl,byte [rdx+rcx*1]
	mov	byte [post_disasm_action],dl
;	call	qword [rbx + 8*4 + rcx*8]
;	ret
	jmp	qword [rbx + 8*4 + rcx*8]

process_opcode_0FE7:
	mov	rax,'movntq '
	mov	rbx,'movntdq '
	lea	rcx,[process_Mq_Pq]	; ???? najprv call, potom ',', potom ..._DESTINATION !!!
	lea	rdx,[process_Mo_Vo]
	call	search_disassemble_prefix_66
	cmovz	rax,rbx
	cmovz	rcx,rdx
	mov	dl,post_disasm_mm_64_hexa
	mov	bl,post_disasm_xmm_128_hexa
	cmovz	edx,ebx
	mov	byte [post_disasm_action],dl
	mov	rbx,'???'
	cmp	byte [rsi+1],0C0h
	cmovnc	rax,rbx			; only memory destination allowed, not register
	mov	[rdi],rax
	shr	rax,32+16+8
	setnz	al
	lea	rdi,[rdi+rax+7]
	jmp	rcx

process_opcode_lddqu_F20FF0:
	mov	rax,'lddqu '
	mov	ebx,'???'
	call	search_disassemble_prefix_F2
	cmovnz	rax,rbx
	cmp	byte [rsi+1],0C0h
	cmovnc	rax,rbx
	stosq
	mov	byte [post_disasm_action],post_disasm_xmm_128_hexa
	dec	rdi
	dec	rdi
	call	process_Vo_Mo
	ret

process_opcode_0FF7:
	mov	eax,'mask'
	stosd
	mov	rax,'movq '
	mov	rbx,'movdqu '
	lea	rcx,[process_Pq_PRq]
	lea	rdx,[process_Vo_VRo]
	call	search_disassemble_prefix_66
	cmovz	rax,rbx
	cmovz	rcx,rdx
	mov	dl,post_disasm_mm_64_hexa
	mov	bl,post_disasm_xmm_128_hexa
	cmovz	edx,ebx
	mov	byte [post_disasm_action],dl
	or	byte [post_disasm_flag],post_disasm_flag_rdi
	mov	ebx,'??? '
	cmp	byte [rsi+1],0C0h
	cmovc	rax,rbx
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	jmp	rcx

process_opcode_ud_0FFF:
	mov	eax,'ud'
	stosd
	dec	rdi
	dec	rdi
	mov	eax,2-1
	ret

process_opcode_unknown_as_db_1_byte:
;	mov	al,'d'
;	stosb
;	mov	al,'b'
;	stosb
;	mov	al,' '
;	stosb
	mov	eax,'db '
	stosd
	dec	rdi
	lea	rbx,[table_transform_hexa]

	mov	edx,dword [disassemble_prefix]
	or	edx,edx
	jz	process_opcode_unknown_as_db_1_byte_prefixes_done
	xor	ecx,ecx
process_opcode_unknown_as_db_1_byte_prefixes:
	mov	cl,dl
	shr	ecx,4
	mov	al,[rbx+rcx]
	stosb
	mov	cl,dl
	and	cl,00001111b
	mov	al,[rbx+rcx]
	stosb
	mov	al,','
	stosb
	shr	edx,8
	jnz	process_opcode_unknown_as_db_1_byte_prefixes
process_opcode_unknown_as_db_1_byte_prefixes_done:

	xor	eax,eax
	mov	al,[rsi]
	shr	al,4
	mov	al,[rbx+rax*1]
	stosb
	lodsb
	and	al,00001111b
	mov	al,[rbx+rax*1]
	stosb
	mov	al,0
	stosb
	jmp	process_opcode_common_end_58

process_opcode_mm_instruction_universal:
	xor	ecx,ecx
	mov	edx,1
	call	search_disassemble_prefix_66
	cmovz	ecx,edx
	mov	edx,2
	call	search_disassemble_prefix_F3
	cmovz	ecx,edx
	mov	edx,3
	call	search_disassemble_prefix_F2
	cmovz	ecx,edx
	mov	rax,[rbx+rcx*8]
	mov	[rdi],rax
	bsr	rax,rax
	add	al,7
	shr	eax,3
	add	rdi,rax
	mov	al,' '
	stosb
	ret

align 16
process_Vo_Mo:
; cmp byte [rsi+1],0C0h
; jnc invalid
process_Vo_Wo:
	call	process_Vo
process_Vo_Wo_common_begin:
	mov	rdx,',oword '
	mov	[rdi],rdx
	mov	ecx,7
	mov	edx,1
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	add	rdi,rcx
	call	process_Wo
	ret

align 16
process_Wo_Vo:
	mov	rdx,'oword '
	mov	[rdi],rdx
	mov	ecx,6
	xor	edx,edx
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	add	rdi,rcx
	call	process_Wo
	mov	byte [rdi],','
	scasb
	call	process_Vo
	ret

align 16
process_Pq_Wo:
	call	process_Pq_destination
	jmp	process_Vo_Wo_common_begin

align 16
process_Pq_Wq:
	call	process_Pq_destination
	jmp	process_Vq_Wq_common_begin

align 16
process_Vo_Eq:
	call	process_Vo
	jmp	process_Pq_Eq_common_begin

align 16
process_Pq_Qq:
process_Pq_Qd:
	call	process_Pq_destination
	mov	byte [rdi],','
	scasb
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_Qq_subroutines]
	call	qword [rbx+rax*8]
	ret

align 16
process_Qq_Pq:
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_Qq_subroutines]
	call	qword [rbx+rax*8]
	mov	byte [rdi],','
	scasb
	push	rax				; thanx to Valy for finding bad opcode size calculation by process_Pq_destination destroing RAX
	call	process_Pq_destination		; !!! bad is process_Pq !!!
	pop	rax				; restore good opcode size returned by process_Qq_subroutines
	ret

align 16
process_Vd_Wq:
process_Vo_Mq:
process_Vo_Wq:
process_Vq_Mq:
process_Vq_Wq:
	call	process_Vq
process_Vq_Wq_common_begin:
	mov	rdx,',qword '
	mov	[rdi],rdx
	mov	ecx,7
	mov	edx,1
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	add	rdi,rcx
	call	process_Wq
	ret

align 16
process_Vo_Mq_Vo_Pq:
process_Vq_Mq_Vq_Pq:
	call	process_Vq
	mov	rdx,',qword '
	mov	[rdi],rdx
	mov	ecx,7
	mov	edx,1
	lea	rax,[process_Wq]
	lea	rbx,[process_Pq]
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	cmovnc	rax,rbx
	add	rdi,rcx
	call	rax
	ret

;align 16
process_Mo_Vo:
	mov	rdx,'oword '
	mov	[rdi],rdx
	mov	ecx,6
	xor	edx,edx
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	add	rdi,rcx
	call	process_Mq
	mov	byte [rdi],','
	scasb
	call	process_Vo
	ret

align 16
process_Mq_Pq:
	mov	rax,'qword '
	stosq
	dec	rdi
	dec	rdi
	call	process_Mq
	mov	byte [rdi],','
	scasb
	push	rax			; save opcode size, thanx to Valy for find it
	call	process_Pq_destination	; this call recalculate new opcode size that is bad
					; maybe call process_Pq should be used ... it destroys RAX too.
	pop	rax
	ret

;align 16
;process_Eq_Pq:
;	call	process_Vq
;	mov	byte [rdi],','
;	scasb
;	push	rax
;	call	process_opcode_xmm_reg
;	pop	rax
;	ret

align 16
process_Wo_Vq:
process_Mq_Vq:
process_Wq_Vq:
	mov	rdx,'qword '
process_Wq_Vq_common_begin:
	mov	[rdi],rdx
	mov	ecx,6
	xor	edx,edx
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	add	rdi,rcx

process_Mo_Vo_without_operand_size:
	call	process_Mq
	mov	byte [rdi],','
	scasb
	call	process_Vq
	ret

align 16
process_Vq_Wd:
process_Vd_Wd:
	call	process_Vd
	mov	rdx,',dword '
	mov	[rdi],rdx
	mov	ecx,7
	mov	edx,1
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	add	rdi,rcx
	call	process_Wd
	ret

align 16
process_Wd_Vd:
	mov	rdx,'dword '
	mov	[rdi],rdx
	mov	ecx,6
	xor	edx,edx
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	add	rdi,rcx
	call	process_Wd
	mov	byte [rdi],','
	scasb
	call	process_Vd
	ret

align 16
process_Vo_Ed:
process_Vq_Ed:
process_Vd_Ed:
	call	process_Vd
process_Vd_Ed_common_begin:
	mov	rdx,',dword '
	mov	[rdi],rdx
	mov	ecx,7
	mov	edx,1
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	add	rdi,rcx
;	call	process_Ed
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_Ed_table]
	call	qword [rbx+rax*8]
	ret

align 16
process_Pq_Ed:
	call	process_Pq_destination
	jmp	process_Vd_Ed_common_begin

align 16
process_Pq_Eq:
	call	process_Pq_destination
process_Pq_Eq_common_begin:
	mov	rdx,',qword '
	mov	[rdi],rdx
	mov	ecx,7
	mov	edx,1
	cmp	byte [rsi+1],0C0h
	cmovnc	ecx,edx
	add	rdi,rcx
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_Eq_table]
	call	qword [rbx+rax*8]
	ret

align 16
process_Eq_Vq:
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_Eq_table]
	call	qword [rbx+rax*8]
	mov	byte [rdi],','
	scasb
	call	process_Vq
	ret

align 16
process_Ed_Vd:
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_Ed_table]
	call	qword [rbx+rax*8]
	mov	byte [rdi],','
	scasb
	call	process_Vd
	ret

align 16
process_Ed_Pd:
; we should check if mod field of mod R/M byte is 11b, but...
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_Ed_table]
	call	qword [rbx+rax*8]
	mov	byte [rdi],','
	scasb
	push	rax			; found by me according to Valy's experiments. Valy thanx you !
	call	process_Pq_destination	; !!! bad is:	call	process_Pd !!!
	pop	rax
	ret

;align 16
;process_Gd_Wd:
;	call	process_Gd
;	mov	rdx,',dword '
;	mov	[rdi],rdx
;	mov	ecx,7
;	mov	edx,1
;	movzx	eax,byte [rsi+1]
;;	cmp	byte [rsi+1],0C0h
;	cmp	al,0C0h
;	cmovnc	ecx,edx
;	lea	rdi,[rdi+rcx*1]
;;	jnc	process_Wd_source
;;process_Gd_Wd_mem:
;;	movzx	eax,byte [rsi+1]
;	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
;;	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_d]
;	lea	rbx,[process_Wo_subroutines]
;	call	qword [rbx+rax*8]
;	ret
;
;align 16
;process_Gd_Wq:
;	call	process_Gd
;	mov	rdx,',qword '
;	mov	qword [rdi],rdx
;	mov	ecx,7
;	mov	edx,1
;	cmp	byte [rsi+1],0C0h
;	cmovnc	ecx,edx
;	lea	rdi,[rdi+rcx*1]
;;	jnc	process_Wq_source
;;process_Gd_Wq_mem:
;	movzx	eax,byte [rsi+1]
;	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
;;	lea	rbx,[process_opcode_big_fucking_gun_mem_memb_memd_reg_table_q]
;	lea	rbx,[process_Wo_subroutines]
;	call	qword [rbx+rax*8]
;	ret

align 16
process_Gd_Wd:
	call	process_Gd_Gq
	mov	rbx,',dword '
	jmp	process_Gd_Wx_common

align 16
process_Gd_Wq:
	call	process_Gd_Gq
	mov	rbx,',qword '
process_Gd_Wx_common:
	mov	[rdi],rbx
	mov	ecx,7
	mov	edx,1
	movzx	eax,byte [rsi+1]
	cmp	al,0C0h
	cmovnc	ecx,edx
	add	rdi,rcx
	shr	eax,6		; 00-3F -> 0   40-7F -> 1   80-BF -> 2   C0-FF -> 3
	lea	rbx,[process_Wo_subroutines]
	call	qword [rbx+rax*8]
	ret

align 16
process_Gd_VRo:
	call	process_Gd
	mov	byte [rdi],','
	scasb
	call	process_opcode_xmm_reg	; has to be only register, not memory, e.g. 0F5000 - 0F50BF illegal instructions
					; 0F50C0 - 0F50FF are real opcodes
	ret

align 16
process_Gd_PRq:
	call	process_Gd
	mov	byte [rdi],','
	scasb
	call	process_PRq
	ret

align 16
process_Pq_VRq:
	call	process_Pq_destination
	mov	byte [rdi],','
	scasb
	call	process_VRq
	ret

align 16
process_Vo_PRq:
	call	process_Vo_destination
	mov	byte [rdi],','
	scasb
	call	process_PRq
	ret

align 16
process_Pq_PRq:
	call	process_Pq_destination
	mov	byte [rdi],','
	scasb
	call	process_PRq
	ret

align 16
process_Vo_VRo:
	call	process_Vo
	mov	byte [rdi],','
	scasb
	call	process_VRo
	ret

align 16
process_Vo_destination:
process_Vo:
process_Vq:
process_Vd:
	push	rax
	mov	eax,'xmm0'
	stosd
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	call	search_disassemble_prefix_REX
	bt	edx,2
process_Vd_common_begin:
	setc	dl
	lea	eax,[rax+rdx*8]
	lea	rdx,[mm_transform]
	mov	ax,[rdx+rax*2]
	mov	[rdi-1],ax
	shr	eax,8
	setnz	al
	add	rdi,rax
	pop	rax
	mov	byte [rdi],0
	ret

process_Wq_source:
process_Wd_source:
process_Vq_source:
	push	rax
	mov	eax,'xmm0'
	stosd
	movzx	eax,byte [rsi+1]
	and	al,00000111b
	call	search_disassemble_prefix_REX
	bt	edx,0
	jmp	process_Vd_common_begin

align 16
process_Mq:
process_Wo:
process_Wq:
process_Wd:
	movzx	eax,byte [rsi+1]
	shr	eax,6
	lea	rbx,[process_Wo_subroutines]
	call	qword [rbx+rax*8]
	ret

align 16
process_VRq:
process_VRo:
process_opcode_xmm_reg:
	mov	eax,'xmm0'
	stosd
process_opcode_xmm_reg_common_begin:
	movzx	eax,byte [rsi+1]
	and	al,00000111b
	call	search_disassemble_prefix_REX
	bt	edx,0
process_opcode_xmm_reg_common_begin_2:
	setc	dl
	lea	eax,[rax+rdx*8]
	lea	rdx,[mm_transform]
	mov	ax,[rdx+rax*2]
	mov	[rdi-1],ax
	shr	eax,8
	setnz	al
	add	rdi,rax
	mov	byte [rdi],0
	mov	eax,2	; opcode size
	ret

align 16
process_Pq_destination:
	mov	eax,'mm0'
	stosd
	dec	rdi
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	call	search_disassemble_prefix_REX
	bt	edx,2
	jmp	process_opcode_xmm_reg_common_begin_2

align 16
process_Pd:
process_PRq:
process_Pq:
	mov	eax,'mm0'
	stosd
	dec	rdi
	jmp	process_opcode_xmm_reg_common_begin

align 16
process_opcode_reg_32:
	movzx	eax,byte [rsi+1]
	and	al,00000111b
	lea	rbx,[gpr_32]
	call	search_disassemble_prefix_REX
	bt	edx,0
process_opcode_reg_32_common_begin:
	setc	dl
	shl	edx,3+2
	add	rbx,rdx
	mov	eax,[rbx+rax*4]
	stosd
;	xor	eax,eax
;	cmp	byte [rdi-1],0
;	setz	al
;	sub	rdi,rax
	shr	eax,8*3		; we destroy low 3 bytes and search if higher byte is 0
	setnz	al
	lea	rdi,[rdi+rax*1-1]
	mov	byte [rdi],0
	mov	eax,2
	ret

process_opcode_reg_64:
	movzx	eax,byte [rsi+1]
	and	al,00000111b
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3
	cmovc	rbx,rcx
	bt	edx,0
	jmp	process_Gd_Gq_common

align 16
process_Gd_Gq:
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	lea	rbx,[gpr_32]
	lea	rcx,[gpr_64]
	call	search_disassemble_prefix_REX
	bt	edx,3
	cmovc	rbx,rcx
	bt	edx,2
process_Gd_Gq_common:
	setc	dl
	shl	edx,3+2
	add	rbx,rdx
	mov	eax,[rbx+rax*4]
	stosd
	shr	eax,8*2		; we destroy low 2 bytes and search if higher word is 0
	setz	cl
	shr	eax,8		; we destroy 3rd byte of eax
	setz	al
	add	al,cl
	sub	rdi,rax
	mov	byte [rdi],0
	mov	eax,2
	ret

align 16
process_Gd:
	movzx	eax,byte [rsi+1]
	and	al,00111000b
	shr	eax,3
	lea	rbx,[gpr_32]
	call	search_disassemble_prefix_REX
	bt	edx,2
	jmp	process_opcode_reg_32_common_begin

;process_nothing:
;	or	rax,-1		; opcode size -1 +1 (opcode 0F) = 0 so we procede only prefix 0F3 or 0F2
;	ret

align 16
process_lb_binary:
; destroy CL (RCX) register
	push	rax		; save opcode size
	mov	cl,[rsi+rax*1]
	mov	al,','
	stosb
	   repeat 8
	xor	eax,eax
	shl	cl,1
	adc	al,'0'
	stosb
	   end repeat
	mov	al,'b'
	stosb
	mov	byte [rdi],0
	pop	rax		; restore opcode size
	inc	eax		; this routine process 1 byte of opcode
	ret

process_lb_hexa:
	push	rax		; save opcode size
	mov	byte [rdi],','
	scasb
	mov	al,[rsi+rax*1]
	shl	rax,32+16+8
	call	hexa_64_rax
	scasw
	mov	byte [rdi],0
	pop	rax		; restore opcode size
	inc	eax		; this routine process 1 byte of opcode
	ret
